@@ -7,8 +7,8 @@ do 'acl-lib.pl';
 # Converts logged information from this module into human-readable form
 sub parse_webmin_log
 {
-local ($user, $script, $action, $type, $object, $p) = @_;
-local $g = $type eq 'group' ? "_g" : "";
+my ($user, $script, $action, $type, $object, $p) = @_;
+my $g = $type eq 'group' ? "_g" : "";
 if ($action eq 'modify') {
 	if ($p->{'old'} ne $p->{'name'}) {
 		return &text('log_rename'.$g, "<tt>$p->{'old'}</tt>",
@@ -30,7 +30,12 @@ elsif ($action eq 'create') {
 		}
 	}
 elsif ($action eq 'delete') {
-	return &text('log_delete'.$g, "<tt>$object</tt>");
+	if ($type eq "users" || $type eq "groups") {
+		return &text('log_delete_'.$type, $object);
+		}
+	else {
+		return &text('log_delete'.$g, "<tt>$object</tt>");
+		}
 	}
 elsif ($action eq 'acl') {
 	return &text('log_acl', "<tt>$object</tt>",
@@ -47,7 +52,7 @@ elsif ($action eq 'switch') {
 	return &text('log_switch', "<tt>".&html_escape($object)."</tt>");
 	}
 else {
-	return undef;
+	return $text{'log_'.$action};
 	}
 }
 
