@@ -5,25 +5,23 @@ require 'acl-lib.pl';
 sub module_install
 {
 # Fix up .acl files
-local @mods = &get_all_module_infos();
-local %isuser = map { $_->{'name'}, 1 } &list_users();
-local $g;
-foreach $g (&list_groups()) {
+my @mods = &get_all_module_infos();
+my %isuser = map { $_->{'name'}, 1 } &list_users();
+foreach my $g (&list_groups()) {
 	next if ($isuser{$g->{'name'}});
-	local $m;
-	foreach $m (@mods) {
+	next if ($g->{'proto'});
+	foreach my $m (@mods) {
 		if (-r "$config_directory/$m->{'dir'}/$g->{'name'}.acl") {
-			print STDERR "renaming $config_directory/$m->{'dir'}/$g->{'name'}.acl $config_directory/$m->{'dir'}/$g->{'name'}.gacl\n";
 			rename("$config_directory/$m->{'dir'}/$g->{'name'}.acl",
-			       "$config_directory/$m->{'dir'}/$g->{'name'}.gacl");
+			     "$config_directory/$m->{'dir'}/$g->{'name'}.gacl");
 			}
 		}
 	}
 
 # Update sub-groups in webmin.groups file to use @ names
-foreach $g (&list_groups()) {
-	local ($u, @newmembers, $any);
-	foreach $u (@{$g->{'members'}}) {
+foreach my $g (&list_groups()) {
+	my (@newmembers, $any);
+	foreach my $u (@{$g->{'members'}}) {
 		if ($u !~ /^\@/ && !$isuser{$u}) {
 			push(@newmembers, '@'.$u);
 			$any = 1;
@@ -34,7 +32,6 @@ foreach $g (&list_groups()) {
 		}
 	$g->{'members'} = \@newmembers;
 	if ($any) {
-		print STDERR "Changing members of $g->{'name'} to ",join(" ", @newmembers),"\n";
 		&modify_group($g->{'name'}, $g);
 		}
 	}
