@@ -24,6 +24,9 @@
 #		15- Perl
 #		16- PHP
 #		17- Vhost aliases
+#		18- Filters
+#		19- Character Sets
+#		20- Image maps
 #  multiple -	Can this directive appear multiple times
 #  global -	Can be used in the global server context
 #  virtual -	Can be used in a VirtualHost section or in the global section
@@ -33,53 +36,62 @@ sub core_directives
 {
 local($rv);
 $rv = [	[ 'AccessFileName', 0, 5, 'virtual', undef, 5 ],
-	[ 'ClearModuleList AddModule', 1, 2, 'global' ],
+	[ 'AddDefaultCharset', 0, 19, 'virtual directory htaccess', 2.0 ],
+	&can_configure_apache_modules() ? ( ) :
+		( [ 'ClearModuleList AddModule', 1, 2, 'global', -2.0 ] ),
 	[ 'AllowOverride', 0, 5, 'directory' ],
 	[ 'AuthName', 0, 4, 'directory htaccess', undef, 10 ],
 	[ 'AuthType', 0, 4, 'directory htaccess', undef, 8 ],
-	[ 'BindAddress Listen Port', 1, 1, 'global' ],
+	[ 'BindAddress Listen Port', 1, 1, 'global', -2.0, 10 ],
 	[ 'ContentDigest', 0, 5, 'virtual directory htaccess' ],
-	[ 'CoreDumpDirectory', 0, 9, 'global', 1.3 ],
+	[ 'CoreDumpDirectory', 0, 9, 'global', '1.3-2.0' ],
 	[ 'DefaultType', 0, 6, 'virtual directory htaccess' ],
 	[ 'DocumentRoot', 0, 5, 'virtual', undef, 10 ],
 	[ 'ErrorDocument', 1, 7, 'virtual directory htaccess' ],
 	[ 'ErrorLog', 0, 3, 'virtual' ],
-	[ 'Group', 0, 8, 'virtual' ],
-	[ 'HostNameLookups', 0, 1, 'virtual directory htaccess' ],
-	[ 'IdentityCheck', 0, 1, 'virtual directory htaccess' ],
+	[ 'FileETag', 0, 5, 'virtual directory htaccess', 2.0 ],
+	[ 'ForceType', 0, 6, 'directory htaccess', 2.0 ],
+	[ 'Group', 0, 8, 'virtual', -2.0 ],
+	[ 'HostNameLookups', 0, 1, 'virtual directory' ],
+	[ 'IdentityCheck', 0, 1, 'virtual directory' ],
 	[ 'KeepAlive MaxKeepAliveRequests', 0, 1, 'global' ],
 	[ 'KeepAliveTimeout', 0, 1, 'global' ],
-	[ 'ListenBacklog', 0, 1, 'global', 1.2 ],
-	[ 'LockFile', 0, 9, 'global' ],
+	[ 'ListenBacklog', 0, 1, 'global', '1.2-2.0' ],
+	[ 'LockFile', 0, 9, 'global', -2.0 ],
 	[ 'LimitRequestBody', 0, 0, 'virtual directory htaccess', 1.302 ],
 	[ 'LimitRequestFields', 0, 0, 'global', 1.302 ],
 	[ 'LimitRequestFieldsize', 0, 0, 'global', 1.302 ],
 	[ 'LimitRequestLine', 0, 0, 'global', 1.302 ],
-	[ 'MaxClients', 0, 0, 'global' ],
-	[ 'MaxRequestsPerChild', 0, 0, 'global' ],
-	[ 'StartServers', 0, 0, 'global' ],
-	[ 'MinSpareServers', 0, 0, 'global' ],
-	[ 'MaxSpareServers', 0, 0, 'global' ],
-	[ 'NameVirtualHost', 1, 1, 'global', 1.3 ],
+	[ 'LimitXMLRequestBody', 0, 0, 'virtual directory htaccess', 2.0 ],
+	[ 'LogLevel', 0, 3, 'virtual', 1.3 ],
+	[ 'MaxClients', 0, 0, 'global', -2.0 ],
+	[ 'MaxRequestsPerChild', 0, 0, 'global', -2.0 ],
+	[ 'StartServers', 0, 0, 'global', -2.0 ],
+	[ 'MinSpareServers', 0, 0, 'global', -2.0 ],
+	[ 'MaxSpareServers', 0, 0, 'global', -2.0 ],
+	[ 'NameVirtualHost', 1, 1, 'global', 1.3, 5 ],
 	[ 'Options', 0, 5, 'virtual directory htaccess', undef, 3 ],
-	[ 'PidFile', 0, 9, 'global' ],
+	[ 'PidFile', 0, 9, 'global', -2.0 ],
 	[ 'require', 0, 4, 'directory htaccess', undef, 6 ],
 	[ 'RLimitCPU', 0, 0, 'virtual', 1.2 ],
 	[ 'RLimitMEM', 0, 0, 'virtual', 1.2 ],
 	[ 'RLimitNPROC', 0, 0, 'virtual', 1.2 ],
 	[ 'Satisfy', 0, 4, 'directory htaccess', 1.2, 4 ],
-	[ 'ScoreBoardFile', 0, 9, 'global', 1.2 ],
-	[ 'SendBufferSize', 0, 1, 'global' ],
+	[ 'ScoreBoardFile', 0, 9, 'global', '1.2-2.0' ],
+	[ 'SendBufferSize', 0, 1, 'global', -2.0 ],
 	[ 'ServerAdmin', 0, 1, 'virtual' ],
-	[ 'ServerAlias', 1, 1, 'virtual virtualonly', 1.2 ],
-	[ 'ServerName', 0, 1, 'virtual' ],
+	$access{'names'} ? (
+		[ 'ServerAlias', 1, 1, 'virtual virtualonly', 1.2 ],
+		[ 'ServerName', 0, 1, 'virtual' ] ) : ( ),
 	[ 'ServerPath', 0, 5, 'virtual' ],
-	[ 'ServerType', 0, 9, 'global' ],
+	[ 'ServerType', 0, 9, 'global', -2.0 ],
 	[ 'ServerTokens', 0, 9, 'global', 1.3 ],
 	[ 'ServerSignature', 0, 5, 'virtual directory htaccess', 1.3 ],
+	[ 'SetOutputFilter', 0, 18, 'virtual directory htaccess', 2.0 ],
+	[ 'SetInputFilter', 0, 18, 'virtual directory htaccess', 2.0 ],
 	[ 'TimeOut', 0, 1, 'global' ],
-	[ 'UseCanonicalName', 0, 1, 'virtual directory htaccess', 1.3 ],
-	[ 'User', 0, 8, 'virtual', undef, 10 ] ];
+	[ 'UseCanonicalName', 0, 1, 'virtual directory', 1.3 ],
+	[ 'User', 0, 8, 'virtual', -2.0, 10 ] ];
 return &make_directives($rv, $_[0], "core");
 }
 
@@ -239,13 +251,24 @@ $bref = $_[0]; $lref = $_[1]; $pref = $_[2];
 if (@$lref) {
 	# listen directives in use.. so BindAddress and Port are unused
 	foreach $l (@$lref) {
-		if ($l->{'value'} =~ /^(\S+):(\d+)$/) {
+		if ($l->{'value'} =~ /^\[(\S+)\]:(\d+)$/) {
+			# IPv6 address and port
+			push(@blist, $1); push(@plist, $2);
+			}
+		elsif ($l->{'value'} =~ /^\[(\S+)\]$/) {
+			# IPv6 address only
+			push(@blist, $1); push(@plist, undef);
+			}
+		elsif ($l->{'value'} =~ /^(\S+):(\d+)$/) {
+			# IPv4 address and port
 			push(@blist, $1); push(@plist, $2);
 			}
 		elsif ($l->{'value'} =~ /^(\d+)$/) {
+			# Port only
 			push(@blist, "*"); push(@plist, $1);
 			}
 		elsif ($l->{'value'} =~ /^(\S+)$/) {
+			# IPv4 address or hostname only
 			push(@blist, $1); push(@plist, undef);
 			}
 		}
@@ -257,7 +280,9 @@ else {
 	push(@plist, undef);
 	}
 $port = @$pref ? $pref->[@$pref-1]->{'value'} : 80;
-$inp = "<b>$text{'core_dport'}</b> <input name=Port size=6 value=\"$port\"><br>\n";
+if ($_[3]->{'version'} < 2.0) {
+	$inp = "<b>$text{'core_dport'}</b> <input name=Port size=6 value=\"$port\"><br>\n";
+	}
 $inp .=	"<table border>\n".
 	"<tr $tb> <td><b>$text{'core_address'}</b></td> <td><b>$text{'core_port'}</b></td> </tr>\n";
 for($i=0; $i<@blist+1; $i++) {
@@ -270,7 +295,12 @@ for($i=0; $i<@blist+1; $i++) {
 		$blist[$i] eq "*" ? "checked" : "",
 		$blist[$i] && $blist[$i] ne "*" ? "checked" : "",
 		$blist[$i] eq "*" ? "" : $blist[$i];
-	$inp .= "<td>".&opt_input($plist[$i], "Port_$i", "$text{'core_default'}", 5)."</td>";
+	if ($_[3]->{'version'} < 2.0) {
+		$inp .= "<td>".&opt_input($plist[$i], "Port_$i", "$text{'core_default'}", 5)."</td>";
+		}
+	else {
+		$inp .= "<td><input name=Port_$i size=5 value='$plist[$i]'></td>\n";
+		}
 	}
 $inp .= "</table>\n";
 return (2, $text{'core_listen'}, $inp);
@@ -286,7 +316,8 @@ for($i=0; defined($in{"Port_$i"}); $i++) {
 	if ($bdef == 2) { next; }
 
 	if ($bdef) { push(@blist, "*"); }
-	elsif ($b =~ /^\S+$/ && gethostbyname($b)) { push(@blist, $b); }
+	elsif ($b =~ /^\S+$/ &&
+	       (&to_ipaddress($b) || &to_ip6address($b))) { push(@blist, $b); }
 	else { &error(&text('core_eaddress', $b)); }
 
 	if ($pdef) { push(@plist, undef); }
@@ -296,22 +327,40 @@ for($i=0; defined($in{"Port_$i"}); $i++) {
 if (!@blist) { &error($text{'core_eoneaddr'}); }
 
 # Return directives
-$in{'Port'} =~ /^\d+$/ || &error($text{'core_edefport'});
-if (@blist == 1 && !$plist[0]) {
-	# Only one address, and the default port
-	return ( $blist[0] eq "*" ? [ ] : [ $blist[0] ], [ ], [ $in{'Port'} ] );
+if ($_[0]->{'version'} < 2.0) {
+	# Older apaches have a port directive as well
+	$in{'Port'} =~ /^\d+$/ || &error($text{'core_edefport'});
+	if (@blist == 1 && !$plist[0]) {
+		# Only one address, and the default port
+		return ( $blist[0] eq "*" ? [ ] : [ $blist[0] ], [ ],
+			 [ $in{'Port'} ] );
+		}
+	else {
+		# More than one address, or a non-default port. Must use Listens
+		for($i=0; $i<@blist; $i++) {
+			if ($blist[$i] ne "*" && $plist[$i]) {
+				push(@l, "$blist[$i]:$plist[$i]");
+				}
+			elsif ($blist[$i] ne "*") { push(@l, $blist[$i]); }
+			elsif ($plist[$i]) { push(@l, "*:$plist[$i]"); }
+			else { push(@l, $in{'Port'}); }
+			}
+		return ( [], \@l, [ $in{'Port'} ] );
+		}
 	}
 else {
-	# More than one address, or a non-default port. Must use Listens
+	# Apache 2.0 just uses Listen directives
 	for($i=0; $i<@blist; $i++) {
+		if (&check_ip6address($blist[$i])) {
+			$blist[$i] = "[".$blist[$i]."]";
+			}
 		if ($blist[$i] ne "*" && $plist[$i]) {
 			push(@l, "$blist[$i]:$plist[$i]");
 			}
 		elsif ($blist[$i] ne "*") { push(@l, $blist[$i]); }
-		elsif ($plist[$i]) { push(@l, "*:$plist[$i]"); }
-		else { push(@l, $in{'Port'}); }
+		else { push(@l, "*:$plist[$i]"); }
 		}
-	return ( [], \@l, [ $in{'Port'} ] );
+	return ( [], \@l );
 	}
 }
 
@@ -476,8 +525,18 @@ sub save_ServerAlias
 {
 local(@al);
 @al = split(/\s+/, $in{'ServerAlias'});
-if (@al) { return ( [ join(' ', @al) ] ) }
-else { return ( [ ] ); }
+if (@al) {
+	local @spal;
+	while(@al > 200) {
+		push(@spal, join(" ", @al[0 .. 199]));
+		@al = @al[200 .. $#al];
+		}
+	push(@spal, join(" ", @al)) if (@al);
+	return ( \@spal );
+	}
+else {
+	return ( [ ] );
+	}
 }
 
 sub edit_ServerName
@@ -492,9 +551,19 @@ return &parse_opt("ServerName", '^\S+$', $text{'core_ehostname'});
 
 sub edit_NameVirtualHost
 {
-local(@nv, $nv);
-foreach $nv (@{$_[0]}) { push(@nv, $nv->{'value'}); }
+local(@nv, $nv, $star);
+foreach $nv (@{$_[0]}) {
+	if ($nv->{'value'} eq "*" && $_[1]->{'version'} >= 1.312) { $star++; }
+	elsif ($nv->{'value'} =~ /^\[(\S+)\]$/) { push(@nv, $1); }
+	else { push(@nv, $nv->{'value'}); }
+	}
+if ($_[1]->{'version'} >= 1.312) {
+	$starui = sprintf
+	  "<input type=checkbox name=NameVirtualHost_star value=1 %s> %s<br>\n",
+	  $star ? "checked" : "", $text{'core_virtaddr_star'};
+	}
 return (1, $text{'core_virtaddr'},
+	$starui.
 	"<textarea name=NameVirtualHost rows=3 cols=30>".
 	join("\n", @nv)."</textarea>");
 }
@@ -502,12 +571,20 @@ sub save_NameVirtualHost
 {
 local(@nv, $nv, $addr);
 @nv = split(/\s+/, $in{'NameVirtualHost'});
+@nv = ( "*", @nv ) if ($in{'NameVirtualHost_star'});
 foreach $nv (@nv) {
 	if ($nv =~ /^(\S+):(\d+|\*)$/) { $addr = $1; }
 	else { $addr = $nv; }
-	if (!gethostbyname($addr) && $addr ne '*') {
+	if (!&to_ipaddress($addr) &&
+	    !&to_ip6address($addr) && $addr ne '*') {
 		&error(&text('core_evirtaddr', $addr));
 		}
+	if ($nv =~ /^(\S+):(\d+|\*)$/ && &check_ip6address($1)) {
+		$nv = "[$1]:$2";
+		}
+	elsif (&check_ip6address($nv)) {
+		$nv = "[$nv]";
+		}
 	}
 if (@nv) { return ( \@nv ); }
 else { return ( [ ] ); }
@@ -590,8 +667,12 @@ return (2, $text{'core_docroot'},
 }
 sub save_DocumentRoot
 {
-$in{'DocumentRoot_def'} || -d $in{'DocumentRoot'} ||
-	&error(&text('core_enodoc', $in{'DocumentRoot'}));
+if (!$in{'DocumentRoot_def'}) {
+	-d $in{'DocumentRoot'} ||
+		&error(&text('core_enodoc', $in{'DocumentRoot'}));
+	&allowed_doc_dir($in{'DocumentRoot'}) ||
+		&error(&text('core_ecandoc', $in{'DocumentRoot'}));
+	}
 return &parse_opt("DocumentRoot");
 }
 
@@ -669,13 +750,39 @@ return &parse_opt("ServerPath", '^\/\S*$',
 sub edit_ServerSignature
 {
 return (1, $text{'core_footer'},
-	&choice_input($_[0]->{'value'}, "ServerSignature", undef,
+	&select_input($_[0]->{'value'}, "ServerSignature", undef,
 		      "$text{'core_sigemail'},Email", "$text{'core_signame'},On",
 		      "$text{'core_signone'},Off", "$text{'core_default'},"));
 }
 sub save_ServerSignature
 {
-return &parse_choice("ServerSignature", undef);
+return &parse_select("ServerSignature", undef);
+}
+
+sub edit_FileETag
+{
+local (%et, $rv);
+map { $et{lc($_)}++ } @{$_[0]->{'words'}} if ($_[0]);
+$rv .= sprintf "<input type=radio name=FileETag_def value=1 %s> %s\n",
+			$_[0] ? "" : "checked", $text{'default'};
+$rv .= sprintf "<input type=radio name=FileETag_def value=0 %s> %s\n",
+			$_[0] ? "checked" : "", $text{'core_fileetag_sel'};
+foreach $e ('INode', 'MTime', 'Size') {
+	$rv .= sprintf "<input type=checkbox name=FileETag value=%s %s> %s\n",
+			$e, $et{lc($e)} || $et{'all'} ? "checked" : "",
+			$text{'core_fileetag_'.lc($e)};
+	}
+return (2, $text{'core_fileetag'}, $rv);
+}
+sub save_FileETag
+{
+if ($in{'FileETag_def'}) {
+	return ( [ ] );
+	}
+else {
+	local @e = split(/\0/, $in{'FileETag'});
+	return @e ? ( [ join(" ", @e) ] ) : ( [ "None" ] );
+	}
 }
 
 #########################################################################
@@ -692,6 +799,67 @@ return &parse_opt("DefaultType", '^(\S+)\/(\S+)$',
 		  $text{'core_edefmime'});
 }
 
+sub edit_ForceType
+{
+return (1, $text{'mod_mime_defmime'},
+	&opt_input($_[0]->{'value'}, "ForceType", $text{'mod_mime_real'}, 15));
+}
+sub save_ForceType
+{
+return &parse_opt("ForceType", '^\S+\/\S+$', $text{'mod_mime_etype'});
+}
+
+sub edit_SetOutputFilter
+{
+local @vals = split(/[\s;]+/, $_[0]->{'value'});
+return (2, $text{'core_outfilter'},
+	&filters_input(\@vals, "SetOutputFilter"));
+}
+sub save_SetOutputFilter
+{
+return &parse_filters("SetOutputFilter");
+}
+
+sub edit_SetInputFilter
+{
+local @vals = split(/[\s;]+/, $_[0]->{'value'});
+return (2, $text{'core_infilter'},
+	&filters_input(\@vals, "SetInputFilter"));
+}
+sub save_SetInputFilter
+{
+return &parse_filters("SetInputFilter");
+}
+
+sub edit_AddDefaultCharset
+{
+local $rv;
+local $m = lc($_[0]->{'value'}) eq 'off' ? 2 :
+	   $_[0]->{'value'} ? 0 : 1;
+$rv .= sprintf "<input type=radio name=AddDefaultCharset_def value=1 %s> %s\n",
+		$m == 1 ? "checked" : "", $text{'default'};
+$rv .= sprintf "<input type=radio name=AddDefaultCharset_def value=2 %s> %s\n",
+		$m == 2 ? 'checked' : "", $text{'core_none'};
+$rv .= sprintf "<input type=radio name=AddDefaultCharset_def value=0 %s>\n",
+		$m == 0 ? "checked" : "";
+$rv .= sprintf "<input name=AddDefaultCharset size=12 value='%s'>\n",
+		$m == 0 ? $_[0]->{'value'} : "";
+return (1, $text{'core_defchar'}, $rv);
+}
+sub save_AddDefaultCharset
+{
+if ($in{'AddDefaultCharset_def'} == 1) {
+	return ( [ ] );
+	}
+elsif ($in{'AddDefaultCharset_def'} == 2) {
+	return ( [ "Off" ] );
+	}
+else {
+	$in{'AddDefaultCharset'} =~ /^\S+$/ || &error($text{'core_edefchar'});
+	return ( [ $in{'AddDefaultCharset'} ] );
+	}
+}
+
 #########################################################################
 # Access control directives
 sub edit_AuthName
@@ -728,25 +896,46 @@ else { return ( [ ] ); }
 sub edit_require
 {
 local($rv, $mode, $list);
-$_[0]->{'value'} =~ /^(\S+)\s*(.*)$/; $mode = $1; $list = $2;
+local @w = @{$_[0]->{'words'}};
+$mode = shift(@w);
+$list = join(" ", map { $_ =~ /\s/ ? "\"$_\"" : $_ } @w);
+
+# All users
 $rv = sprintf
       "<input type=radio name=require_mode value=0 %s> $text{'default'}<br>\n",
       $mode ? "" : "checked";
+
+# Only some users
 $rv .= sprintf
       "<input type=radio name=require_mode value=1 %s> $text{'core_users'}:\n",
       $mode eq "user" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_user size=20 value=\"%s\"><br>\n",
-      $mode eq "user" ? $list : "";
+      $mode eq "user" ? &html_escape($list) : "";
+
+# Only members of groups
 $rv .= sprintf
       "<input type=radio name=require_mode value=2 %s> $text{'core_groups'}:\n",
       $mode eq "group" ? "checked" : "";
 $rv .= sprintf
       "<input name=require_group size=20 value=\"%s\"><br>\n",
-      $mode eq "group" ? $list : "";
+      $mode eq "group" ? &html_escape($list) : "";
+
+# All users
 $rv .= sprintf
-      "<input type=radio name=require_mode value=3 %s> $text{'core_allusers'}:<br>\n",
+      "<input type=radio name=require_mode value=3 %s> $text{'core_allusers'}<br>\n",
       $mode eq "valid-user" ? "checked" : "";
+
+if ($httpd_modules{'mod_authz_owner'} >= 2.2) {
+	# File owner / group matches
+	$rv .= sprintf
+	      "<input type=radio name=require_mode value=4 %s> $text{'core_fileowner'}<br>\n",
+	      $mode eq "file-owner" ? "checked" : "";
+	$rv .= sprintf
+	      "<input type=radio name=require_mode value=5 %s> $text{'core_filegroup'}<br>\n",
+	      $mode eq "file-group" ? "checked" : "";
+
+	}
 return (1, $text{'core_authlog'}, $rv);
 }
 sub save_require
@@ -754,7 +943,10 @@ sub save_require
 if ($in{'require_mode'} == 0) { return ( [ ] ); }
 elsif ($in{'require_mode'} == 1) { return ( [ "user $in{'require_user'}" ] ); }
 elsif ($in{'require_mode'} == 2) { return ( [ "group $in{'require_group'}" ] ); }
-else { return ( [ "valid-user" ] ); }
+elsif ($in{'require_mode'} == 3) { return ( [ "valid-user" ] ); }
+elsif ($in{'require_mode'} == 4) { return ( [ "file-owner" ] ); }
+elsif ($in{'require_mode'} == 5) { return ( [ "file-group" ] ); }
+else { return ( [ ] ); }	# huh?
 }
 
 sub edit_Satisfy
@@ -773,7 +965,7 @@ return &parse_choice("Satisfy", "");
 sub edit_CoreDumpDirectory
 {
 return (1, $text{'core_coredir'},
-	 &opt_input($_[0]->{'value'}, "CoreDumpDirectory", $text{'core_sroot'}, 20).
+	 &opt_input($_[0]->{'words'}->[0], "CoreDumpDirectory", $text{'core_sroot'}, 20).
 	 &file_chooser_button("CoreDumpDirectory", 1));
 }
 sub save_CoreDumpDirectory
@@ -784,7 +976,7 @@ return &parse_opt("CoreDumpDirectory", '^\S+$', $text{'core_ecore'});
 sub edit_LockFile
 {
 return (1, $text{'core_lockfile'},
-        &opt_input($_[0]->{'value'}, "LockFile", $text{'core_default'}, 20).
+        &opt_input($_[0]->{'words'}->[0], "LockFile", $text{'core_default'}, 20).
         &file_chooser_button("LockFile", 0));
 }
 sub save_LockFile
@@ -803,6 +995,19 @@ sub save_LimitRequestBody
 return &parse_opt("LimitRequestBody", '^\d+$', $text{'core_ebody'});
 }
 
+sub edit_LimitXMLRequestBody
+{
+return (1, $text{'core_maxxml'},
+	&opt_input($_[0]->{'value'}, "LimitXMLRequestBody",
+		   $text{'core_default'}, 8).$text{'bytes'});
+}
+sub save_LimitXMLRequestBody
+{
+return &parse_opt("LimitXMLRequestBody", '^\d+$', $text{'core_exml'});
+}
+
+
+
 sub edit_LimitRequestFields
 {
 return (1, $text{'core_maxhead'},
@@ -836,7 +1041,7 @@ return &parse_opt("LimitRequestLine", '^\d+$', $text{'core_eline'});
 sub edit_PidFile
 {
 return (1, $text{'core_pid'},
-        &opt_input($_[0]->{'value'}, "PidFile", $text{'core_default'}, 20).
+        &opt_input($_[0]->{'words'}->[0], "PidFile", $text{'core_default'}, 20).
         &file_chooser_button("PidFile", 0));
 }
 sub save_PidFile
@@ -847,7 +1052,7 @@ return &parse_opt("PidFile", '^\S+$', $text{'core_epid'});
 sub edit_ScoreBoardFile
 {
 return (1, $text{'core_memsco'},
-        &opt_input($_[0]->{'value'}, "ScoreBoardFile", $text{'core_default'}, 20).
+        &opt_input($_[0]->{'words'}->[0], "ScoreBoardFile", $text{'core_default'}, 20).
         &file_chooser_button("ScoreBoardFile", 0));
 }
 sub save_ScoreBoardFile
@@ -870,12 +1075,18 @@ sub edit_ServerTokens
 {
 local $v = $_[0]->{'value'};
 $v = "ProductOnly" if ($v eq "Prod");
+$v = "Min" if ($v eq "Minimal");
 return (1, $text{'core_header'},
 	&select_input($v, "ServerTokens", "Full",
 		      "$text{'core_verosmod'},Full",
-		      "$text{'core_veros'},OS", "$text{'core_ver'},Min",
+		      "$text{'core_veros'},OS",
+		      "$text{'core_ver'},Min",
+		      "$text{'core_minor'},Minor",
 		      $_[1]->{'version'} >= 1.313 ?
-			("$text{'core_product'},ProductOnly") : ()));
+			("$text{'core_product'},ProductOnly") : (),
+		      $_[1]->{'version'} >= 2.041 ?
+			("$text{'core_major'},Major") : ()
+			));
 }
 sub save_ServerTokens
 {
@@ -890,14 +1101,14 @@ local($rv, @ginfo);
 $rv = sprintf "<input type=radio name=Group value=0 %s>$text{'core_default'}&nbsp;\n",
        $_[0] ? "" : "checked";
 $rv .= sprintf "<input type=radio name=Group value=1 %s>$text{'core_group'}\n",
-        $_[0] && $_[0]->{'value'} !~ /^#/ ? "checked" : "";
+        $_[0] && $_[0]->{'words'}->[0] !~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=Group_name size=8 value=\"%s\"> %s&nbsp;\n",
-	$_[0]->{'value'} !~ /^#/ ? $_[0]->{'value'} : "",
+	$_[0]->{'words'}->[0] !~ /^#/ ? $_[0]->{'words'}->[0] : "",
 	&group_chooser_button("Group_name", 0);
 $rv .= sprintf "<input type=radio name=Group value=2 %s>$text{'core_gid'}\n",
-        $_[0]->{'value'} =~ /^#/ ? "checked" : "";
+        $_[0]->{'words'}->[0] =~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=Group_id size=6 value=\"%s\">\n",
-	 $_[0]->{'value'} =~ /^#(.*)$/ ? $1 : "";
+	 $_[0]->{'words'}->[0] =~ /^#(.*)$/ ? $1 : "";
 return (2, $text{'core_asgroup'}, $rv);
 }
 sub save_Group
@@ -907,7 +1118,7 @@ elsif ($in{'Group'} == 1) { return ( [ $in{'Group_name'} ] ); }
 elsif ($in{'Group_id'} !~ /^\-?\d+$/) {
 	&error(&text('core_euid', $in{'Group_id'}));
 	}
-else { return ( [ "#$in{'Group_id'}" ] ); }
+else { return ( [ "\"#$in{'Group_id'}\"" ] ); }
 }
 
 sub edit_User
@@ -916,14 +1127,14 @@ local($rv, @uinfo);
 $rv = sprintf "<input type=radio name=User value=0 %s>$text{'core_default'}&nbsp;\n",
        $_[0] ? "" : "checked";
 $rv .= sprintf "<input type=radio name=User value=1 %s>$text{'core_user'}\n",
-        $_[0] && $_[0]->{'value'} !~ /^#/ ? "checked" : "";
+        $_[0] && $_[0]->{'words'}->[0] !~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=User_name size=8 value=\"%s\"> %s&nbsp;\n",
-	$_[0]->{'value'} !~ /^#/ ? $_[0]->{'value'} : "",
+	$_[0]->{'words'}->[0] !~ /^#/ ? $_[0]->{'words'}->[0] : "",
 	&user_chooser_button("User_name", 0);
 $rv .= sprintf "<input type=radio name=User value=2 %s>$text{'core_uid'}\n",
-        $_[0]->{'value'} =~ /^#/ ? "checked" : "";
+        $_[0]->{'words'}->[0] =~ /^#/ ? "checked" : "";
 $rv .= sprintf "<input name=User_id size=6 value=\"%s\">\n",
-	 $_[0]->{'value'} =~ /^#(.*)$/ ? $1 : "";
+	 $_[0]->{'words'}->[0] =~ /^#(.*)$/ ? $1 : "";
 return (2, $text{'core_asuser'}, $rv);
 }
 sub save_User
@@ -933,7 +1144,7 @@ elsif ($in{'User'} == 1) { return ( [ $in{'User_name'} ] ); }
 elsif ($in{'User_id'} !~ /^\-?\d+$/) {
 	&error(&text('core_egid', $in{'User_id'}));
 	}
-else { return ( [ "#$in{'User_id'}" ] ); }
+else { return ( [ "\"#$in{'User_id'}\"" ] ); }
 }
 
 #########################################################################
@@ -950,6 +1161,7 @@ for($i=0; $i<$len; $i++) {
 	if ($v =~ /^(\d+)\s+((http|ftp|gopher):\S+)$/)
 		{ $code = $1; $type = 0; $url = $2; }
 	elsif ($v =~ /^(\d+)\s+(\/.*)$/) { $code = $1; $type = 0; $url = $2; }
+	elsif ($v =~ /^(\d+)\s+"(.*)"$/) { $code = $1; $type = 1; $url = $2; }
 	elsif ($v =~ /^(\d+)\s+"?(.*)$/) { $code = $1; $type = 1; $url = $2; }
 	else { $code = ""; $type = 0; $url = ""; }
 	$rv .= "<tr $cb>\n";
@@ -969,12 +1181,13 @@ sub save_ErrorDocument
 local($i, $code, $url, @rv);
 for($i=0; defined($in{"ErrorDocument_code_$i"}); $i++) {
 	$code = $in{"ErrorDocument_code_$i"}; $url = $in{"ErrorDocument_url_$i"};
-	if ($code !~ /\S/ && $url !~ /\S/) { next; }
+	if ($code !~ /\S/ || $url !~ /\S/) { next; }
 	$code =~ /^\d\d\d$/ || &error(&text('core_eerror', $code));
 	if ($in{"ErrorDocument_type_$i"} == 0) {
 		$url =~ /^\S+$/ || &error(&text('core_eurl', $url));
 		push(@rv, "$code $url");
 		}
+	elsif ($_[0]->{'version'} >= 2.0) { push(@rv, "$code \"$url\""); }
 	elsif ($_[0]->{'version'} >= 1.2) { push(@rv, "$code \"$url"); }
 	else { push(@rv, "$code $url"); }
 	}
@@ -1017,6 +1230,8 @@ sub save_ErrorLog
 if ($_[0]->{'version'} < 1.3) {
 	$in{'ErrorLog_def'} || &allowed_auth_file($in{'ErrorLog'}) ||
 		&error($text{'core_edirlog'});
+	$in{'ErrorLog_def'} || &directory_exists($in{'ErrorLog'}) ||
+		    &error($text{'core_eerrordir'});
 	return &parse_opt("ErrorLog", '^\S+$', $text{'core_efilelog'});
 	}
 else {
@@ -1028,6 +1243,8 @@ else {
 		    &error($text{'core_efilemiss'});
 		&allowed_auth_file($in{'ErrorLog_file'}) ||
 		    &error($text{'core_edirlog'});
+		&directory_exists($in{'ErrorLog_file'}) ||
+		    &error($text{'core_eerrordir'});
 		return ( [ $in{'ErrorLog_file'} ] );
 		}
 	elsif ($in{'ErrorLog_type'} == 1) {
@@ -1043,18 +1260,48 @@ else {
 	}
 }
 
+sub edit_LogLevel
+{
+return (1, $text{'core_loglevel'},
+	&select_input($_[0]->{'value'}, "LogLevel", "",
+		      "$text{'core_log_emerg'} (emerg),emerg",
+		      "$text{'core_log_alert'} (alert),alert",
+		      "$text{'core_log_crit'} (crit),crit",
+		      "$text{'core_log_error'} (error),error",
+		      "$text{'core_log_warn'} (warn),warn",
+		      "$text{'core_log_notice'} (notice),notice",
+		      "$text{'core_log_info'} (info),info",
+		      "$text{'core_log_debug'} (debug),debug"));
+}
+sub save_LogLevel
+{
+return &parse_select("LogLevel", "");
+}
+
 #########################################################################
 # Module directives
+# This isn't shown if the distro has a way of managing these, such as Debian's
+# /etc/apache/mods-enabled
 sub edit_ClearModuleList_AddModule
 {
 local($mods, @allmods, $d, %mods, $m, $i, $rv);
-($ver, $mods) = &httpd_info($config{'httpd_path'});
+local $httpd = &find_httpd();
+($ver, $mods) = &httpd_info($httpd);
 @allmods = grep { !/^core$/ } @$mods;
 local $conf = &get_config();
 foreach $d (&find_directive_struct("LoadModule", $conf)) {
+	# Add mod_ like modules
 	if ($d->{'words'}->[1] =~ /(mod_\S+)\.(so|dll)/) {
 		push(@allmods, $1);
 		}
+	# nodo50 v0.1 - Change 000002 - Bug fixed: Apache-ssl module detected as mod_ssl. Now Apache-ssl module included as mod_apachessl
+	# nodo50 v0.1 - Change 000002 - Bug corregido: El modulo Apache-ssl se detecta como mod_ssl. Ahora Apache-ssl se incluye como mod_apachessl
+	# Add apache-ssl libssl.so as mod_apachessl module
+	elsif ($d->{'words'}->[1] =~ /libssl\.so/) {
+		push(@allmods, "mod_apachessl");
+		}
+	# Add others lib* like as mod_ modules
+	# nodo50 v0.1 - Change 000002 - End
 	elsif ($d->{'words'}->[1] =~ /lib([^\/\s]+)\.(so|dll)/) {
 		push(@allmods, "mod_$1");
 		}
