@@ -1,24 +1,65 @@
-# cron-lib.pl
-# Common crontab functions
-# XXX support for envs in /etc/crontab and /etc/cron.d (impossible!)
+=head1 cron-lib.pl
 
-do '../web-lib.pl';
+Functions for listing, creating and managing Unix users' cron jobs.
+
+ foreign_require("cron", "cron-lib.pl");
+ @jobs = cron::list_cron_jobs();
+ $job = { 'user' => 'root',
+          'active' => 1,
+          'command' => 'ls -l >/dev/null',
+          'special' => 'hourly' };
+ cron::create_cron_job($job);
+
+=cut
+
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
-require '../ui-lib.pl';
 %access = &get_module_acl();
 $env_support = $config{'vixie_cron'};
 if ($module_info{'usermin'}) {
 	$single_user = $remote_user;
 	&switch_to_remote_user();
+	&create_user_config_dirs();
+	$range_cmd = "$user_module_config_directory/range.pl";
+	$hourly_only = 0;
 	}
+else {
+	$range_cmd = "$module_config_directory/range.pl";
+	$hourly_only = $access{'hourly'} == 0 ? 0 :
+		       $access{'hourly'} == 1 ? 1 :
+			$config{'hourly_only'};
+	}
+$temp_delete_cmd = "$module_config_directory/tempdelete.pl";
 $cron_temp_file = &transname();
+use Time::Local;
 
-# list_cron_jobs()
-# Returns a lists of structures of all cron jobs
+=head2 list_cron_jobs
+
+Returns a lists of structures of all cron jobs, each of which is a hash
+reference with the following keys :
+
+=item user - Unix user the job runs as.
+
+=item command - The full command to be run.
+
+=item active - Set to 0 if the job is commented out, 1 if active.
+
+=item mins - Minute or comma-separated list of minutes the job will run, or * for all.
+
+=item hours - Hour or comma-separated list of hours the job will run, or * for all.
+
+=item days - Day or comma-separated list of days of the month the job will run, or * for all.
+
+=item month - Month number or comma-separated list of months (started from 1) the job will run, or * for all.
+
+=item weekday - Day of the week or comma-separated list of days (where 0 is sunday) the job will run, or * for all
+
+=cut
 sub list_cron_jobs
 {
 local (@rv, $lnum, $f);
-if (defined(@cron_jobs_cache)) {
+if (scalar(@cron_jobs_cache)) {
 	return @cron_jobs_cache;
 	}
 
@@ -27,6 +68,9 @@ if ($config{'system_crontab'}) {
 	$lnum = 0;
 	&open_readfile(TAB, $config{'system_crontab'});
 	while(<TAB>) {
+		# Comment line in Fedora 13
+		next if (/^#+\s+\*\s+\*\s+\*\s+\*\s+\*\s+(user-name\s+)?command\s+to\s+be\s+executed/);
+
 		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(\S+)\s+(.*)/i) {
 			# A normal h m s d w time
 			push(@rv, { 'file' => $config{'system_crontab'},
@@ -99,6 +143,42 @@ while($f = readdir(DIR)) {
 	}
 closedir(DIR);
 
+# Read a single user's crontab file
+if ($config{'single_file'}) {
+	&open_readfile(TAB, $config{'single_file'});
+	$lnum = 0;
+	while(<TAB>) {
+		if (/^(#+)?[\s\&]*(-)?\s*([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+([0-9\-\*\/,]+)\s+(([0-9\-\*\/]+|jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec|,)+)\s+(([0-9\-\*\/]+|sun|mon|tue|wed|thu|fri|sat|,)+)\s+(.*)/i) {
+			# A normal m h d m wd time
+			push(@rv, { 'file' => $config{'single_file'},
+				    'line' => $lnum,
+				    'type' => 3,
+				    'active' => !$1, 'nolog' => $2,
+				    'mins' => $3, 'hours' => $4,
+				    'days' => $5, 'months' => $6,
+				    'weekdays' => $8, 'user' => "NONE",
+				    'command' => $10,
+				    'index' => scalar(@rv) });
+			&fix_names($rv[$#rv]);
+			}
+		elsif (/^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*'([^']*)'/ ||
+		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*"([^']*)"/ ||
+		       /^(#+)?\s*([a-zA-Z0-9\_]+)\s*=\s*(\S+)/) {
+			# An environment variable
+			push(@rv, { 'file' => $config{'single_file'},
+				    'line' => $lnum,
+				    'active' => !$1,
+				    'name' => $2,
+				    'value' => $3,
+				    'user' => "NONE",
+				    'index' => scalar(@rv) });
+			}
+		$lnum++;
+		}
+	close(TAB);
+	}
+
+
 # read per-user cron files
 local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
 local @users;
@@ -170,6 +250,11 @@ closedir(DIR);
 return @cron_jobs_cache;
 }
 
+=head2 cron_job_line(&job)
+
+Internal function to generate a crontab format line for a cron job.
+
+=cut
 sub cron_job_line
 {
 local @c;
@@ -191,13 +276,19 @@ else {
 			 $_[0]->{'hours'}, $_[0]->{'days'},
 			 $_[0]->{'months'}, $_[0]->{'weekdays'});
 		}
-	push(@c, $_[0]->{'user'}) if ($_[0]->{'type'});
+	push(@c, $_[0]->{'user'}) if ($_[0]->{'type'} != 0 &&
+				      $_[0]->{'type'} != 3);
 	push(@c, $_[0]->{'command'});
 	}
 return join(" ", @c);
 }
 
-# copy_cron_temp(&job)
+=head2 copy_cron_temp(&job)
+
+Copies a job's user's current cron configuration to the temp file. For internal
+use only.
+
+=cut
 sub copy_cron_temp
 {
 local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
@@ -216,44 +307,85 @@ else {
 	}
 }
 
-# create_cron_job(&job)
-# Add a Cron job to a user's file
+=head2 create_cron_job(&job)
+
+Add a Cron job to a user's file. The job parameter must be a hash reference
+in the same format as returned by list_cron_jobs.
+
+=cut
 sub create_cron_job
 {
+&check_cron_config_or_error();
 &list_cron_jobs();	# init cache
-&copy_cron_temp($_[0]);
-local $lref = &read_file_lines($cron_temp_file);
-$_[0]->{'line'} = scalar(@$lref);
-push(@$lref, &cron_job_line($_[0]));
-&flush_file_lines();
-system("chown $_[0]->{'user'} $cron_temp_file");
-&copy_crontab($_[0]->{'user'});
-$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
-$_[0]->{'index'} = scalar(@cron_jobs_cache);
-push(@cron_jobs_cache, $_[0]);
+if ($config{'add_file'}) {
+	# Add to a specific file, typically something like /etc/cron.d/webmin
+	$_[0]->{'type'} = 1;
+	local $lref = &read_file_lines($config{'add_file'});
+	push(@$lref, &cron_job_line($_[0]));
+	&flush_file_lines($config{'add_file'});
+	}
+elsif ($config{'single_file'} && !$config{'cron_dir'}) {
+	# Add to the single file
+	$_[0]->{'type'} = 3;
+	local $lref = &read_file_lines($config{'single_file'});
+	push(@$lref, &cron_job_line($_[0]));
+	&flush_file_lines($config{'single_file'});
+	}
+else {
+	# Add to the specified user's crontab
+	&copy_cron_temp($_[0]);
+	local $lref = &read_file_lines($cron_temp_file);
+	$_[0]->{'line'} = scalar(@$lref);
+	push(@$lref, &cron_job_line($_[0]));
+	&flush_file_lines();
+	&set_ownership_permissions($_[0]->{'user'}, undef, undef,
+				   $cron_temp_file);
+	&copy_crontab($_[0]->{'user'});
+	$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
+	$_[0]->{'index'} = scalar(@cron_jobs_cache);
+	push(@cron_jobs_cache, $_[0]);
+	}
 }
 
-# insert_cron_job(&job)
-# Add a Cron job at the top of the user's file
+=head2 insert_cron_job(&job)
+
+Add a Cron job at the top of the user's file. The job parameter must be a hash
+reference in the same format as returned by list_cron_jobs.
+
+=cut
 sub insert_cron_job
 {
+&check_cron_config_or_error();
 &list_cron_jobs();	# init cache
-&copy_cron_temp($_[0]);
-local $lref = &read_file_lines($cron_temp_file);
-$_[0]->{'line'} = 0;
-splice(@$lref, 0, 0, &cron_job_line($_[0]));
-&flush_file_lines();
-system("chown $_[0]->{'user'} $cron_temp_file");
-&copy_crontab($_[0]->{'user'});
-$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
-$_[0]->{'index'} = scalar(@cron_jobs_cache);
-&renumber($_[0]->{'file'}, $_[0]->{'line'}, 1);
-push(@cron_jobs_cache, $_[0]);
+if ($config{'single_file'} && !$config{'cron_dir'}) {
+	# Insert into single file
+	$_[0]->{'type'} = 3;
+	local $lref = &read_file_lines($config{'single_file'});
+	splice(@$lref, 0, 0, &cron_job_line($_[0]));
+	&flush_file_lines($config{'single_file'});
+	}
+else {
+	# Insert into the user's crontab
+	&copy_cron_temp($_[0]);
+	local $lref = &read_file_lines($cron_temp_file);
+	$_[0]->{'line'} = 0;
+	splice(@$lref, 0, 0, &cron_job_line($_[0]));
+	&flush_file_lines();
+	system("chown $_[0]->{'user'} $cron_temp_file");
+	&copy_crontab($_[0]->{'user'});
+	$_[0]->{'file'} = "$config{'cron_dir'}/$_[0]->{'user'}";
+	$_[0]->{'index'} = scalar(@cron_jobs_cache);
+	&renumber($_[0]->{'file'}, $_[0]->{'line'}, 1);
+	push(@cron_jobs_cache, $_[0]);
+	}
 }
 
-# renumber(file, line, offset)
-# All jobs in this file whose line is at or after the given one will be
-# incremented by the offset
+=head2 renumber(file, line, offset)
+
+All jobs in this file whose line is at or after the given one will be
+incremented by the offset. For internal use.
+
+=cut
 sub renumber
 {
 local $j;
@@ -265,7 +397,12 @@ foreach $j (@cron_jobs_cache) {
 	}
 }
 
-# renumber_index(index, offset)
+=head2 renumber_index(index, offset)
+
+Internal function to change the index of all cron jobs in the cache after
+some index by a given offset. For internal use.
+
+=cut
 sub renumber_index
 {
 local $j;
@@ -276,7 +413,12 @@ foreach $j (@cron_jobs_cache) {
 	}
 }
 
-# change_cron_job(&job)
+=head2 change_cron_job(&job)
+
+Updates the given cron job, which must be a hash ref returned by list_cron_jobs
+and modified with a new active flag, command or schedule.
+
+=cut
 sub change_cron_job
 {
 if ($_[0]->{'type'} == 0) {
@@ -291,7 +433,12 @@ else {
 	}
 }
 
-# delete_cron_job(&job)
+=head2 delete_cron_job(&job)
+
+Removes the cron job defined by the given hash ref, as returned by
+list_cron_jobs.
+
+=cut
 sub delete_cron_job
 {
 if ($_[0]->{'type'} == 0) {
@@ -307,8 +454,12 @@ else {
 &renumber_index($_[0]->{'index'}, -1);
 }
 
-# read_crontab(user)
-# Return an array containing the lines of the cron table for some user
+=head2 read_crontab(user)
+
+Return an array containing the lines of the cron table for some user. For
+internal use mainly.
+
+=cut
 sub read_crontab
 {
 local(@tab);
@@ -322,12 +473,19 @@ if (@tab >= 3 && $tab[0] =~ /DO NOT EDIT/ &&
 return @tab;
 }
 
+=head2 copy_crontab(user)
 
-# copy_crontab(user)
+Copy the cron temp file to that for this user. For internal use only.
+
+=cut
 sub copy_crontab
 {
+if (&is_readonly_mode()) {
+	# Do nothing
+	return undef;
+	}
 local($pwd);
-if (`cat $cron_temp_file` =~ /\S/) {
+if (&read_file_contents($cron_temp_file) =~ /\S/) {
 	local $temp = &transname();
 	local $rv;
 	if ($config{'cron_edit_command'}) {
@@ -343,6 +501,8 @@ if (`cat $cron_temp_file` =~ /\S/) {
 			"$module_root_directory/cron_editor.pl";
 		$ENV{"CRON_EDITOR_COPY"} = $cron_temp_file;
 		system("chown $_[0] $cron_temp_file");
+		local $oldpwd = &get_current_dir();
+		chdir("/");
 		if ($single_user) {
 			$rv = system($config{'cron_user_edit_command'}.
 				     " >$temp 2>&1 <$notemp");
@@ -353,6 +513,7 @@ if (`cat $cron_temp_file` =~ /\S/) {
 				" >$temp 2>&1 <$notemp");
 			}
 		unlink($notemp);
+		chdir($oldpwd);
 		}
 	else {
 		# use the cron copy command
@@ -367,10 +528,10 @@ if (`cat $cron_temp_file` =~ /\S/) {
 				$cron_temp_file, $temp, $temp);
 			}
 		}
-	local $out = `cat $temp`;
+	local $out = &read_file_contents($temp);
 	unlink($temp);
 	if ($rv || $out =~ /error/i) {
-		local $cronin = `cat $cron_temp_file`;
+		local $cronin = &read_file_contents($cron_temp_file);
 		&error(&text('ecopy', "<pre>$out</pre>", "<pre>$cronin</pre>"));
 		}
 	}
@@ -388,9 +549,12 @@ unlink($cron_temp_file);
 }
 
 
-# parse_job(job)
-# Parse a crontab line into an array containing:
-#  active, mins, hrs, days, mons, weekdays, command
+=head2 parse_job(job-line)
+
+Parse a crontab line into an array containing:
+active, mins, hrs, days, mons, weekdays, command
+
+=cut
 sub parse_job
 {
 local($job, $active) = ($_[0], 1);
@@ -402,8 +566,12 @@ $job =~ /^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(.*)$/;
 return ($active, $1, $2, $3, $4, $5, $6);
 }
 
-# user_sub(command, user)
-# Replace the string 'USER' in the command with the user name
+=head2 user_sub(command, user)
+
+Replace the string 'USER' in the command with the user name. For internal
+use only.
+
+=cut
 sub user_sub
 {
 local($tmp);
@@ -413,8 +581,11 @@ return $tmp;
 }
 
 
-# list_allowed()
-# Returns a list of all users in the cron allow file
+=head2 list_allowed
+
+Returns a list of all Unix usernames who are allowed to use Cron.
+
+=cut
 sub list_allowed
 {
 local(@rv, $_);
@@ -428,8 +599,11 @@ return @rv;
 }
 
 
-# list_denied()
-# Return a list of users from the cron deny file
+=head2 list_denied
+
+Return a list of all Unix usernames who are not allowed to use Cron.
+
+=cut
 sub list_denied
 {
 local(@rv, $_);
@@ -443,8 +617,11 @@ return @rv;
 }
 
 
-# save_allowed(user, user, ...)
-# Save the list of allowed users
+=head2 save_allowed(user, user, ...)
+
+Save the list of allowed Unix usernames.
+
+=cut
 sub save_allowed
 {
 local($_);
@@ -457,8 +634,11 @@ chmod(0444, $config{cron_allow_file});
 }
 
 
-# save_denied(user, user, ...)
-# Save the list of denied users
+=head2 save_denied(user, user, ...)
+
+Save the list of denied Unix usernames.
+
+=cut
 sub save_denied
 {
 local($_);
@@ -470,9 +650,12 @@ foreach (@_) {
 chmod(0444, $config{cron_deny_file});
 }
 
-# read_envs(user)
-# Returns an array of name,value pairs containing the environment settings
-# from the crontab for some user
+=head2 read_envs(user)
+
+Returns an array of "name value" strings containing the environment settings
+from the crontab for some user
+
+=cut
 sub read_envs
 {
 local(@tab, @rv, $_);
@@ -484,9 +667,12 @@ foreach (@tab) {
 return @rv;
 }
 
-# save_envs(user, [name, value]*)
-# Updates the cron file for some user with the given list of environment
-# variables. All others in the file are removed
+=head2 save_envs(user, [name, value]*)
+
+Updates the cron file for some user with the given list of environment
+variables. All others in the file are removed.
+
+=cut
 sub save_envs
 {
 local($i, @tab, $line);
@@ -503,7 +689,12 @@ close(TAB);
 &copy_crontab($_[0]);
 }
 
-# expand_run_parts(directory)
+=head2 expand_run_parts(directory)
+
+Internal function to convert a directory like /etc/cron.hourly into a list
+of scripts in that directory.
+
+=cut
 sub expand_run_parts
 {
 local $dir = $_[0];
@@ -517,14 +708,24 @@ closedir(DIR);
 return @rv;
 }
 
-# is_run_parts(command)
+=head2 is_run_parts(command)
+
+Returns the dir if some cron job runs a list of commands in some directory, 
+like /etc/cron.hourly. Returns undef otherwise.
+
+=cut
 sub is_run_parts
 {
 local $rp = $config{'run_parts'};
-return $rp && $_[0] =~ /$rp(.*)\s+(\S+)$/ ? $2 : undef;
+return $rp && $_[0] =~ /$rp(.*)\s+(\-\-\S+\s+)*([a-z0-9\.\-\/_]+)(\s*\))?$/i ? $3 : undef;
 }
 
-# can_edit_user(&access, user)
+=head2 can_edit_user(&access, user)
+
+Returns 1 if the Webmin user whose permissions are defined by the access hash
+ref can manage cron jobs for a given Unix user.
+
+=cut
 sub can_edit_user
 {
 local %umap;
@@ -549,16 +750,24 @@ else {
 	}
 }
 
-# show_times_input(&job, [nospecial])
+=head2 show_times_input(&job, [nospecial])
+
+Print HTML for inputs for selecting the schedule for a cron job, defined
+by the first parameter which must be a hash ref returned by list_cron_jobs.
+This must be used inside a <table>, as the HTML starts and ends with <tr>
+tags.
+
+=cut
 sub show_times_input
 {
+return &theme_show_times_input(@_) if (defined(&theme_show_times_input));
 local $job = $_[0];
-if ($config{'vixie_cron'} && !$_[1]) {
+if ($config{'vixie_cron'} && (!$_[1] || $_[0]->{'special'})) {
 	# Allow selection of special @ times
 	print "<tr $cb> <td colspan=6>\n";
 	printf "<input type=radio name=special_def value=1 %s> %s\n",
 		$job->{'special'} ? "checked" : "", $text{'edit_special1'};
-	print "<select name=special>\n";
+	print "<select name=special onChange='change_special_mode(form, 1)'>\n";
 	local $s;
 	local $sp = $job->{'special'} eq 'midnight' ? 'daily' :
 	    $job->{'special'} eq 'annually' ? 'yearly' : $job->{'special'};
@@ -572,6 +781,27 @@ if ($config{'vixie_cron'} && !$_[1]) {
 	print "</td></tr>\n";
 	}
 
+# Javascript to disable and enable fields
+print <<EOF;
+<script>
+function enable_cron_fields(name, form, ena)
+{
+var els = form.elements[name];
+els.disabled = !ena;
+for(i=0; i<els.length; i++) {
+  els[i].disabled = !ena;
+  }
+change_special_mode(form, 0);
+}
+
+function change_special_mode(form, special)
+{
+form.special_def[0].checked = special;
+form.special_def[1].checked = !special;
+}
+</script>
+EOF
+
 print "<tr $tb>\n";
 print "<td><b>$text{'edit_mins'}</b></td> <td><b>$text{'edit_hours'}</b></td> ",
       "<td><b>$text{'edit_days'}</b></td> <td><b>$text{'edit_months'}</b></td>",
@@ -614,17 +844,27 @@ foreach $arr ("mins", "hours", "days", "months", "weekdays") {
 
 	# Output selection list
 	print "<td valign=top>\n";
-	printf "<input type=radio name=all_$arr value=1 %s> $text{'edit_all'}<br>\n",
-		$job->{$arr} eq "*" ? "checked" : "";
-	printf "<input type=radio name=all_$arr value=0 %s> $text{'edit_selected'}<br>\n",
-		$job->{$arr} ne "*" ? "checked" : "";
+        printf "<input type=radio name=all_$arr value=1 %s %s %s> %s<br>\n",
+                $arr eq "mins" && $hourly_only ? "disabled" : "",
+		$job->{$arr} eq "*" ||  $job->{$arr} eq "" ? "checked" : "",
+		"onClick='enable_cron_fields(\"$arr\", form, 0)'",
+		$text{'edit_all'};
+	printf "<input type=radio name=all_$arr value=0 %s %s> %s<br>\n",
+		$job->{$arr} eq "*" || $job->{$arr} eq "" ? "" : "checked",
+		"onClick='enable_cron_fields(\"$arr\", form, 1)'",
+		$text{'edit_selected'};
 	print "<table> <tr>\n";
-	for($j=0; $j<@$arr; $j+=12) {
-		$jj = $j+11;
+        for($j=0; $j<@$arr; $j+=($arr eq "mins" && $hourly_only ? 60 : 12)) {
+                $jj = $j+($arr eq "mins" && $hourly_only ? 59 : 11);
 		if ($jj >= @$arr) { $jj = @$arr - 1; }
 		@sec = @$arr[$j .. $jj];
-		printf "<td valign=top><select multiple size=%d name=$arr>\n",
-			@sec > 12 ? 12 : scalar(@sec);
+                printf "<td valign=top><select %s size=%d name=$arr %s %s>\n",
+                        $arr eq "mins" && $hourly_only ? "" : "multiple",
+                        @sec > 12 ? ($arr eq "mins" && $hourly_only ? 1 : 12)
+				  : scalar(@sec),
+			$job->{$arr} eq "*" ||  $job->{$arr} eq "" ?
+				"disabled" : "",
+			"onChange='change_special_mode(form, 0)'";
 		foreach $v (@sec) {
 			if ($v =~ /^(.*)=(.*)$/) { $disp = $1; $code = $2; }
 			else { $disp = $code = $v; }
@@ -638,7 +878,13 @@ foreach $arr ("mins", "hours", "days", "months", "weekdays") {
 print "</tr> <tr $cb> <td colspan=5>$text{'edit_ctrl'}</td> </tr>\n";
 }
 
-# parse_times_input(&job, &in)
+=head2 parse_times_input(&job, &in)
+
+Parses inputs from the form generated by show_times_input, and updates a cron
+job hash ref. The in parameter must be a hash ref as generated by the 
+ReadParse function.
+
+=cut
 sub parse_times_input
 {
 local $job = $_[0];
@@ -697,12 +943,71 @@ else {
 	}
 }
 
+=head2 show_range_input(&job)
+
+Given a cron job, prints fields for selecting it's run date range.
+
+=cut
+sub show_range_input
+{
+local ($job) = @_;
+local $has_start = $job->{'start'};
+local $rng;
+$rng = &text('range_start', &ui_date_input(
+	$job->{'start'}->[0], $job->{'start'}->[1], $job->{'start'}->[2],
+	"range_start_day", "range_start_month", "range_start_year"))."\n".
+      &date_chooser_button(
+	"range_start_day", "range_start_month", "range_start_year")."\n".
+      &text('range_end', &ui_date_input(
+	$job->{'end'}->[0], $job->{'end'}->[1], $job->{'end'}->[2],
+	"range_end_day", "range_end_month", "range_end_year"))."\n".
+      &date_chooser_button(
+	"range_end_day", "range_end_month", "range_end_year")."\n";
+print &ui_oneradio("range_def", 1, $text{'range_all'}, !$has_start),
+      "<br>\n";
+print &ui_oneradio("range_def", 0, $rng, $has_start),"\n";
+}
+
+=head2 parse_range_input(&job, &in)
+
+Updates the job object with the specified date range. May call &error
+for invalid inputs.
+
+=cut
+sub parse_range_input
+{
+local ($job, $in) = @_;
+if ($in->{'range_def'}) {
+	# No range used
+	delete($job->{'start'});
+	delete($job->{'end'});
+	}
+else {
+	# Validate and store range
+	foreach my $r ("start", "end") {
+		eval { timelocal(0, 0, 0, $in->{'range_'.$r.'_day'},
+					  $in->{'range_'.$r.'_month'}-1,
+					  $in->{'range_'.$r.'_year'}-1900) };
+		if ($@) {
+			&error($text{'range_e'.$r}." ".$@);
+			}
+		$job->{$r} = [ $in->{'range_'.$r.'_day'},
+			       $in->{'range_'.$r.'_month'},
+			       $in->{'range_'.$r.'_year'} ];
+		}
+	}
+}
+
 @cron_month = ( 'jan', 'feb', 'mar', 'apr', 'may', 'jun',
 		'jul', 'aug', 'sep', 'oct', 'nov', 'dec' );
 @cron_weekday = ( 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' );
 
-# fix_names(&cron)
-# Convert day and month names to numbers
+=head2 fix_names(&cron)
+
+Convert day and month names to numbers. For internal use when parsing
+the crontab file.
+
+=cut
 sub fix_names
 {
 local ($m, $w);
@@ -723,41 +1028,79 @@ foreach $w (@wds) {
 $_[0]->{'weekdays'} = join(",", @wds);
 }
 
-# create_wrapper(wrapper-path, module, script)
-# Creates a wrapper script which calls a script in some module's directory
-# with the proper webmin environment variables set. This should always be used
-# when setting up a cron job, instead of attempting to run a command in the
-# module directory directly.
+=head2 create_wrapper(wrapper-path, module, script)
+
+Creates a wrapper script which calls a script in some module's directory
+with the proper webmin environment variables set. This should always be used
+when setting up a cron job, instead of attempting to run a command in the
+module directory directly.
+
+The parameters are :
+
+=item wrapper-path - Full path to the wrapper to create, like /etc/webmin/yourmodule/foo.pl
+
+=item module - Module containing the real script to call.
+
+=item script - Program within that module for the wrapper to run.
+
+=cut
 sub create_wrapper
 {
 local $perl_path = &get_perl_path();
 &open_tempfile(CMD, ">$_[0]");
 &print_tempfile(CMD, <<EOF
 #!$perl_path
-open(CONF, "$config_directory/miniserv.conf");
+open(CONF, "$config_directory/miniserv.conf") || die "Failed to open $config_directory/miniserv.conf : \$!";
 while(<CONF>) {
-\$root = \$1 if (/^root=(.*)/);
-}
+        \$root = \$1 if (/^root=(.*)/);
+        }
 close(CONF);
+\$root || die "No root= line found in $config_directory/miniserv.conf";
+\$ENV{'PERLLIB'} = "\$root";
 \$ENV{'WEBMIN_CONFIG'} = "$ENV{'WEBMIN_CONFIG'}";
 \$ENV{'WEBMIN_VAR'} = "$ENV{'WEBMIN_VAR'}";
-chdir("\$root/$_[1]");
-exec("\$root/$_[1]/$_[2]", \@ARGV) || die "Failed to run \$root/$_[1]/$_[2] : \$!";
 EOF
 	);
+if ($gconfig{'os_type'} eq 'windows') {
+	# On windows, we need to chdir to the drive first, and use system
+	&print_tempfile(CMD, "if (\$root =~ /^([a-z]:)/i) {\n");
+	&print_tempfile(CMD, "       chdir(\"\$1\");\n");
+	&print_tempfile(CMD, "       }\n");
+	&print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
+	&print_tempfile(CMD, "exit(system(\"\$root/$_[1]/$_[2]\", \@ARGV));\n");
+	}
+else {
+	# Can use exec on Unix systems
+	if ($_[1]) {
+		&print_tempfile(CMD, "chdir(\"\$root/$_[1]\");\n");
+		&print_tempfile(CMD, "exec(\"\$root/$_[1]/$_[2]\", \@ARGV) || die \"Failed to run \$root/$_[1]/$_[2] : \$!\";\n");
+		}
+	else {
+		&print_tempfile(CMD, "chdir(\"\$root\");\n");
+		&print_tempfile(CMD, "exec(\"\$root/$_[2]\", \@ARGV) || die \"Failed to run \$root/$_[2] : \$!\";\n");
+		}
+	}
 &close_tempfile(CMD);
 chmod(0755, $_[0]);
 }
 
-# cron_file(&job)
-# Returns the file that a cron job is in, or would be in
+=head2 cron_file(&job)
+
+Returns the file that a cron job is in, or will be in when it is created
+based on the username.
+
+=cut
 sub cron_file
 {
-return $_[0]->{'file'} || "$config{'cron_dir'}/$_[0]->{'user'}";
+return $_[0]->{'file'} || $config{'add_file'} ||
+       "$config{'cron_dir'}/$_[0]->{'user'}";
 }
 
-# when_text(&job, [upper-case-first])
-# Returns a text string describing when a cron job is run
+=head2 when_text(&job, [upper-case-first])
+
+Returns a human-readable text string describing when a cron job is run.
+
+=cut
 sub when_text
 {
 local $pfx = $_[1] ? "uc" : "";
@@ -785,9 +1128,12 @@ else {
 	}
 }
 
-# can_use_cron(user)
-# Returns 1 if some user is allowed to use cron, based on cron.allow and
-# cron.deny files
+=head2 can_use_cron(user)
+
+Returns 1 if some user is allowed to use cron, based on cron.allow and
+cron.deny files.
+
+=cut
 sub can_use_cron
 {
 local $err;
@@ -808,8 +1154,12 @@ elsif ($config{cron_deny_all} == 1) {
 return !$err;
 }
 
-# swap_cron_jobs(&job1, &job2)
-# Swaps two Cron jobs, which must be in the same file
+=head2 swap_cron_jobs(&job1, &job2)
+
+Swaps two Cron jobs, which must be in the same file, identified by their
+hash references as returned by list_cron_jobs.
+
+=cut
 sub swap_cron_jobs
 {
 if ($_[0]->{'type'} == 0) {
@@ -828,7 +1178,15 @@ else {
 	}
 }
 
-# find_cron_process(&job, [&procs])
+=head2 find_cron_process(&job, [&procs])
+
+Finds the running process that was launched from a cron job. The parameters are:
+
+=item job - A cron job hash reference
+
+=item procs - An optional array reference of running process hash refs
+
+=cut
 sub find_cron_process
 {
 local @procs;
@@ -864,5 +1222,193 @@ if (!$proc && $cmd =~ /^$config_directory\/(.*\.pl)(.*)$/) {
 return $proc;
 }
 
+=head2 find_cron_job(command, [&jobs], [user])
+
+Returns the cron job object that runs some command (perhaps with redirection)
+
+=cut
+sub find_cron_job
+{
+my ($cmd, $jobs, $user) = @_;
+if (!$jobs) {
+	$jobs = [ &list_cron_jobs() ];
+	}
+$user ||= "root";
+my @rv = grep { $_->{'user'} eq $user &&
+	     $_->{'command'} =~ /(^|[ \|\&;\/])\Q$cmd\E($|[ \|\&><;])/ } @$jobs;
+return wantarray ? @rv : $rv[0];
+}
+
+=head2 extract_input(command)
+
+Given a line formatted like I<command%input>, returns the command and input
+parts, taking any escaping into account.
+
+=cut
+sub extract_input
+{
+local ($cmd) = @_;
+$cmd =~ s/\\%/\0/g;
+local ($cmd, $input) = split(/\%/, $cmd, 2);
+$cmd =~ s/\0/%/g;
+$input =~ s/\0/%/g;
+return ($cmd, $input);
+}
+
+=head2 convert_range(&job)
+
+Given a cron job that uses range.pl, work out the date range and update
+the job object command. Mainly for internal use.
+
+=cut
+sub convert_range
+{
+local ($job) = @_;
+local ($cmd, $input) = &extract_input($job->{'command'});
+if ($cmd =~ /^\Q$range_cmd\E\s+(\d+)\-(\d+)\-(\d+)\s+(\d+)\-(\d+)\-(\d+)\s+(.*)$/) {
+	# Looks like a range command
+	$job->{'start'} = [ $1, $2, $3 ];
+	$job->{'end'} = [ $4, $5, $6 ];
+	$job->{'command'} = $7;
+	$job->{'command'} =~ s/\\(.)/$1/g;
+	if ($input) {
+		$job->{'command'} .= '%'.$input;
+		}
+	return 1;
+	}
+return 0;
+}
+
+=head2 unconvert_range(&job)
+
+Give a cron job with start and end fields, updates the command to wrap it in
+range.pl with those dates as parameters.
+
+=cut
+sub unconvert_range
+{
+local ($job) = @_;
+if ($job->{'start'}) {
+	# Need to add range command
+	local ($cmd, $input) = &extract_input($job->{'command'});
+	$job->{'command'} = $range_cmd." ".join("-", @{$job->{'start'}})." ".
+					   join("-", @{$job->{'end'}})." ".
+					   quotemeta($cmd);
+	if ($input) {
+		$job->{'command'} .= '%'.$input;
+		}
+	delete($job->{'start'});
+	delete($job->{'end'});
+	&copy_source_dest("$module_root_directory/range.pl", $range_cmd);
+	&set_ownership_permissions(undef, undef, 0755, $range_cmd);
+	return 1;
+	}
+return 0;
+}
+
+=head2 convert_comment(&job)
+
+Given a cron job with a # comment after the command, sets the comment field
+
+=cut
+sub convert_comment
+{
+local ($job) = @_;
+if ($job->{'command'} =~ /^(.*)\s*#([^#]*)$/) {
+	$job->{'command'} = $1;
+	$job->{'comment'} = $2;
+	return 1;
+	}
+return 0;
+}
+
+=head2 unconvert_comment(&job)
+
+Adds an comment back to the command in a cron job, based on the comment field
+of the given hash reference.
+
+=cut
+sub unconvert_comment
+{
+local ($job) = @_;
+if ($job->{'comment'} =~ /\S/) {
+	$job->{'command'} .= " #".$job->{'comment'};
+	return 1;
+	}
+return 0;
+}
+
+=head2 check_cron_config
+
+Returns an error message if the cron config doesn't look valid, or some needed
+command is missing.
+
+=cut
+sub check_cron_config
+{
+# Check for single file and getter command
+if ($config{'single_file'} && !-r $config{'single_file'}) {
+	return &text('index_esingle', "<tt>$config{'single_file'}</tt>");
+	}
+if ($config{'cron_get_command'} =~ /^(\S+)/ && !&has_command("$1")) {
+	return &text('index_ecmd', "<tt>$1</tt>");
+	}
+# Check for directory
+local $fcron = ($config{'cron_dir'} =~ /\/fcron$/);
+if (!$single_user && !$config{'single_file'} &&
+    !$fcron && !-d $config{'cron_dir'}) {
+	return &text('index_ecrondir', "<tt>$config{'cron_dir'}</tt>");
+	}
+return undef;
+}
+
+=head2 check_cron_config_or_error
+
+Calls check_cron_config, and then error if any problems were detected.
+
+=cut
+sub check_cron_config_or_error
+{
+local $err = &check_cron_config();
+if ($err) {
+	&error(&text('index_econfigcheck', $err));
+	}
+}
+
+=head2 cleanup_temp_files
+
+Called from cron to delete old files in the Webmin /tmp directory
+
+=cut
+sub cleanup_temp_files
+{
+# Don't run if disabled
+if (!$gconfig{'tempdelete_days'}) {
+	print STDERR "Temp file clearing is disabled\n";
+	return;
+	}
+if ($gconfig{'tempdir'} && !$gconfig{'tempdirdelete'}) {
+	print STDERR "Temp file clearing is not done for the custom directory $gconfig{'tempdir'}\n";
+	return;
+	}
+
+local $tempdir = &transname();
+$tempdir =~ s/\/([^\/]+)$//;
+if (!$tempdir || $tempdir eq "/") {
+	$tempdir = "/tmp/.webmin";
+	}
+
+local $cutoff = time() - $gconfig{'tempdelete_days'}*24*60*60;
+opendir(DIR, $tempdir);
+foreach my $f (readdir(DIR)) {
+	next if ($f eq "." || $f eq "..");
+	local @st = lstat("$tempdir/$f");
+	if ($st[9] < $cutoff) {
+		&unlink_file("$tempdir/$f");
+		}
+	}
+closedir(DIR);
+}
+
 1;
 
