@@ -1,8 +1,10 @@
 # dfs-lib.pl
 # Common functions for managing dfstab files
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
+%access = &get_module_acl();
 
 $default_type = 'nfs';
 if ($config{'fstypes_file'} && open(TYPES, $config{'fstypes_file'})) {
@@ -11,6 +13,7 @@ if ($config{'fstypes_file'} && open(TYPES, $config{'fstypes_file'})) {
 		}
 	close(TYPES);
 	}
+%access = &get_module_acl();
 
 # list_shares()
 # Returns a list of structures containing share details
@@ -34,7 +37,7 @@ while(<DFS>) {
 			$lnum++;
 			}
 		$share->{'eline'} = $lnum;
-		if ($line =~ /(\/\S+)/) {
+		if ($line =~ /\s(\/\S+)/) {
 			$share->{'dir'} = $1;
 			}
 		if ($line =~ /-d\s+"([^"]+)"/) { $share->{'desc'} = $1; }
@@ -55,9 +58,9 @@ return @rv;
 # Add a new share to the dfstab file
 sub create_share
 {
-open(DFS, ">> $config{dfstab_file}");
-print DFS &share_line($_[0]),"\n";
-close(DFS);
+&open_tempfile(DFS, ">> $config{dfstab_file}");
+&print_tempfile(DFS, &share_line($_[0]),"\n");
+&close_tempfile(DFS);
 }
 
 # modify_share(&share)
@@ -127,5 +130,21 @@ foreach $k (keys %$o) {
 return join(',', @list);
 }
 
+# apply_configuration()
+# Apply the NFS configuration, returning undef on success or an error message
+# on failure
+sub apply_configuration
+{
+local $temp = &transname();
+&system_logged("$config{unshare_all_command} >/dev/null 2>&1");
+&system_logged("$config{share_all_command} >/dev/null 2>$temp");
+local $why = `/bin/cat $temp`;
+unlink($temp);
+if ($why =~ /\S+/) {
+	return $why;
+	}
+return undef;
+}
+
 1;
 
