@@ -1,29 +1,35 @@
 # dhcpd-lib.pl
 # Functions for parsing the DHCPD config file
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
 
-# get_parent_config()
+# get_parent_config([file])
 # Returns a dummy parent structure for the DHCP config
 sub get_parent_config
 { 
-return $get_parent_config_cache if ($get_parent_config_cache);
-return $get_parent_config_cache = {
-	 'file' => $config{'dhcpd_conf'},
-	 'members' => &get_config(),
+local ($file) = @_;
+$file ||= $config{'dhcpd_conf'};
+return $get_parent_config_cache{$file} if ($get_parent_config_cache{$file});
+return $get_parent_config_cache{$file} = {
+	 'file' => $file,
+	 'members' => &get_config($file),
 	 'line' => -1,
+	 'fline' => -1,
 	 'eline' => $get_config_lines };
 }
 
-# get_config()
+# get_config([file])
 # Parses the DHCPD config file into a data structure
 sub get_config
 {
-return \@get_config_cache if (@get_config_cache);
-local @rv = &get_config_file($config{'dhcpd_conf'}, \$get_config_lines);
-@get_config_cache = @rv;
-return \@get_config_cache;
+local ($file) = @_;
+$file ||= $config{'dhcpd_conf'};
+return $get_config_cache{$file} if ($get_config_cache{$file});
+local @rv = &get_config_file($file, \$get_config_lines);
+$get_config_cache{$file} = \@rv;
+return $get_config_cache{$file};
 }
 
 # get_config_file(file, [&lines])
@@ -96,10 +102,10 @@ while($line = <FILE>) {
 		if ($line =~ /^\s*"([^"]*)"(.*)$/) {
 			push(@{$_[1]}, [ $1, 1, $lines ]); $line = $2;
 			}
-		elsif ($line =~ /^\s*([{};])(.*)$/) {
+		elsif ($line =~ /^\s*([{};,])(.*)$/) {
 			push(@{$_[1]}, [ $1, 0, $lines ]); $line = $2;
 			}
-		elsif ($line =~ /^\s*([^{}; \t]+)(.*)$/) {
+		elsif ($line =~ /^\s*([^{}; \t,]+)(.*)$/) {
 			push(@{$_[1]}, [ $1, 0, $lines ]); $line = $2;
 			}
 		else { last; }
@@ -130,7 +136,8 @@ if ($str{'line'} && $lref->[$str{'line'}-1] =~ /^\s*(#|\/\/)\s*(.*)/) {
 while(1) {
 	# Add values between directive name and { or ;
 	$t = $_[0]->[++$i];
-	if ($t->[0] eq "{" || $t->[0] eq ";") { last; }
+	if (($t->[0] eq "{" && $str{'name'} ne 'option') ||
+	    $t->[0] eq ";") { last; }
 	elsif (!defined($t->[0])) { ${$_[1]} = $i; return undef; }
 	else { push(@vals, $t->[0]); push(@quotes, $t->[1]); }
 	push(@text, $t->[1] ? "\"$t->[0]\"" : $t->[0]);
@@ -290,6 +297,7 @@ $dir = { 'name' => $_[0], 'type' => 1, 'members' => \@vals };
 }
 
 # opt_input(text, name, &config, default, size, units)
+# Returns HTML for an optional text field
 sub opt_input
 {
 local($v, $rv);
@@ -306,6 +314,7 @@ return $rv;
 }
 
 # save_opt(name, &func, &parent, [indent], [quote])
+# Saves an optional text field
 sub save_opt
 {
 local($dir);
@@ -321,7 +330,7 @@ else {
 	}
 }
 
-# save_directive(&parent, [name|&oldvalues], &values, indent, start)
+# save_directive(&parent, [name|&oldvalues], &values, indent, start, [after])
 # Given a structure containing a directive name, type, values and members
 # add, update or remove that directive in config structure and data files.
 # Updating of files assumes that there is no overlap between directives -
@@ -333,10 +342,28 @@ $pm = $_[0]->{'members'};
 @oldv = ref($_[1]) ? @{$_[1]} : &find($_[1], $pm);
 @newv = @{$_[2]};
 for($i=0; $i<@oldv || $i<@newv; $i++) {
-	if ($i >= @oldv && $_[4]) {
+	if ($i >= @oldv && $_[5]) {
+		# a new directive is being added.. put it after some other
+		$lref = $_[0]->{'file'} ? &read_file_lines($_[0]->{'file'})
+					: [ ];
+		@nl = &directive_lines($newv[$i], $_[3]);
+		$nline = $_[5]->{'line'}+1;
+		$nidx = &indexof($_[5], @$pm) + 1;
+		splice(@$lref, $nline, 0, @nl);
+		&renumber(&get_config(), $nline,
+			  $_[0]->{'file'}, scalar(@nl));
+		&renumber_index($_[0]->{'members'}, $nidx, 1);
+		$newv[$i]->{'index'} = $nidx;
+		$newv[$i]->{'file'} = $_[0]->{'file'};
+		$newv[$i]->{'line'} = $nline;
+		$newv[$i]->{'eline'} = $nline + scalar(@nl);
+		splice(@$pm, $nidx, 0, $newv[$i]);
+		}
+	elsif ($i >= @oldv && $_[4]) {
 		# a new directive is being added.. put it at the start of
 		# the parent
-		$lref = &read_file_lines($_[0]->{'file'});
+		$lref = $_[0]->{'file'} ? &read_file_lines($_[0]->{'file'})
+					: [ ];
 		@nl = &directive_lines($newv[$i], $_[3]);
 		$nline = $_[0]->{'fline'}+1;
 		splice(@$lref, $nline, 0, @nl);
@@ -351,7 +378,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 	elsif ($i >= @oldv) {
 		# a new directive is being added.. put it at the end of
 		# the parent
-		$lref = &read_file_lines($_[0]->{'file'});
+		$lref = $_[0]->{'file'} ? &read_file_lines($_[0]->{'file'}) : [ ];
 		@nl = &directive_lines($newv[$i], $_[3]);
 		splice(@$lref, $_[0]->{'eline'}, 0, @nl);
 		&renumber(&get_config(), $_[0]->{'eline'},
@@ -363,7 +390,7 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		}
 	elsif ($i >= @newv) {
 		# a directive was deleted
-		$lref = &read_file_lines($oldv[$i]->{'file'});
+		$lref = $oldv[$i]->{'file'} ? &read_file_lines($oldv[$i]->{'file'}) : [ ];
 		$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
 		splice(@$lref, $oldv[$i]->{'line'}, $ol);
 		&renumber(&get_config(), $oldv[$i]->{'eline'},
@@ -373,7 +400,10 @@ for($i=0; $i<@oldv || $i<@newv; $i++) {
 		}
 	else {
 		# updating some directive
-		$lref = &read_file_lines($oldv[$i]->{'file'});
+		if (!defined($newv[$i]->{'comment'})) {
+			$newv[$i]->{'comment'} = $oldv[$i]->{'comment'};
+			}
+		$lref = $oldv[$i]->{'file'} ? &read_file_lines($oldv[$i]->{'file'}) : [ ];
 		@nl = &directive_lines($newv[$i], $_[3]);
 		$ol = $oldv[$i]->{'eline'} - $oldv[$i]->{'line'} + 1;
 		splice(@$lref, $oldv[$i]->{'line'}, $ol, @nl);
@@ -476,7 +506,10 @@ sub group_name
 {
 local @opts = &find("option", $_[1]->{'members'});
 local ($dn) = grep { $_->{'values'}->[0] eq 'domain-name' } @opts;
-return  $config{'group_name'} && $dn ? &text('index_gdom',$dn->{'values'}->[1]):
+return  $config{'group_name'} == 1 && $dn ?
+		&text('index_gdom',$dn->{'values'}->[1]) :
+	$config{'group_name'} == 2 && $_[1]->{'comment'} ?
+		$_[1]->{'comment'} :
 	$_[1]->{'values'}->[0] ? $_[1]->{'values'}->[0] :
         $_[0] == 0 ? $text{'index_nomemb'} :
 	$_[0] == 1 ? $text{'index_1memb'} :
@@ -560,10 +593,36 @@ local ($sr, $hr) = &get_subnets_and_hosts();
 return @{$hr};
 }
 
+sub get_host_shared_network
+{
+local ($h) = @_;
+local $shar;
+while($h) {
+	if ($h->{'name'} eq 'shared-network') {
+		return $h;
+		}
+	$h = $h->{'parent'};
+	}
+return undef;
+}
+
+sub get_my_shared_network_hosts
+{
+local ($h) = @_;
+local $shar = &get_host_shared_network($h);
+local @rv;
+foreach my $oh (&get_hosts()) {
+	if (&get_host_shared_network($oh) eq $shar) {
+		push(@rv, $oh);
+		}
+	}
+return @rv;
+}
+
 # hash that links objtypes shortcuts with object names
 %obj_names2types = qw(host hst group grp subnet sub shared-network sha);
 
-# get_branch($objtype) 
+# get_branch(objtype, [addmode]) 
 # usefull for edit_*.cgi and save_*.cgi scripts
 # $objtype = one of 'hst' 'grp' 'sub' 'sha'
 sub get_branch
@@ -571,7 +630,9 @@ sub get_branch
 local %obj_types2names = reverse %obj_names2types;
 local $name = $obj_types2names{$_[0]};
 local ($parnode, $nparnode, $node, $indent, $nindent);
-$parnode = $nparnode = &get_parent_config();
+$parnode = $nparnode = &get_parent_config(
+	$_[1] && $in{'sidx'} eq '' && $in{'uidx'} eq '' && $in{'gidx'} eq '' &&
+	$in{'parent'} eq '' ? $config{'add_file'} : undef);
 $indent = $nindent = 0;
 foreach ($in{'sidx'}, $in{'uidx'}, $in{'gidx'}) {
     if ($_ ne '') {
@@ -643,7 +704,8 @@ local $acl;
 local ($perm, $acc, $node, $smode) = @_;
 local @perm = split(//, $perm);
 
-if ($node ne get_parent_config()) {
+if ($node ne get_parent_config() &&
+    $node ne get_parent_config($config{'add_file'})) {
 	foreach (@perm) { 
 		next if ($_ ne 'c') &&  ($_ ne 'r') && ($_ ne 'w');
 		return 0 unless $acc->{$_ . '_' . $obj_names2types{$node->{'name'}} };
@@ -691,7 +753,12 @@ if ($acc->{'per_' . $otype . '_acls'}) {
 		local @opts = &find("option", $node->{'members'});
 		local ($dn) = grep { $_->{'values'}->[0] eq 'domain-name' }
 				   @opts;
-		$name = $dn->{'values'}->[1] if ($dn);
+		if ($dn) {
+			$name = $dn->{'values'}->[1];
+			}
+		else {
+			$name = $node->{'index'};
+			}
 		}
 	local $acl = $acc->{'ACL' . $otype . '_' . $name};
 	foreach (@{$rperm}) {
@@ -769,8 +836,10 @@ sub get_dhcpd_version
 {
 local $out = `$config{'dhcpd_path'} -v 2>&1`;
 ${$_[0]} = $out;
+return $config{'version'} if ($config{'version'});
 return undef if ($out !~ /DHCP/ || $out =~ /V1/);
 return $out =~ /\sV([0-9\.]+)/ ? $1 :
+       $out =~ /\sDHCP\s+Server\s+([0-9\.]+)/ ? $1 :
        $out =~ /-T/ ? 3 : 2;
 }
 
@@ -782,15 +851,10 @@ local $out;
 if ($config{'restart_cmd'}) {
 	# Run the restart script
 	$out = &backquote_logged("$config{'restart_cmd'} 2>&1");
-	return "<pre>$out</pre>" if ($?);
 	}
 else {
 	# Kill and re-run the server
-	local $pid;
-	if (open(PID, $config{'pid_file'})) {
-		chop($pid = <PID>);
-		close(PID);
-		}
+	local $pid = &is_dhcpd_running();
 	$pid && &kill_logged('TERM', $pid) ||
 		return "$text{'restart_errmsg2'} $pid : $!";
 	if ($config{'start_cmd'}) {
@@ -799,11 +863,78 @@ else {
 	else {
 		$out = &backquote_logged("$config{'dhcpd_path'} -cf $config{'dhcpd_conf'} -lf $config{'lease_file'} $config{'interfaces'} 2>&1");
 		}
-	return "<pre>$out</pre>" if ($?);
+	}
+if ($?) {
+	return &parse_error_out($out);
 	}
 return undef;
 }
 
+# Find and add config file lines around those in an error message
+sub parse_error_out
+{
+local ($out) = @_;
+local $conftext;
+if ($out =~ /(\S+)\s+line\s+(\d+):/) {
+	local ($file, $line) = ($1, $2);
+	local $lref = &read_file_lines($file, 1);
+	local $start = $line - 5;
+	local $end = $line + 5;
+	$start = 0 if ($start < 0);
+	$end = @$lref-1 if ($end > @$lref-1);
+	$conftext = &text('restart_conftext', $line, $file)."<br>".
+	    "<pre>".&html_escape(join("\n", @$lref[$start .. $end]))."</pre>";
+	}
+return "<pre>".&html_escape($out)."</pre>".$conftext;
+}
+
+# stop_dhcpd()
+# Stop the running DHCP server. Returns undef on success, or an error message
+# on failure.
+sub stop_dhcpd
+{
+if ($config{'stop_cmd'}) {
+	local $out = &backquote_logged("($config{'stop_cmd'} 2>&1)");
+	return $? ? "<pre>$out</pre>" : undef;
+	}
+else {
+	local $pid = &is_dhcpd_running();
+	if ($pid && &kill_logged('TERM', $pid)) {
+		return undef;
+		}
+	else {
+		return $text{'stop_ekill'};
+		}
+	}
+}
+
+# start_dhcpd()
+# Attempt to start the DHCP server, returning undef on success or an error
+# message on failure.
+sub start_dhcpd
+{
+if (!-r $config{'lease_file'}) {
+	# first time.. need to create the lease file
+	$config{'lease_file'} =~ /^(\S+)\/([^\/]+)$/;
+	if (!-d $1) { mkdir($1, 0755); }
+	open(LEASE, ">$config{'lease_file'}");
+	close(LEASE);
+	}
+local $out;
+if ($config{'start_cmd'}) {
+	$out = &backquote_logged("($config{'start_cmd'} 2>&1)");
+	}
+else {
+	$out = &backquote_logged("$config{'dhcpd_path'} -cf $config{'dhcpd_conf'} -lf $config{'lease_file'} $config{'interfaces'} 2>&1");
+	}
+if ($? || $out =~ /error|failed/i) {
+	return &parse_error_out($out);
+	}
+else {
+	return undef;
+	}
+}
+
 # search_re(value, match)
 sub search_re
 {
@@ -818,4 +949,45 @@ else {
 	}
 }
 
+# get_pid_file()
+# Returns the DHCP server PID file
+sub get_pid_file
+{
+local $conf = &get_config();
+local $file = &find_value("pid-file-name", $conf);
+return $file || $config{'pid_file'};
+}
+
+sub expand_ip_range
+{
+local ($s, $e) = @_;
+local @rs = split(/\./, $s);
+local @re = split(/\./, $e);
+local @rv;
+for(my $i=$rs[0]; $i<=$re[0]; $i++) {
+	for(my $j=$rs[1]; $j<=$re[1]; $j++) {
+		for(my $k=$rs[2]; $k<=$re[2]; $k++) {
+			for(my $l=$rs[3]; $l<=$re[3]; $l++) {
+				push(@rv, "$i.$j.$k.$l");
+				}
+			}
+		}
+	}
+return @rv;
+}
+
+# is_dhcpd_running()
+# Returns the pid if the DHCP server is running
+sub is_dhcpd_running
+{
+local $pidfile = &get_pid_file();
+if ($pidfile) {
+	return &check_pid_file($pidfile);
+	}
+else {
+	local ($pid) = &find_byname("dhcpd");
+	return $pid;
+	}
+}
+
 1;
