@@ -1,7 +1,7 @@
 # log_parser.pl
 # Functions for parsing this module's logs
 
-do './exports-lib.pl';
+do 'exports-lib.pl';
 
 # parse_webmin_log(user, script, action, type, object, &params, [long])
 # Converts logged information from this module into human-readable form
@@ -10,7 +10,10 @@ sub parse_webmin_log
 local ($user, $script, $action, $type, $object, $p, $long) = @_;
 $p->{'host'} = $p->{'host'} ? &html_escape($p->{'host'}) : '*';
 $object = &html_escape($object);
-if ($action eq 'modify') {
+if ($type eq 'exports') {
+	return &text('log_'.$action.'_exports', $object);
+	}
+elsif ($action eq 'modify') {
 	return &text($long ? 'log_modify_l' : 'log_modify',
 		     "<tt>$object</tt>", "<tt>$p->{'host'}</tt>");
 	}
