@@ -1,7 +1,8 @@
 # fetchmail-lib.pl
 # Functions for parsing fetchmail config files
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
 %access = &get_module_acl();
 
@@ -11,15 +12,20 @@ if ($module_info{'usermin'}) {
 		}
 	else {
 		&switch_to_remote_user();
+		&create_user_config_dirs();
 		}
-	$cron_cmd = "$module_config_directory/check.pl";
+	$cron_cmd = "$user_module_config_directory/check.pl";
 	$cron_user = $remote_user;
 	$fetchmail_config = "$remote_user_info[7]/.fetchmailrc";
+	$can_cron = $config{'can_cron'};
+	$can_daemon = $config{'can_daemon'};
 	}
 else {
 	$cron_cmd = "$module_config_directory/check.pl";
 	$cron_user = "root";
 	$fetchmail_config = $config{'config_file'};
+	$can_cron = $access{'cron'};
+	$can_daemon = $access{'daemon'};
 	}
 
 # parse_config_file(file, [&global])
@@ -191,21 +197,23 @@ splice(@$lref, $_[0]->{'line'}, $_[0]->{'eline'} - $_[0]->{'line'} + 1,
 sub poll_lines
 {
 local @rv;
+local $name = $_[0]->{'poll'};
+$name = "\"$name\"" if ($name =~ /[\s:;,]/);
 if ($_[0]->{'skip'}) {
-	push(@rv, "skip $_[0]->{'poll'}");
+	push(@rv, "skip $name");
 	}
 elsif ($_[0]->{'defaults'}) {
-	push(@rv, "defaults $_[0]->{'poll'}");
+	push(@rv, "defaults $name");
 	}
 else {
-	push(@rv, "poll $_[0]->{'poll'}");
+	push(@rv, "poll $name");
 	}
 push(@rv, "\tproto $_[0]->{'proto'}") if ($_[0]->{'proto'});
 push(@rv, "\tauth $_[0]->{'auth'}") if ($_[0]->{'auth'});
 push(@rv, "\tvia $_[0]->{'via'}") if ($_[0]->{'via'});
 push(@rv, "\tport $_[0]->{'port'}") if ($_[0]->{'port'});
 push(@rv, "\ttimeout $_[0]->{'timeout'}") if ($_[0]->{'timeout'});
-push(@rv, "\tinterface $_[0]->{'interface'}") if ($_[0]->{'interface'});
+push(@rv, "\tinterface \"$_[0]->{'interface'}\"") if ($_[0]->{'interface'});
 push(@rv, "\tmonitor $_[0]->{'monitor'}") if ($_[0]->{'monitor'});
 push(@rv, "\t".join(" ", map { /^\S+$/ ? $_ : "\"$_\"" }
 			     @{$_[0]->{'unknown'}})) if (@{$_[0]->{'unknown'}});
@@ -250,10 +258,50 @@ else {
 # get_fetchmail_version([&out])
 sub get_fetchmail_version
 {
-local $out = `$config{'fetchmail_path'} -V 2>&1 </dev/null`;
+local $out = &backquote_command("$config{'fetchmail_path'} -V 2>&1 </dev/null");
 ${$_[0]} = $out if ($_[0]);
 return $out =~ /fetchmail\s+release\s+(\S+)/ ? $1 : undef;
 }
 
+# show_polls(&polls, file, user)
+sub show_polls
+{
+if (@{$_[0]}) {
+	print &ui_columns_start([ $text{'index_poll'},
+				  $text{'index_active'},
+				  $text{'index_proto'},
+				  $text{'index_users'} ], 100);
+	foreach $p (@{$_[0]}) {
+		local @cols;
+		push(@cols, "<a href='edit_poll.cgi?file=$_[1]&".
+		      	    "idx=$p->{'index'}&user=$_[2]'>".
+			    &html_escape($p->{'poll'})."</a>");
+		push(@cols, $p->{'skip'} ?
+		    "<font color=#ff0000>$text{'no'}</font>" : $text{'yes'});
+		push(@cols, $p->{'proto'} ? &html_escape(uc($p->{'proto'}))
+					  : $text{'default'});
+		local $ulist;
+		foreach $u (@{$p->{'users'}}) {
+			$ulist .= sprintf "%s -> %s<br>\n",
+				&html_escape($u->{'user'}),
+				&html_escape(@{$u->{'is'}} ?
+				   join(" ", @{$u->{'is'}}) : $_[2]);
+			}
+		push(@cols, $ulist);
+		print &ui_columns_row(\@cols);
+		}
+	print &ui_columns_end();
+	}
+local @links = (
+  "<a href='edit_poll.cgi?new=1&file=$_[1]&user=$_[2]'>$text{'index_add'}</a>",
+  "<a href='edit_global.cgi?file=$_[1]&user=$_[2]'>$text{'index_global'}</a>"
+	);
+if (@{$_[0]}) {
+	push(@links, "<a href='check.cgi?file=$_[1]&user=$_[2]'>$text{'index_run'}</a>");
+	}
+print &ui_links_row(\@links);
+}
+
+
 1;
 
