@@ -1,7 +1,8 @@
 # export-lib.pl
 # Common functions for exports file
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
 do "hpux-lib.pl";
 %access = &get_module_acl();
@@ -110,7 +111,7 @@ if (!@h) {
         }
 
 foreach (@h) {
-        if (!inet_aton($_)) { &error(&text('save_ehost', $_, $_[0])); }
+        if (!&to_ipaddress($_)) { &error(&text('save_ehost', $_, $_[0])); }
         }
 }
 
