@@ -3,8 +3,17 @@
 # Install a single module file
 
 # Check arguments
+$nodeps = 0;
+if ($ARGV[0] eq "--nodeps") {
+	shift(@ARGV);
+	$nodeps = 1;
+	}
+while($ARGV[0] eq "--acl") {
+	shift(@ARGV);
+	push(@grant, shift(@ARGV));
+	}
 if (@ARGV > 2 || !@ARGV) {
-	die "usage: install-module.pl <module.wbm> [config_directory]";
+	die "usage: install-module.pl [--nodeps] [--acl user]* <module.wbm> [config_directory]";
 	}
 $file = $ARGV[0];
 $config = $ARGV[1] ? $ARGV[1] : "/etc/webmin";
@@ -32,12 +41,19 @@ $ENV{'WEBMIN_VAR'} = $var;
 $no_acl_check++;
 chdir($root);
 $0 = "$root/install-module.pl";
-do './web-lib.pl';
+eval "use WebminCore;";
 &init_config();
 
 # Install it, using the standard function
 &foreign_require("webmin", "webmin-lib.pl");
-$rv = &webmin::install_webmin_module($file, 0, 0, [ ]);
+if (@grant) {
+	$newusers = \@grant;
+	}
+else {
+	$newusers = &webmin::get_newmodule_users();
+	$newusers ||= [ "root", "admin" ];
+	}
+$rv = &webmin::install_webmin_module($file, 0, $nodeps, $newusers);
 if (ref($rv)) {
 	for($i=0; $i<@{$rv->[0]}; $i++) {
 		printf "Installed %s in %s (%d kb)\n",
@@ -47,6 +63,7 @@ if (ref($rv)) {
 		}
 	}
 else {
+	$rv =~ s/<[^>]+>//g;
 	print STDERR "Install failed : $rv\n";
 	}
 
