@@ -1,5 +1,5 @@
 
-require './lpadmin-lib.pl';
+require 'lpadmin-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the lpadmin module
@@ -75,6 +75,20 @@ printf "<td><input type=radio name=test value=1 %s> $text{'yes'}\n",
 	$_[0]->{'test'} ? "checked" : "";
 printf "<input type=radio name=test value=0 %s> $text{'no'}</td> </tr>\n",
 	$_[0]->{'test'} ? "" : "checked";
+
+print "<tr> <td><b>$text{'acl_delete'}</b></td>\n";
+printf "<td><input type=radio name=delete value=1 %s> $text{'yes'}\n",
+	$_[0]->{'delete'} ? "checked" : "";
+printf "<input type=radio name=delete value=0 %s> $text{'no'}</td>\n",
+	$_[0]->{'delete'} ? "" : "checked";
+
+print "<td><b>$text{'acl_cluster'}</b></td>\n";
+printf "<td><input type=radio name=cluster value=1 %s> $text{'yes'}\n",
+	$_[0]->{'cluster'} ? "checked" : "";
+printf "<input type=radio name=cluster value=0 %s> $text{'no'}</td> </tr>\n",
+	$_[0]->{'cluster'} ? "" : "checked";
+
+print "</tr>\n";
 }
 
 # acl_security_save(&options)
@@ -94,5 +108,8 @@ $_[0]->{'stop'} = $in{'stop'};
 $_[0]->{'view'} = $in{'view'};
 $_[0]->{'user'} = $in{'user_def'} == 1 ? '*' :
 		  $in{'user_def'} == 2 ? undef : $in{'user'};
+$_[0]->{'delete'} = $in{'delete'};
+$_[0]->{'test'} = $in{'test'};
+$_[0]->{'cluster'} = $in{'cluster'};
 }
 
