@@ -1,14 +1,15 @@
 # lpadmin-lib.pl
 # Functions for configuring and adding printers
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
-do "./$config{'print_style'}-lib.pl";
+do "$config{'print_style'}-lib.pl";
 if ($config{'driver_style'}) {
-	do "./$config{'driver_style'}-driver.pl";
+	do "$config{'driver_style'}-driver.pl";
 	}
 else {
-	do "./webmin-driver.pl";
+	do "webmin-driver.pl";
 	}
 %access = &get_module_acl();
 
@@ -77,47 +78,45 @@ else {
 	}
 
 # Create script to call GS
-&lock_file($drv);
-open(DRV, ">$drv");
-print DRV "#!/bin/sh\n";
-print DRV "# Name: $_[0]->{'name'}\n";
-print DRV "# Type: ",$_[1]->{'upp'} ? 'uniprint' : $_[1]->{'type'},"\n";
-print DRV "# DPI: ",$_[1]->{'upp'} ? $_[1]->{'upp'} : $_[1]->{'dpi'},"\n";
+&open_lock_tempfile(DRV, ">$drv");
+&print_tempfile(DRV, "#!/bin/sh\n");
+&print_tempfile(DRV, "# Name: $_[0]->{'name'}\n");
+&print_tempfile(DRV, "# Type: ",$_[1]->{'upp'} ? 'uniprint'
+					       : $_[1]->{'type'},"\n");
+&print_tempfile(DRV, "# DPI: ",$_[1]->{'upp'} ? $_[1]->{'upp'}
+					      : $_[1]->{'dpi'},"\n");
 if ($gconfig{'ld_env'}) {
-	print DRV "$gconfig{'ld_env'}=$gconfig{'ld_path'}\n";
+	&print_tempfile(DRV, "$gconfig{'ld_env'}=$gconfig{'ld_path'}\n");
 	}
-print DRV "PATH=$gconfig{'path'}\n";
+&print_tempfile(DRV, "PATH=$gconfig{'path'}\n");
 if ($config{'gs_fontpath'}) {
-	print DRV "GS_FONTPATH=$config{'gs_fontpath'}\n";
+	&print_tempfile(DRV, "GS_FONTPATH=$config{'gs_fontpath'}\n");
 	}
 if ($config{'gs_lib'}) {
-	print DRV "GS_LIB=$config{'gs_lib'}\n";
+	&print_tempfile(DRV, "GS_LIB=$config{'gs_lib'}\n");
 	}
-print DRV "export $gconfig{'ld_env'} PATH GS_FONTPATH GS_LIB\n";
+&print_tempfile(DRV, "export $gconfig{'ld_env'} PATH GS_FONTPATH GS_LIB\n");
 $res = $_[1]->{'upp'} ? "\@$_[1]->{'upp'}.upp" :
        $_[1]->{'dpi'} ? "-r$_[1]->{'dpi'}" : "";
-open(PERL, "$config_directory/perl-path");
-chop($perl = <PERL>);
-close(PERL);
+$perl = &get_perl_path();
 if ($config{'iface_arg'}) {
 	for($i=0; $i<$config{'iface_arg'}-1; $i++) {
-		print DRV "shift\n";
+		&print_tempfile(DRV, "shift\n");
 		}
-	print DRV "cat \$* | $perl -e 'while(<STDIN>) { print if (!/^\\s*#####/); }' >/tmp/\$\$.gsin\n";
+	&print_tempfile(DRV, "cat \$* | $perl -e 'while(<STDIN>) { print if (!/^\\s*#####/); }' >/tmp/\$\$.gsin\n");
 	}
 else {
-	print DRV "$perl -e 'while(<STDIN>) { print if (!/^\\s*#####/); }' >/tmp/\$\$.gsin\n";
+	&print_tempfile(DRV, "$perl -e 'while(<STDIN>) { print if (!/^\\s*#####/); }' >/tmp/\$\$.gsin\n");
 	}
-print DRV "$config{'gs_path'} -sOutputFile=/tmp/\$\$.gs -dSAFER -sDEVICE=$gsdrv $res -dNOPAUSE /tmp/\$\$.gsin </dev/null >/dev/null 2>&1\n";
-print DRV "rm /tmp/\$\$.gsin\n";
-print DRV "cat /tmp/\$\$.gs\n";
-print DRV "rm /tmp/\$\$.gs\n";
-close(DRV);
-&unlock_file($drv);
-&system_logged("chmod $config{'iface_perms'} $drv >/dev/null 2>&1");
+&print_tempfile(DRV, "$config{'gs_path'} -sOutputFile=/tmp/\$\$.gs -dSAFER -sDEVICE=$gsdrv $res -dNOPAUSE /tmp/\$\$.gsin </dev/null >/dev/null 2>&1\n");
+&print_tempfile(DRV, "rm /tmp/\$\$.gsin\n");
+&print_tempfile(DRV, "cat /tmp/\$\$.gs\n");
+&print_tempfile(DRV, "rm /tmp/\$\$.gs\n");
+&close_tempfile(DRV);
 if ($config{'iface_owner'}) {
-	&system_logged("chown $config{'iface_owner'} $drv >/dev/null 2>&1");
+	&system_logged("chown '$config{'iface_owner'}' '$drv' >/dev/null 2>&1");
 	}
+&system_logged("chmod '$config{'iface_perms'}' '$drv' >/dev/null 2>&1");
 return $drv;
 }
 
@@ -133,48 +132,46 @@ mkdir($drivers_directory, 0755);
 $drv = "$drivers_directory/$_[0]->{'name'}.smb";
 
 # Create script to call smbclient
-&lock_file($drv);
-open(DRV, ">$drv");
-print DRV "#!/bin/sh\n";
-print DRV "# Name: $_[0]->{'name'}\n";
-print DRV "# Server: $_[1]->{'server'}\n";
-print DRV "# Share: $_[1]->{'share'}\n";
-print DRV "# User: $_[1]->{'user'}\n";
-print DRV "# Password: $_[1]->{'pass'}\n";
-print DRV "# Workgroup: $_[1]->{'workgroup'}\n";
-print DRV "# Program: $_[1]->{'program'}\n";
+&open_lock_tempfile(DRV, ">$drv");
+&print_tempfile(DRV, "#!/bin/sh\n");
+&print_tempfile(DRV, "# Name: $_[0]->{'name'}\n");
+&print_tempfile(DRV, "# Server: $_[1]->{'server'}\n");
+&print_tempfile(DRV, "# Share: $_[1]->{'share'}\n");
+&print_tempfile(DRV, "# User: $_[1]->{'user'}\n");
+&print_tempfile(DRV, "# Password: $_[1]->{'pass'}\n");
+&print_tempfile(DRV, "# Workgroup: $_[1]->{'workgroup'}\n");
+&print_tempfile(DRV, "# Program: $_[1]->{'program'}\n");
 if ($gconfig{'ld_env'}) {
-	print DRV "$gconfig{'ld_env'}=$gconfig{'ld_path'}\n";
+	&print_tempfile(DRV, "$gconfig{'ld_env'}=$gconfig{'ld_path'}\n");
 	}
-print DRV "PATH=$gconfig{'path'}\n";
-print DRV "export $gconfig{'ld_env'} PATH\n";
+&print_tempfile(DRV, "PATH=$gconfig{'path'}\n");
+&print_tempfile(DRV, "export $gconfig{'ld_env'} PATH\n");
 if (!$_[1]->{'program'}) {
 	if ($config{'iface_arg'}) {
 		for($i=0; $i<$config{'iface_arg'}-1; $i++) {
-			print DRV "shift\n";
+			&print_tempfile(DRV, "shift\n");
 			}
-		print DRV "cat \$* >/tmp/\$\$.smb\n";
+		&print_tempfile(DRV, "cat \$* >/tmp/\$\$.smb\n");
 		}
-	else { print DRV "cat >/tmp/\$\$.smb\n"; }
+	else { &print_tempfile(DRV, "cat >/tmp/\$\$.smb\n"); }
 	}
 else {
-	print DRV "$_[1]->{'program'} \"\$1\" \"\$2\" \"\$3\" \"\$4\" ",
+	&print_tempfile(DRV, "$_[1]->{'program'} \"\$1\" \"\$2\" \"\$3\" \"\$4\" ",
 		  "\"\$5\" \"\$6\" \"\$7\" \"\$8\" \"\$9\" ",
-		  "\"\$10\" \"\$11\" \"\$12\" \"\$13\" >/tmp/\$\$.smb\n";
-	system("chmod a+rx $_[1]->{'program'}");
+		  "\"\$10\" \"\$11\" \"\$12\" \"\$13\" >/tmp/\$\$.smb\n");
+	&system_logged("chmod a+rx '$_[1]->{'program'}'");
 	}
-print DRV "$config{'smbclient_path'} '\\\\$_[1]->{'server'}\\$_[1]->{'share'}' ",
+&print_tempfile(DRV, "$config{'smbclient_path'} '//$_[1]->{'server'}/$_[1]->{'share'}' ",
 	  $_[1]->{'pass'} ? $_[1]->{'pass'} : "-N",
 	  $_[1]->{'user'} ? " -U $_[1]->{'user'}" : "",
 	  $_[1]->{'workgroup'} ? " -W $_[1]->{'workgroup'}" : "",
-	  " -c \"print /tmp/\$\$.smb\" -P\n";
-print DRV "rm /tmp/\$\$.smb\n";
-close(DRV);
-&unlock_file($drv);
-&system_logged("chmod $config{'iface_perms'} $drv >/dev/null 2>&1");
+	  " -c \"print /tmp/\$\$.smb\"\n");
+&print_tempfile(DRV, "rm /tmp/\$\$.smb\n");
+&close_tempfile(DRV);
 if ($config{'iface_owner'}) {
-	&system_logged("chown $config{'iface_owner'} $drv >/dev/null 2>&1");
+	&system_logged("chown '$config{'iface_owner'}' '$drv' >/dev/null 2>&1");
 	}
+&system_logged("chmod '$config{'iface_perms'}' '$drv' >/dev/null 2>&1");
 return $drv;
 }
 
@@ -190,42 +187,41 @@ mkdir($drivers_directory, 0755);
 $drv = "$drivers_directory/$_[0]->{'name'}.hpnp";
 
 # Create script to call hpnp
-&lock_file($drv);
-open(DRV, ">$drv");
-print DRV "#!/bin/sh\n";
-print DRV "# Name: $_[0]->{'name'}\n";
-print DRV "# Server: $_[1]->{'server'}\n";
-print DRV "# Port: $_[1]->{'port'}\n";
-print DRV "# Program: $_[1]->{'program'}\n";
+&open_lock_tempfile(DRV, ">$drv");
+&print_tempfile(DRV, "#!/bin/sh\n");
+&print_tempfile(DRV, "# Name: $_[0]->{'name'}\n");
+&print_tempfile(DRV, "# Server: $_[1]->{'server'}\n");
+&print_tempfile(DRV, "# Port: $_[1]->{'port'}\n");
+&print_tempfile(DRV, "# Program: $_[1]->{'program'}\n");
 if ($gconfig{'ld_env'}) {
-	print DRV "$gconfig{'ld_env'}=$gconfig{'ld_path'}\n";
+	&print_tempfile(DRV, "$gconfig{'ld_env'}=$gconfig{'ld_path'}\n");
 	}
-print DRV "PATH=$gconfig{'path'}\n";
-print DRV "export $gconfig{'ld_env'} PATH\n";
+&print_tempfile(DRV, "PATH=$gconfig{'path'}\n");
+&print_tempfile(DRV, "export $gconfig{'ld_env'} PATH\n");
 if (!$_[1]->{'program'}) {
 	if ($config{'iface_arg'}) {
 		for($i=0; $i<$config{'iface_arg'}-1; $i++) {
-			print DRV "shift\n";
+			&print_tempfile(DRV, "shift\n");
 			}
-		print DRV "cat \$* >/tmp/\$\$.hpnp\n";
+		&print_tempfile(DRV, "cat \$* >/tmp/\$\$.hpnp\n");
 		}
-	else { print DRV "cat >/tmp/\$\$.hpnp\n"; }
+	else { &print_tempfile(DRV, "cat >/tmp/\$\$.hpnp\n"); }
 	}
 else {
-	print DRV "$_[1]->{'program'} \"\$1\" \"\$2\" \"\$3\" \"\$4\" ",
+	&print_tempfile(DRV, "$_[1]->{'program'} \"\$1\" \"\$2\" \"\$3\" \"\$4\" ",
 		  "\"\$5\" \"\$6\" \"\$7\" \"\$8\" \"\$9\" ",
-		  "\"\$10\" \"\$11\" \"\$12\" \"\$13\" >/tmp/\$\$.hpnp\n";
-	system("chmod a+rx $_[1]->{'program'}");
+		  "\"\$10\" \"\$11\" \"\$12\" \"\$13\" >/tmp/\$\$.hpnp\n");
+	&system_logged("chmod a+rx '$_[1]->{'program'}'");
 	}
-print DRV "$config{'hpnp_path'} -x $_[1]->{'server'}",
+&print_tempfile(DRV, "$config{'hpnp_path'} -x $_[1]->{'server'}",
 	  $_[1]->{'port'} ? " -p $_[1]->{'port'}" : "",
-	  " /tmp/\$\$.hpnp\n";
-print DRV "rm /tmp/\$\$.hpnp\n";
-close(DRV);
-&system_logged("chmod $config{'iface_perms'} $drv >/dev/null 2>&1");
+	  " /tmp/\$\$.hpnp\n");
+&print_tempfile(DRV, "rm /tmp/\$\$.hpnp\n");
+&close_tempfile(DRV);
 if ($config{'iface_owner'}) {
 	&system_logged("chown $config{'iface_owner'} $drv >/dev/null 2>&1");
 	}
+&system_logged("chmod '$config{'iface_perms'}' '$drv' >/dev/null 2>&1");
 &unlock_file($drv);
 return $drv;
 }
@@ -343,7 +339,7 @@ printf "<td><input name=iface value=\"%s\" size=35></td> </tr>\n",
 	$drv->{'mode'} == 2 ? $drv->{'prog'} : "";
 
 if (&has_ghostscript()) {
-	local $out = `$config{'gs_path'} -help 2>&1`;
+	local $out = &backquote_command("$config{'gs_path'} -help 2>&1", 1);
 	if ($out =~ /Available devices:\n((\s+.*\n)+)/i) {
 		print "<tr> <td valign=top>\n";
 		printf "<input type=radio name=drv value=1 %s>\n",
@@ -353,9 +349,10 @@ if (&has_ghostscript()) {
 		print "<select name=driver size=7>\n";
 		foreach $d (&list_webmin_drivers()) {
 			if ($drvsupp{$d->[0]}) {
-				printf "<option %s>%s\n",
+				printf "<option value='%s' %s>%s&nbsp;&nbsp;&nbsp;(%s)\n",
+				    $d->[1],
 				    $d->[1] eq $drv->{'type'} ? "selected" : "",
-				    $d->[1];
+				    $d->[1], $d->[0];
 				}
 			}
 		print "</select>&nbsp;&nbsp;";
@@ -405,7 +402,8 @@ if ($in{'drv'} == 0) {
 	return { 'mode' => 0 };
 	}
 elsif ($in{'drv'} == 2) {
-	(-x $in{'iface'}) || &error(&text('webmin_edriver', $in{'iface'}));
+	my @iface = split(/\s+/, $in{'iface'});
+	-x $iface[0] || &error(&text('webmin_edriver', $iface[0]));
 	return { 'mode' => 2,
 		 'program' => $in{'iface'} };
 	}
@@ -426,7 +424,7 @@ elsif ($in{'drv'} == 3) {
 sub list_webmin_drivers
 {
 local(@rv, $_);
-open(DRIVERS, "drivers");
+open(DRIVERS, "$module_root_directory/drivers");
 while(<DRIVERS>) {
 	/^(\S+)\s+(.*)/;
 	push(@rv, [ $1, $2 ]);
@@ -478,7 +476,7 @@ return 0;
 sub list_uniprint
 {
 local (@rv, $f, $d);
-local $out = `$config{'gs_path'} -help 2>&1`;
+local $out = &backquote_command("$config{'gs_path'} -help 2>&1", 1);
 if ($out =~ /Search path:\n((\s+.*\n)+)/i) {
 	foreach $d (split(/\s+/, $1)) {
 		next if ($d !~ /^\//);
@@ -525,3 +523,215 @@ return { 'driver' => $drv->{'desc'},
 		   &dev_name($_[0]->{'dev'}) };
 }
 
+# parse_cups_ppd(file)
+# Converts a CUPS-style .ppd file into a hash of names and values
+sub parse_cups_ppd
+{
+local ($file) = @_;
+local %ppd;
+if ($file =~ /\.gz$/) {
+	open(PPD, "gunzip -c ".quotemeta($file)." |");
+	}
+else {
+	open(PPD, $file);
+	}
+while(<PPD>) {
+	if (/^\s*\*(\S+):\s*"(.*)"/ || /^\s*\*(\S+):\s*(\S+)/) {
+		$ppd{$1} = $2;
+		}
+	elsif (/^\s*\*(\S+)\s+(\S+)\/([^:]+):/) {
+		$ppd{$1}->{$2} = $3 if (!defined($ppd{$1}->{$2}));
+		}
+	}
+close(PPD);
+return \%ppd;
+}
+
+# list_cluster_servers()
+# Returns a list of servers on which printers are managed
+sub list_cluster_servers
+{
+&foreign_require("servers", "servers-lib.pl");
+local %ids = map { $_, 1 } split(/\s+/, $config{'servers'});
+return grep { $ids{$_->{'id'}} } &servers::list_servers();
+}
+
+# add_cluster_server(&server)
+sub add_cluster_server
+{
+local @sids = split(/\s+/, $config{'servers'});
+$config{'servers'} = join(" ", @sids, $_[0]->{'id'});
+&save_module_config();
+}
+
+# delete_cluster_server(&server)
+sub delete_cluster_server
+{
+local @sids = split(/\s+/, $config{'servers'});
+$config{'servers'} = join(" ", grep { $_ != $_[0]->{'id'} } @sids);
+&save_module_config();
+}
+
+# server_name(&server)
+sub server_name
+{
+return $_[0]->{'desc'} ? $_[0]->{'desc'} : $_[0]->{'host'};
+}
+
+# save_printer_cluster(new, &printer, &driver, &connection, webmin-driver, mode)
+# Creates or updates the specified printer on all cluster hosts, and returns a
+# list of error messages
+sub save_on_cluster
+{
+return ( ) if (!$config{'servers'});
+return ( ) if (&is_readonly_mode());
+local ($new, $prn, $drv, $conn, $webmin, $mode) = @_;
+&remote_error_setup(\&slave_error_handler);
+local $slave;
+local @slaveerrs;
+foreach $slave (&list_cluster_servers()) {
+	# Connect to server
+        $slave_error = undef;
+        &remote_foreign_require($slave, "lpadmin", "lpadmin-lib.pl");
+        if ($slave_error) {
+                push(@slaveerrs, [ $slave, $slave_error ]);
+                next;
+                }
+
+	# Create the driver and the printer
+	local $err = &remote_foreign_call($slave,
+		"lpadmin", "save_printer_and_driver",
+		$new, $prn, $drv, $conn, $webmin, $mode);
+	if ($slave_error) {
+                push(@slaveerrs, [ $slave, $slave_error ]);
+		}
+	elsif ($err == 1) {
+		push(@slaveerrs, [ $slave, &text('save_edup', $prn->{'name'}) ]);
+		}
+	elsif ($err == 2) {
+		push(@slaveerrs, [ $slave, $text{'save_evalid'} ]);
+		}
+	elsif ($err == 3) {
+		push(@slaveerrs, [ $slave, &text('save_egone', $prn->{'name'}) ]);
+		}
+	}
+return @slaveerrs;
+}
+
+# delete_on_cluster(&printer)
+# Deletes the specified printer on all cluster hosts, and returns a list of
+# error messages.
+sub delete_on_cluster
+{
+return ( ) if (!$config{'servers'});
+return ( ) if (&is_readonly_mode());
+local ($prn) = @_;
+&remote_error_setup(\&slave_error_handler);
+local $slave;
+local @slaveerrs;
+foreach $slave (&list_cluster_servers()) {
+	# Connect to server
+        $slave_error = undef;
+        &remote_foreign_require($slave, "lpadmin", "lpadmin-lib.pl");
+        if ($slave_error) {
+                push(@slaveerrs, [ $slave, $slave_error ]);
+                next;
+                }
+
+	# Call the delete function
+	local $err = &remote_foreign_call($slave,
+		"lpadmin", "delete_printer_and_driver", $prn);
+	if ($slave_error) {
+                push(@slaveerrs, [ $slave, $slave_error ]);
+		}
+	elsif ($err == 3) {
+		push(@slaveerrs, [ $slave, &text('save_egone', $prn->{'name'}) ]);
+		}
+	}
+return @slaveerrs;
+}
+
+# save_printer_and_driver(new, &printer, &driver, &connection, webmin-driver, mode)
+# Attempts to setup or modify a printer and driver. Returns 0 if OK, 1 if the
+# printer already exists, 2 if some print system error occurred, or 3 if it
+# doesn't exist but should.
+sub save_printer_and_driver
+{
+local ($new, $prn, $drv, $conn, $webmin, $mode) = @_;
+if ($new && &get_printer($prn->{'name'})) {
+	return 1;
+	}
+elsif (!$new && !&get_printer($prn->{'name'})) {
+	return 2;
+	}
+local $dfunc = $webmin ? \&create_webmin_driver : \&create_driver;
+if ($mode <= 2 || $mode == 5) {
+	# Device, file or LPR host
+	$prn->{'iface'} = &$dfunc($prn, $drv);
+	}
+elsif ($mode == 3) {
+	# Windows server
+	$prn->{'dev'} = "/dev/null";
+	$prn->{'iface'} = $webmin ? &create_webmin_windows_driver($prn, $conn)
+				  : &create_windows_driver($prn, $conn);
+	}
+elsif ($mode == 4) {
+	# HPNP server
+	$prn->{'dev'} = "/dev/null";
+	$prn->{'iface'} = &create_hpnp_driver($prn, $conn);
+	}
+
+# Call os-specific validation function
+if (defined(&validate_printer)) {
+	local $err = &validate_printer($prn);
+	return 2 if ($err);
+	}
+
+# Actually create or update it
+if ($new) {
+	&create_printer($prn);
+	}
+else {
+	&modify_printer($prn);
+	}
+&system_logged("$config{'apply_cmd'} >/dev/null 2>&1 </dev/null")
+	if ($config{'apply_cmd'});
+return 0;
+}
+
+# delete_printer_and_driver(&printer)
+# Deletes a printer, returning 1 if it could not be found, or 0 if everything
+# went OK.
+sub delete_printer_and_driver
+{
+local ($prn) = @_;
+&delete_printer($prn->{'name'});
+&delete_driver($prn->{'name'});
+&delete_webmin_driver($prn->{'name'});
+}
+
+sub slave_error_handler
+{
+$slave_error = $_[0];
+}
+
+# delete_from_acls(name)
+# Remove some named printer from all ACLs
+sub delete_from_acls
+{
+local ($name) = @_;
+local $wusers;
+&read_acl(undef, \%wusers);
+foreach my $u (keys %wusers) {
+	my %uaccess = &get_module_acl($u);
+	if ($uaccess{'printers'} ne '*') {
+		$uaccess{'printers'} =
+			join(' ', grep { $_ ne $name }
+				  split(/\s+/, $uaccess{'printers'}));
+		&save_module_acl(\%uaccess, $u);
+		}
+	}
+}
+
+1;
+
