@@ -21,14 +21,15 @@ return &unique(@rv);
 sub get_printer
 {
 local($stat, @rv, $body, $avl, $con, $sys, %prn, @jobs, $_, $out);
-$out = `lpstat -l -p $_[0]`;
-if ($out =~ /^printer\s+(\S+)\s*(.*)\s+(enabled|disabled)\s+since\s+([^\.]*)\.\s+(.*)\.\n([\000-\177]*)$/) {
+local $esc = quotemeta($_[0]);
+$out = `lpstat -l -p $esc`;
+if ($out =~ /^printer\s+(\S+)\s*(.*)\s+(enabled|disabled)\s+since\s+([^\.]*)\.\s+(.*)\.\n([\000-\377]*)$/) {
 	# printer exists
 	$prn{'name'} = $1;
 	$prn{'enabled'} = $3 eq "enabled";
 	$body = $6;
 	}
-elsif ($out =~ /^printer\s+(\S+)\s+waiting for auto-retry.\s+(\S+)\.\n([\000-\177]*)$/) {
+elsif ($out =~ /^printer\s+(\S+)\s+waiting for auto-retry.\s+(\S+)\.\n([\000-\377]*)$/) {
 	# printer has some problem
 	$prn{'name'} = $1;
 	$prn{'enabled'} = 1;
@@ -60,7 +61,7 @@ if ($body =~ /Users (allowed|denied):\n((\s+\S+\n)+)/) {
 
 if (!$_[1]) {
 	# request availability
-	$avl = `lpstat -a $prn{'name'} 2>&1`;
+	$avl = `lpstat -a $esc 2>&1`;
 	if ($avl =~ /^\S+\s+not accepting.*\n\s+(.*)/) {
 		$prn{'accepting'} = 0;
 		$prn{'accepting_why'} = $1;
@@ -72,7 +73,7 @@ if (!$_[1]) {
 	}
 
 # request connection
-$con = `lpstat -v $prn{'name'} 2>&1`;
+$con = `lpstat -v $esc 2>&1`;
 if ($con =~ /^device for \S+:\s+(\S+)/) { $prn{'dev'} = $1; }
 elsif ($con =~ /^system for \S+:\s+(\S+)\s+\(as printer (\S+)\)/) {
 	$prn{'rhost'} = $1;
@@ -96,7 +97,8 @@ local @jobs;
 local $lang = $gconfig{'lang'};
 
 # Added 'env LANG=$lang'
-open(STAT, "env LANG=$lang lpstat -o $_[0] |");
+local $esc = quotemeta($_[0]);
+open(STAT, "env LANG=$lang lpstat -o $esc |");
 while(<STAT>) {
 # original line which does not work
 #	if (/^(\S+-\d+)\s+(\S+)\s+(\d+)\s+(\S+ \d+ \d+:\d+)\s+(.*)/) {
@@ -153,7 +155,7 @@ sub list_classes
 {
 local($stat, %rv);
 $stat = `lpstat -c 2>&1`;
-while($stat =~ /^members of class (\S+):\n((\s+\S+\n)+)([\000-\177]*)$/) {
+while($stat =~ /^members of class (\S+):\n((\s+\S+\n)+)([\000-\377]*)$/) {
 	$stat = $4;
 	$rv{$1} = [ grep { $_ ne "" } split(/\s+/, $2) ];
 	}
@@ -175,23 +177,27 @@ local(%prn, $cmd, $out);
 %prn = %{$_[0]};
 
 # call lpsystem if needed
+local $tesc = quotemeta($prn{'rtype'});
+local $resc = quotemeta($prn{'rhost'});
+local $qesc = quotemeta($prn{'rqueue'});
 if ($prn{'rhost'}) {
-	$out = &backquote_logged(
-		"lpsystem -t $prn{'rtype'} $prn{'rhost'} 2>&1");
+	$out = &backquote_logged("lpsystem -t $tesc $resc 2>&1");
 	if ($?) { &error("lpsystem failed : <pre>$out</pre>"); }
 	}
 
 # call lpadmin
-$cmd = "lpadmin -p $prn{'name'} -D \"$prn{'desc'}\"";
+local $esc = quotemeta($prn{'name'});
+local $desc = quotemeta($prn{'desc'}) || "''";
+$cmd = "lpadmin -p $esc -D $desc";
 if ($prn{'allow_all'}) { $cmd .= " -u allow:all"; }
 elsif ($prn{'deny_all'}) { $cmd .= " -u deny:all"; }
 elsif ($prn{'allow'}) {
-	system("lpadmin -p $prn{'name'} -u deny:all >/dev/null 2>&1");
-	$cmd .= " -u allow:".join(',', @{$prn{'allow'}});
+	&system_logged("lpadmin -p $esc -u deny:all >/dev/null 2>&1");
+	$cmd .= " -u allow:".join(',', map { quotemeta($_) } @{$prn{'allow'}});
 	}
 elsif ($prn{'deny'}) {
-	system("lpadmin -p $prn{'name'} -u allow:all >/dev/null 2>&1");
-	$cmd .= " -u deny:".join(',', @{$prn{'deny'}});
+	&system_logged("lpadmin -p $esc -u allow:all >/dev/null 2>&1");
+	$cmd .= " -u deny:".join(',', map { quotemeta($_) } @{$prn{'deny'}});
 	}
 if ($prn{'dev'}) {
 	$cmd .= " -v $prn{'dev'}";
@@ -199,7 +205,7 @@ if ($prn{'dev'}) {
 	if ($prn{'banner'}) { $cmd .= " -o banner"; }
 	else { $cmd .= " -o nobanner"; }
 	}
-else { $cmd .= " -s $prn{'rhost'}!$prn{'rqueue'}"; }
+else { $cmd .= " -s $resc!$qesc"; }
 @ctype = @{$prn{'ctype'}};
 if (@ctype) { $cmd .= " -I ".join(',' , @ctype); }
 $out = &backquote_logged("$cmd 2>&1");
@@ -207,22 +213,26 @@ if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 
 # make the default
 if ($prn{'default'}) {
-	$out = &backquote_logged("lpadmin -d $prn{'name'} 2>&1");
+	$out = &backquote_logged("lpadmin -d $esc 2>&1");
 	if ($?) { &error("Failed to set default : <pre>$out</pre>"); }
 	}
 
 # enable or disable
-if ($prn{'enabled'}) { $cmd = "enable $prn{'name'}"; }
-elsif ($prn{'enabled_why'})
-	{ $cmd = "enable $prn{'name'} ; disable -r \"$prn{'enabled_why'}\" $prn{'name'}"; }
-else { $cmd = "enable $prn{'name'} ; disable $prn{'name'}"; }
+if ($prn{'enabled'}) { $cmd = "enable $esc"; }
+elsif ($prn{'enabled_why'}) {
+	local $wesc = quotemeta($prn{'enabled_why'});
+	$cmd = "enable $esc ; disable -r $wesc $esc";
+	}
+else { $cmd = "enable $esc ; disable $esc"; }
 $out = &backquote_logged("$cmd 2>&1");
 
 # accepting or rejecting requests
-if ($prn{'accepting'}) { $cmd = "accept $prn{'name'}"; }
-elsif ($prn{'accepting_why'})
-	{ $cmd = "accept $prn{'name'} ; reject -r \"$prn{'accepting_why'}\" $prn{'name'}"; }
-else { $cmd = "accept $prn{'name'} ; reject $prn{'name'}"; }
+if ($prn{'accepting'}) { $cmd = "accept $esc"; }
+elsif ($prn{'accepting_why'}) {
+	local $wesc = quotemeta($prn{'accepting_why'});
+	$cmd = "accept $esc ; reject -r $wesc $esc";
+	}
+else { $cmd = "accept $esc ; reject $esc"; }
 $out = &backquote_logged("$cmd 2>&1");
 }
 
@@ -231,7 +241,8 @@ $out = &backquote_logged("$cmd 2>&1");
 sub delete_printer
 {
 local($out);
-$out = &backquote_logged("lpadmin -x $_[0] 2>&1");
+local $esc = quotemeta($_[0]);
+$out = &backquote_logged("lpadmin -x $esc 2>&1");
 if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 }
 
@@ -240,7 +251,8 @@ if ($?) { &error("lpadmin failed : <pre>$out</pre>"); }
 sub cancel_job
 {
 local($out);
-$out = &backquote_logged("cancel $_[1] 2>&1");
+local $iesc = quotemeta($_[1]);
+$out = &backquote_logged("cancel $iesc 2>&1");
 if ($?) { &error("cancel failed : <pre>$out</pre>"); }
 sleep(1);
 }
@@ -275,7 +287,17 @@ if ($?) { &error("lpshut failed : <tt>$out</tt>"); }
 # Returns the command to print some file on some printer
 sub print_command
 {
-return "lp -d $_[0] $_[1]";
+local $esc = quotemeta($_[0]);
+local $fesc = quotemeta($_[1]);
+return "lp -d $esc $fesc";
+}
+
+# check_print_system()
+sub check_print_system
+{
+&has_command("lpstat") || return &text('unixware_ecmd', "<tt>lpstat</tt>");
+&has_command("/usr/lib/lp/lpsched") || return &text('unixware_ecmd', "<tt>lpsched</tt>");
+return undef;
 }
 
 @device_files = ("/dev/lp0", "/dev/lp1", "/dev/lp2",
