@@ -1,5 +1,5 @@
 
-require './majordomo-lib.pl';
+require 'majordomo-lib.pl';
 
 # acl_security_form(&options)
 # Output HTML for editing security options for the majordomo module
