@@ -1,5 +1,5 @@
 
-do './majordomo-lib.pl';
+do 'majordomo-lib.pl';
 
 sub feedback_files
 {
