@@ -2,9 +2,9 @@
 # Functions for handling the /etc/[v]fstab file. Some functions are defined in
 # here, and some in OS-specific files named <os_type>-lib.pl
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
-do '../ui-lib.pl';
 %access = &get_module_acl();
 $filesystem_users_file = "$module_config_directory/filesystem-users";
 @access_fs = split(/\s+/, $access{'fs'});
@@ -93,21 +93,20 @@ return @rv ? join(",", @rv) : "-";
 # to a creation program, and then redirected back to the original mount cgi
 sub swap_form
 {
+local ($file) = @_;
 &ui_print_header(undef, "Create Swap File", "");
-print "<form action=create_swap.cgi>\n";
-foreach $k (keys %in) {
-	print "<input type=hidden name=\"$k\" value=\"$in{$k}\">\n";
+print &ui_form_start("create_swap.cgi");
+foreach my $k (keys %in) {
+	print &ui_hidden($k, $in{$k});
 	}
-print "<input type=hidden name=cswap_file value=\"$_[0]\">\n";
-print "The swap file <tt>$_[0]</tt> does not exist.<p>\n";
-print "Create and mount a swap file with size <input name=cswap_size size=6>\n";
-print "<select name=cswap_units>\n";
-print "<option value=k> kB\n";
-print "<option value=m selected> MB\n";
-print "<option value=g> GB\n";
-print "</select>\n";
-print "<input type=submit value=\"Create\"></form>\n";
-&ui_print_footer("", "mount list");
+print &ui_hidden("cswap_file", $file);
+print &text('cswap_file', "<tt>$file</tt>"),"<p>\n";
+print $text{'cswap_size'},"\n";
+print &ui_textbox("cswap_size", undef, 6)," ",
+      &ui_select("cswap_units", "m",
+		 [ [ "m", "MB" ], [ "g", "GB" ], [ "t", "TB" ] ])."\n";
+print &ui_form_end([ [ undef, $text{'create'} ] ]);
+&ui_print_footer("", $text{'index_return'});
 exit;
 }
 
@@ -117,8 +116,9 @@ sub nfs_server_chooser_button
 local($form);
 $form = @_ > 1 ? $_[1] : 0;
 if ($access{'browse'}) {
-	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"../$module_name/nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
+	return "<input type=button onClick='ifield = document.forms[$form].$_[0]; nfs_server = window.open(\"../$module_name/nfs_server.cgi\", \"nfs_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); nfs_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
 	}
+return undef;
 }
 
 # nfs_export_chooser_button(serverinput, exportinput, [form])
@@ -127,8 +127,9 @@ sub nfs_export_chooser_button
 local($form);
 $form = @_ > 2 ? $_[2] : 0;
 if ($access{'browse'}) {
-	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"../$module_name/nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
+	return "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; nfs_export = window.open(\"../$module_name/nfs_export.cgi?server=\"+document.forms[$form].$_[0].value, \"nfs_export\", \"toolbar=no,menubar=no,scrollbars=yes,width=500,height=200\"); nfs_export.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
 	}
+return undef;
 }
 
 # smb_server_chooser_button(serverinput, [form])
@@ -137,8 +138,9 @@ sub smb_server_chooser_button
 local($form);
 $form = @_ > 1 ? $_[1] : 0;
 if (&has_command($config{'smbclient_path'}) && $access{'browse'}) {
-	print "<input type=button onClick='ifield = document.forms[$form].$_[0]; smb_server = window.open(\"../$module_name/smb_server.cgi\", \"smb_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
+	return "<input type=button onClick='ifield = document.forms[$form].$_[0]; smb_server = window.open(\"../$module_name/smb_server.cgi\", \"smb_server\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_server.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
 	}
+return undef;
 }
 
 # smb_share_chooser_button(serverinput, shareinput, [form])
@@ -147,8 +149,9 @@ sub smb_share_chooser_button
 local($form);
 $form = @_ > 2 ? $_[2] : 0;
 if (&has_command($config{'smbclient_path'}) && $access{'browse'}) {
-	print "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; smb_share = window.open(\"../$module_name/smb_share.cgi?server=\"+document.forms[$form].$_[0].value, \"smb_share\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_share.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
+	return "<input type=button onClick='if (document.forms[$form].$_[0].value != \"\") { ifield = document.forms[$form].$_[1]; smb_share = window.open(\"../$module_name/smb_share.cgi?server=\"+document.forms[$form].$_[0].value, \"smb_share\", \"toolbar=no,menubar=no,scrollbars=yes,width=400,height=300\"); smb_share.ifield = ifield; window.ifield = ifield }' value=\"...\">\n";
 	}
+return undef;
 }
 
 # Include the correct OS-specific functions file
@@ -204,13 +207,15 @@ if ($re && $arch =~ /^$re$/i && -r "$module_root_directory/$_[0]" &&
 	}
 else {
 	# Need to compile
-	local ($cc) = (&has_command("gcc") || &has_command("cc"));
+	local ($cc) = (&has_command("gcc") || &has_command("cc") ||
+		       &has_command("gcc-4.0") || &has_command("gcc-3.3"));
 	$cc || &error($text{'egcc'});
 	local $out = &backquote_logged("$cc -o $module_config_directory/$_[0] $module_root_directory/$_[0].c 2>&1");
 	if ($?) {
 		&error(&text('ecompile', "<pre>$out</pre>"));
 		}
 	}
+&set_ownership_permissions(undef, undef, 0755,"$module_config_directory/$_[0]");
 }
 
 # get_filesystem_users()
@@ -284,5 +289,45 @@ foreach my $m (&list_mounted()) {
 return ( );
 }
 
+# local_disk_space([&always-count])
+# Returns the total local and free disk space on the system.
+sub local_disk_space
+{
+my ($always) = @_;
+my ($total, $free) = (0, 0);
+my @mounted = &mount::list_mounted();
+my %donezone;
+my %donevzfs;
+my %donedevice;
+foreach $m (@mounted) {
+	if ($m->[2] =~ /^ext/ ||
+	    $m->[2] eq "reiserfs" || $m->[2] eq "ufs" ||
+	    $m->[2] eq "zfs" || $m->[2] eq "simfs" || $m->[2] eq "vzfs" ||
+	    $m->[2] eq "xfs" || $m->[2] eq "jfs" ||
+	    $m->[1] =~ /^\/dev\// ||
+	    &indexof($m->[1], @$always) >= 0) {
+		if ($m->[1] =~ /^zones\/([^\/]+)/ &&
+                    $m->[2] eq "zfs" && $donezone{$1}++) {
+			# Don't double-count zones
+			next;
+			}
+		if ($donedevice{$m->[0]}++) {
+			# Don't double-count mounts from the same device
+			next;
+			}
+		my ($t, $f) = &mount::disk_space($m->[2], $m->[0]);
+		if (($m->[2] eq "simfs" || $m->[2] eq "vzfs" ||
+		     $m->[0] eq "/dev/vzfs") &&
+		    $donevzfs{$t,$f}++) {
+			# Don't double-count VPS filesystems
+			next;
+			}
+		$total += $t*1024;
+		$free += $f*1024;
+		}
+	}
+return ($total, $free);
+}
+
 1;
 
