@@ -191,6 +191,8 @@
 sub theme_header {
 
 local @available = ("webmin", "system", "servers", "cluster", "hardware", "", "net", "kororaweb");
+my $module_name = &get_module_name();
+my %module_info = &get_module_info($module_name);
 
 local $ll;
 local %access = &get_module_acl();
@@ -208,38 +210,16 @@ if ($charset) {
     }
 print "<link rel='icon' href='$gconfig{'webprefix'}/images/webmin_icon.png' type='image/png'>\n";
 if (@_ > 0) {
-    if ($gconfig{'sysinfo'} == 1) {
-        printf "<title>%s : %s on %s (%s %s)</title>\n",
-            $_[0], $remote_user, &get_display_hostname(),
-            $os_type, $os_version;
-        }
-    elsif ($gconfig{'sysinfo'} == 4) {
-        printf "<title>%s on %s (%s %s)</title>\n",
-            $remote_user, &get_display_hostname(),
-            $os_type, $os_version;
-        }
-    else {
-        print "<title>$_[0]</title>\n";
-        }
+    local $title = &get_html_title($_[0]);
+    print "<title>$title</title>\n";
     print $_[7] if ($_[7]);
     if ($gconfig{'sysinfo'} == 0 && $remote_user) {
-        print "<SCRIPT LANGUAGE=\"JavaScript\">\n";
-	print "defaultStatus=\"".&text('header_statusmsg',
-		    ($ENV{'ANONYMOUS_USER'} ? "Anonymous user"
-					   : $remote_user).
-		    ($ENV{'SSL_USER'} ? " (SSL certified)" :
-		     $ENV{'LOCAL_USER'} ? " (Local user)" : ""),
-		    $text{'programname'},
-		    &get_webmin_version(),
-		    &get_display_hostname(),
-		    $os_type.($os_version eq "*" ? "" :" $os_version")).
-		"\";\n";
-        print "</SCRIPT>\n";
+	print &get_html_status_line(0);
         }
     }
 
 @msc_modules = &get_visible_module_infos()
-	if (!defined(@msc_modules));
+	if (!scalar(@msc_modules));
 
 print "</head>\n";
 local $dir = $current_lang_info->{'dir'} ? "dir=\"$current_lang_info->{'dir'}\""
@@ -295,34 +275,13 @@ if (@_ > 1 && !$one && $remote_user && !$notabs) {
   </tr>
 </table>~;
 
-    local %catnames;
-    &read_file("$config_directory/webmin.catnames", \%catnames);
-    foreach $m (@msc_modules) {
-        local $c = $m->{'category'};
-        next if ($cats{$c});
-        if (defined($catnames{$c})) {
-            $cats{$c} = $catnames{$c};
-            }
-        elsif ($text{"category_$c"}) {
-            $cats{$c} = $text{"category_$c"};
-            }
-        else {
-            # try to get category name from module ..
-            local %mtext = &load_language($m->{'dir'});
-            if ($mtext{"category_$c"}) {
-                $cats{$c} = $mtext{"category_$c"};
-                }
-            else {
-                $c = $m->{'category'} = "";
-                $cats{$c} = $text{"category_$c"};
-                }
-            }
-        }
+    # Get categories
+    %cats = &list_categories(\@msc_modules);
     @cats = sort { $b cmp $a } keys %cats;
     $cats = @cats;
     $per = $cats ? 100.0 / $cats : 100;
 
-    if ($theme_index_page) {
+    if ($main::theme_index_page) {
 	    if (!defined($in{'cat'})) {
 	       
 		# Use default category
@@ -437,13 +396,7 @@ if (@_ > 1 && (!$_[5] || $ENV{'HTTP_WEBMIN_SERVERS'})) {
 
    if ($gconfig{'sysinfo'} == 2 && $remote_user) {
 	&tab_start();
-	printf "%s%s logged into %s %s on %s (%s%s)</td>\n",
-		$ENV{'ANONYMOUS_USER'} ? "Anonymous user" : "<tt>$remote_user</tt>",
-		$ENV{'SSL_USER'} ? " (SSL certified)" :
-		$ENV{'LOCAL_USER'} ? " (Local user)" : "",
-		$text{'programname'},
-		$version, "<tt>".&get_display_hostname()."</tt>",
-		$os_type, $os_version eq "*" ? "" : " $os_version";
+	print &get_html_status_line(1);
 	&tab_end();
 	}
    if ($ENV{'HTTP_WEBMIN_SERVERS'}) {
@@ -548,6 +501,8 @@ else {
 
 sub theme_footer {
 local $i;
+my $module_name = &get_module_name();
+my %module_info = &get_module_info($module_name);
 
 if ($theme_no_table) {
 	print "</ul>\n";
@@ -591,25 +546,9 @@ if (!$_[$i]) {
 
 }
 
-sub chop_font {
-
-if (!$current_lang_info->{'titles'} || $gconfig{'texttitles'}) {
-	print $t;
-} else {
-        foreach $l (split(//, &entities_to_ascii($t))) {
-            $ll = ord($l);
-	    local $gif;
-            if ($ll > 127 && $current_lang_info->{'charset'}) {
-		$gif = "$ll.$current_lang_info->{'charset'}.gif";
-		}
-	    else {
-		$gif = "$ll.gif";
-		}
-	    local $sz = $letter_sizes{$gif};
-	    printf "<img src=$gconfig{'webprefix'}/images/letters2/%s width=%d height=%d alt=\"%s\" align=bottom border=0>",
-		$gif, $sz->[0], $sz->[1], $ll eq " " ? "&nbsp;" : $l;
-            }
-	}
+sub chop_font
+{
+print $t;
 }
 
 sub tab_start {
@@ -665,7 +604,28 @@ sub theme_error
 &header($text{'error'}, "");
 print "<br>\n";
 print "<h3>",($main::whatfailed ? "$main::whatfailed : " : ""),@_,"</h3>\n";
-&footer($ENV{'HTTP_REFERER'}, $text{'error_previous'});
+if ($gconfig{'error_stack'}) {
+	# Show call stack (skipping this function!)
+	print "<h3>$text{'error_stack'}</h3>\n";
+	print "<table>\n";
+	print "<tr> <td><b>$text{'error_file'}</b></td> ",
+	      "<td><b>$text{'error_line'}</b></td> ",
+	      "<td><b>$text{'error_sub'}</b></td> </tr>\n";
+	for($i=1; my @stack = caller($i); $i++) {
+		print "<tr>\n";
+		print "<td>$stack[1]</td>\n";
+		print "<td>$stack[2]</td>\n";
+		print "<td>$stack[3]</td>\n";
+		print "</tr>\n";
+		}
+	print "</table>\n";
+	}
+if ($ENV{'HTTP_REFERER'} && $main::completed_referers_check) {
+	&footer($ENV{'HTTP_REFERER'}, $text{'error_previous'});
+	}
+else {
+	&footer();
+	}
 }
 
 sub theme_ui_print_unbuffered_header
