@@ -11,7 +11,7 @@ local(@rv, @lines, $l);
 local @boot = &boot_interfaces();
 local %boot = map { $_->{'address'}, $_ } @boot;
 local %bootname = map { $_->{'fullname'}, $_ } @boot;
-&open_execute_command(IFC, "ifconfig -a", 1);
+&open_execute_command(IFC, "ifconfig -a", 1, 1);
 while(<IFC>) {
 	s/\r|\n//g;
 	if (/^\S+:/) { push(@lines, $_); }
@@ -43,7 +43,7 @@ foreach $l (@lines) {
 	$ifc{'edit'} = &iface_type($ifc{'name'}) =~ /ethernet|loopback/i;
 	$ifc{'index'} = scalar(@rv);
 	if ($ifc{'ether'}) {
-		$ifc{'ether'} = join(":", map { sprintf "%2.2d", $_ }
+		$ifc{'ether'} = join(":", map { length($_) == 1 ? "0".$_ : $_ }
 					      split(/:/, $ifc{'ether'}));
 		}
 	push(@rv, \%ifc);
@@ -53,11 +53,11 @@ foreach $l (@lines) {
 	local %vtaken = map { $_->{'virtual'}, 1 }
 			    grep { $_->{'name'} eq $vifc{'name'} &&
 				   $_->{'virtual'} ne "" } @boot;
-	while($l =~ s/inet\s+(\S+)\s+netmask\s+(\S+)\s+broadcast\s+(\S+)//) {
+	while($l =~ s/inet\s+(\S+)\s+netmask\s+(\S+)(\s+broadcast\s+(\S+))?//) {
 		local %vifc = %ifc;
 		$vifc{'address'} = $1;
 		$vifc{'netmask'} = &parse_hex($2);
-		$vifc{'broadcast'} = $3;
+		$vifc{'broadcast'} = $4;
 		$vifc{'up'} = 1;
 		$vifc{'edit'} = $ifc{'edit'};
 		local $boot = $boot{$vifc{'address'}};
@@ -280,8 +280,9 @@ return	$_[0] =~ /^tun/ ? "Loopback tunnel" :
 	$_[0] =~ /^ppp/ ? "PPP" :
 	$_[0] =~ /^lo/ ? "Loopback" :
 	$_[0] =~ /^ar/ ? "Arnet" :
-	$_[0] =~ /^(bge|em)/ ? "Gigabit ethernet" :
-	$_[0] =~ /^(ax|mx|pn|rl|tx|wb)/ ? "Fast ethernet" :
+	$_[0] =~ /^(wlan|athi|ral)/ ? "Wireless ethernet" :
+	$_[0] =~ /^(bge|em|myk)/ ? "Gigabit ethernet" :
+	$_[0] =~ /^(ax|mx|nve|pn|rl|tx|wb|nfe|sis)/ ? "Fast ethernet" :
 	$_[0] =~ /^(cs|dc|de|ed|el|ex|fe|fxp|ie|le|lnc|tl|vr|vx|xl|ze|zp)/ ? "Ethernet" : $text{'ifcs_unknown'};
 }
 
@@ -327,19 +328,38 @@ while(<RESOLV>) {
 	}
 close(RESOLV);
 
-&open_readfile(HOST, "/etc/host.conf");
-while(<HOST>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	push(@{$dns->{'order'}}, $_) if (/\S/);
+local @order;
+local $orderfile;
+if (-r "/etc/nsswitch.conf") {
+	# FreeBSD 5.0 and later use nsswitch.conf
+	$orderfile = "/etc/nsswitch.conf";
+	&open_readfile(SWITCH, $orderfile);
+	while(<SWITCH>) {
+		s/\r|\n//g;
+		if (/^\s*hosts:\s+(.*)/) {
+			$dns->{'order'} = $1;
+			}
+		}
+	close(SWITCH);
 	}
-close(HOST);
-$dns->{'files'} = [ "/etc/resolv.conf", "/etc/host.conf" ];
+else {
+	# Older versions use host.conf
+	$orderfile = "/etc/host.conf";
+	&open_readfile(HOST, $orderfile);
+	while(<HOST>) {
+		s/\r|\n//g;
+		s/#.*$//;
+		push(@order, $_) if (/\S/);
+		}
+	close(HOST);
+	$dns->{'order'} = join(" ", @order);
+	}
+$dns->{'files'} = [ "/etc/resolv.conf", $orderfile ];
 return $dns;
 }
 
 # save_dns_config(&config)
-# Writes out the resolv.conf and nsswitch.conf files
+# Writes out the resolv.conf and host.conf files
 sub save_dns_config
 {
 &lock_file("/etc/resolv.conf");
@@ -364,11 +384,32 @@ foreach (@resolv) {
 &close_tempfile(RESOLV);
 &unlock_file("/etc/resolv.conf");
 
-&open_lock_tempfile(HOST, ">/etc/host.conf");
-foreach (@{$_[0]->{'order'}}) {
-	&print_tempfile(HOST, $_,"\n");
+if (-r "/etc/nsswitch.conf") {
+	# Save to new nsswitch.conf, for FreeBSD 5.0 and later
+	&lock_file("/etc/nsswitch.conf");
+	&open_readfile(SWITCH, "/etc/nsswitch.conf");
+	local @switch = <SWITCH>;
+	close(SWITCH);
+	&open_tempfile(SWITCH, ">/etc/nsswitch.conf");
+	foreach (@switch) {
+		if (/^\s*hosts:\s+/) {
+			&print_tempfile(SWITCH, "hosts:\t$_[0]->{'order'}\n");
+			}
+		else {
+			&print_tempfile(SWITCH, $_);
+			}
+		}
+	&close_tempfile(SWITCH);
+	&unlock_file("/etc/nsswitch.conf");
+	}
+else {
+	# Save to older host.conf
+	&open_lock_tempfile(HOST, ">/etc/host.conf");
+	foreach my $o (split(/\s+/, $_[0]->{'order'})) {
+		&print_tempfile(HOST, $o,"\n");
+		}
+	&close_tempfile(HOST);
 	}
-&close_tempfile(HOST);
 }
 
 $max_dns_servers = 3;
@@ -377,38 +418,44 @@ $max_dns_servers = 3;
 # Returns HTML for selecting the name resolution order
 sub order_input
 {
-local @o = @{$_[0]->{'order'}};
-local ($rv, $i, $j);
-local @srcs = ( "", "hosts", "bind", "nis" );
-local @srcn = ( "", "Hosts", "DNS", "NIS" );
-for($i=1; $i<@srcs; $i++) {
-	local $ii = $i-1;
-	$rv .= "<select name=order_$ii>\n";
-	for($j=0; $j<@srcs; $j++) {
-		$rv .= sprintf "<option value=\"%s\" %s>%s\n",
-				$srcs[$j],
-				$o[$ii] eq $srcs[$j] ? "selected" : "",
-				$srcn[$j] ? $srcn[$j] : "&nbsp;";
-		}
-	$rv .= "</select>\n";
+if (-r "/etc/nsswitch.conf") {
+	# FreeBSD 5.0 and later use nsswitch.conf with more options
+	return &common_order_input("order", $_[0]->{'order'},
+		[ [ "files", "Files" ], [ "dns", "DNS" ],
+		  [ "nis", "NIS" ], [ "cache", "NSCD" ] ]);
+	}
+else {
+	# Older FreeBSD's have fewer options
+	local $dnsopt = $_[0]->{'order'} =~ /dns/ ? 'dns' : 'bind';
+	return &common_order_input("order", $_[0]->{'order'},
+		[ [ "hosts", "Hosts" ], [ $dnsopt, "DNS" ], [ "nis", "NIS" ] ]);
 	}
-return $rv;
 }
 
 # parse_order(&dns)
 # Parses the form created by order_input()
 sub parse_order
 {
-local($i, @order);
-for($i=0; defined($in{"order_$i"}); $i++) {
-	push(@order, $in{"order_$i"}) if ($in{"order_$i"});
-	}
-$_[0]->{'order'} = \@order;
+if (defined($in{'order'})) {
+        $in{'order'} =~ /\S/ || &error($text{'dns_eorder'});
+        $_[0]->{'order'} = $in{'order'};
+        }
+else {
+        local($i, @order);
+        for($i=0; defined($in{"order_$i"}); $i++) {
+                push(@order, $in{"order_$i"}) if ($in{"order_$i"});
+                }
+        $_[0]->{'order'} = join(" ", @order);
+        }
 }
 
 # get_hostname()
 sub get_hostname
 {
+local %rc = &get_rc_conf();
+if ($rc{'hostname'}) {
+	return $rc{'hostname'};
+	}
 return &get_system_hostname();
 }
 
@@ -419,6 +466,7 @@ sub save_hostname
 &system_logged("hostname $_[0] >/dev/null 2>&1");
 &save_rc_conf('hostname', $_[0]);
 &unlock_file("/etc/rc.conf");
+undef(@main::get_system_hostname);      # clear cache
 }
 
 sub routing_config_files
@@ -429,28 +477,24 @@ return ( "/etc/defaults/rc.conf", "/etc/rc.conf" );
 sub routing_input
 {
 local %rc = &get_rc_conf();
+
+# Default router
 local $defr = $rc{'defaultrouter'};
-print "<tr> <td><b>$text{'routes_default'}</b></td> <td>\n";
-printf "<input name=defr_def type=radio value=1 %s> %s\n",
-	$defr eq 'NO' ? 'checked' : '', $text{'routes_none'};
-printf "<input name=defr_def type=radio value=0 %s>\n",
-	$defr eq 'NO' ? '' : 'checked';
-printf "<input name=defr size=20 value='%s'></td> </tr>\n",
-	$defr eq 'NO' ? '' : $defr;
+print &ui_table_row($text{'routes_default'},
+	&ui_opt_textbox("defr", $defr eq 'NO' ? '' : $defr, 20,
+			$text{'routes_none'}));
 
+# Act as router?
 local $gw = $rc{'gateway_enable'};
-print "<tr> <td><b>$text{'routes_forward'}</b></td> <td>\n";
-printf "<input name=gw type=radio value=YES %s> %s\n",
-	$gw eq 'YES' ? 'checked' : '', $text{'yes'};
-printf "<input name=gw type=radio value=NO %s> %s</td> </tr>\n",
-	$gw eq 'YES' ? '' : 'checked', $text{'no'};
+print &ui_table_row($text{'routes_forward'},
+	&ui_radio("gw", $gw || 'NO', [ [ 'YES', $text{'yes'} ],
+				       [ 'NO', $text{'no'} ] ]));
 
+# Run route discovery
 local $rd = $rc{'router_enable'};
-print "<tr> <td><b>$text{'routes_routed'}</b></td> <td>\n";
-printf "<input name=rd type=radio value=YES %s> %s\n",
-	$rd eq 'YES' ? 'checked' : '', $text{'yes'};
-printf "<input name=rd type=radio value=NO %s> %s</td> </tr>\n",
-	$rd eq 'YES' ? '' : 'checked', $text{'no'};
+print &ui_table_row($text{'routes_routed'},
+	&ui_radio("rd", $rd || 'NO', [ [ 'YES', $text{'yes'} ],
+				       [ 'NO', $text{'no'} ] ]));
 }
 
 sub parse_routing
@@ -491,7 +535,9 @@ if (!$found && @_ > 1) {
 sub get_rc_conf
 {
 local ($file, %rv);
-foreach $file ("/etc/defaults/rc.conf", "/etc/rc.conf") {
+foreach $file ("/etc/defaults/rc.conf",
+	       glob("/etc/rc.conf.d/*"),
+	       "/etc/rc.conf") {
 	&open_readfile(FILE, $file);
 	while(<FILE>) {
 		s/\r|\n//g;
@@ -510,7 +556,19 @@ return %rv;
 # Apply the interface and routing settings
 sub apply_network
 {
-&system_logged("(cd / ; /etc/netstart) >/dev/null 2>&1");
+local $oldpwd = &get_current_dir();
+chdir("/");
+
+# Take down all active alias interfaces, and any that no longer exist
+local %boot = map { $_->{'fullname'}, $_ } &boot_interfaces();
+foreach my $i (&active_interfaces()) {
+	if ($i->{'virtual'} ne '' || !$boot{$i->{'fullname'}}) {
+		&deactivate_interface($i);
+		}
+	}
+# Bring everything up
+&system_logged("/etc/netstart >/dev/null 2>&1");
+chdir($oldpwd);
 }
 
 sub os_feedback_files
@@ -519,5 +577,13 @@ return ( "/etc/rc.conf", "/etc/resolv.conf", "/etc/host.conf",
 	 "/etc/resolv.conf" );
 }
 
+# supports_address6([&iface])
+# Returns 1 if managing IPv6 interfaces is supported
+sub supports_address6
+{
+local ($iface) = @_;
+return 0;
+}
+
 1;
 
