@@ -1,7 +1,8 @@
 # nis-lib.pl
 # Common functions for NIS client and server management
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
 if (-r "$module_root_directory/$gconfig{'os_type'}-$gconfig{'os_version'}-lib.pl") {
 	do "$gconfig{'os_type'}-$gconfig{'os_version'}-lib.pl";
@@ -9,6 +10,9 @@ if (-r "$module_root_directory/$gconfig{'os_type'}-$gconfig{'os_version'}-lib.pl
 else {
 	do "$gconfig{'os_type'}-lib.pl";
 	}
+if ($gconfig{'os_type'} =~ /-linux$/) {
+	do "linux-lib.pl";
+	}
 &foreign_require("init", "init-lib.pl");
 
 # init_script(action)
@@ -147,8 +151,18 @@ while(<CONF>) {
 			      'value' => $2 eq 'yes' ? 1 : 0,
 			      'line' => $lnum };
 		}
+	elsif (/^\s*([^:\s]+)\s*:\s*([^:\s]+)\s*:\s*([^:\s]+)\s*:\s*(none|port|mangle|deny)(\/mangle(:(\d+))?)?/) {
+		# Found a host and domain line (new format)
+		push(@hosts, { 'host' => $1,
+			       'domain' => $2,
+			       'map' => $3,
+			       'sec' => $4,
+			       'mangle' => $5 ? 1 : 0,
+			       'field' => $7,
+			       'line' => $lnum } );
+		}
 	elsif (/^\s*([^:\s]+)\s*:\s*([^:\s]+)\s*:\s*([^:\s]+)(\s*:\s*([^:\s]+))?(\s*:\s*([^:\s]+))?/) {
-		# Found a host line
+		# Found a host line (old format)
 		push(@hosts, { 'host' => $1,
 			       'map' => $2,
 			       'sec' => $3,
@@ -199,6 +213,12 @@ for($i=0; $i<@lines; $i++) {
 			     'line' => $lines[$i]->{'line'},
 			     'eline' => $lines[$i]->{'eline'} };
 		}
+	elsif ($lines[$i]->{'value'} =~ /^\s*(\S+)\s*\+=\s*(.*)/) {
+		# Adding to a variable
+		if ($var{$1}) {
+			$var{$1}->{'value'} .= ' '.$2;
+			}
+		}
 	elsif ($lines[$i]->{'value'} =~ /^\s*(\S+):\s*(.*)/) {
 		# Found a makefile rule
 		$rule{$1} = { 'name' => $1,
