@@ -38,19 +38,23 @@ printf "<input name=groups size=20 value='%s'> %s<br>\n",
 	&group_chooser_button("groups", 1);
 printf "%s <input type=checkbox name=sec value=1 %s> %s<br>\n",
         "&nbsp;" x 5, $_[0]->{'sec'} ? 'checked' : '',$text{'acl_sec'};
+printf "%s %s <input name=notusers size=20 value='%s'> %s<br>\n",
+	"&nbsp;" x 5, $text{'acl_notusers'},
+	$_[0]->{'mode'} == 5 ? $_[0]->{'notusers'} : '',
+	&user_chooser_button("notusers", 1);
 
 printf "<input type=radio name=mode value=6 %s> %s\n",
 	$_[0]->{'mode'} == 6 ? 'checked' : '', $text{'acl_mode6'};
 printf "<input name=match size=15 value='%s'></td> </tr>\n",
 	$_[0]->{'mode'} == 6 ? $_[0]->{'users'} : '';
 
-print "<tr> <td><b>$text{'acl_repeat'}</b></td> <td>\n";
+print "<tr> <td><b>$text{'acl_repeat'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=repeat value=1 %s> $text{'yes'}\n",
 	$_[0]->{'repeat'} ? "checked" : "";
-printf "<input type=radio name=repeat value=0 %s> $text{'no'}</td>\n",
+printf "<input type=radio name=repeat value=0 %s> $text{'no'}</td> </tr>\n",
 	$_[0]->{'repeat'} ? "" : "checked";
 
-print "<td><b>$text{'acl_expire'}</b></td> <td>\n";
+print "<tr> <td><b>$text{'acl_expire'}</b></td> <td colspan=3>\n";
 printf "<input type=radio name=expire value=1 %s> $text{'yes'}\n",
 	$_[0]->{'expire'} ? "checked" : "";
 printf "<input type=radio name=expire value=0 %s> $text{'no'}</td> </tr>\n",
@@ -82,6 +86,7 @@ $_[0]->{'users'} = $in{'mode'} == 1 ? $in{'users1'} :
 		   $in{'mode'} == 2 ? $in{'users2'} :
 		   $in{'mode'} == 5 ? $in{'groups'} :
 		   $in{'mode'} == 6 ? $in{'match'} : undef;
+$_[0]->{'notusers'} = $in{'mode'} == 5 ? $in{'notusers'} : undef;
 $_[0]->{'low'} = $in{'low'};
 $_[0]->{'high'} = $in{'high'};
 $_[0]->{'repeat'} = $in{'repeat'};
