@@ -3,18 +3,29 @@
 
 use Time::Local;
 
+sub get_ps_version
+{
+if (!$get_ps_version_cache) {
+	local $out = &backquote_command("ps V 2>&1");
+	if ($out =~ /version\s+([0-9\.]+)\./) {
+		$get_ps_version_cache = $1;
+		}
+	}
+return $get_ps_version_cache;
+}
+
 sub list_processes
 {
 local($pcmd, $line, $i, %pidmap, @plist, $dummy, @w, $_);
-local $out = `ps V`;
-if ($out =~ /version\s+(\d+)\./ && $1 >= 2 || $out =~ /version\s+\./) {
+local $ver = &get_ps_version();
+if ($ver >= 2) {
 	# New version of ps, as found in redhat 6
 	local $width;
-	if ($1 >= 3.2) {
+	if ($ver >= 3.2) {
 		# Use width format character if allowed
-		$width = ":100";
+		$width = ":80";
 		}
-	open(PS, "ps --cols 500 -eo user$width,ruser$width,group$width,rgroup$width,pid,ppid,pgid,pcpu,vsz,nice,etime,time,stime,tty,args 2>/dev/null |");
+	open(PS, "ps --cols 2048 -eo user$width,ruser$width,group$width,rgroup$width,pid,ppid,pgid,pcpu,vsz,nice,etime,time,stime,tty,args 2>/dev/null |");
 	$dummy = <PS>;
 	for($i=0; $line=<PS>; $i++) {
 		chop($line);
@@ -24,7 +35,7 @@ if ($out =~ /version\s+(\d+)\./ && $1 >= 2 || $out =~ /version\s+\./) {
 			# Hit a split loop
 			$i--; next;
 			}
-		if ($line =~ /ps -o user,ruser/) {
+		if ($line =~ /ps --cols 500 -eo user/) {
 			# Skip process ID 0 or ps command
 			$i--; next;
 			}
@@ -45,14 +56,14 @@ if ($out =~ /version\s+(\d+)\./ && $1 >= 2 || $out =~ /version\s+\./) {
 		$plist[$i]->{"_ruser"} = $w[1];
 		$plist[$i]->{"_rgroup"} = $w[3];
 		$plist[$i]->{"_pgid"} = $w[6];
-		$plist[$i]->{"_tty"} = $w[13] =~ /\?/ ? $text{'edit_none'} : "/dev/$w[12]";
+		$plist[$i]->{"_tty"} = $w[13] =~ /\?/ ? $text{'edit_none'} : "/dev/$w[13]";
 		}
 	close(PS);
 	}
 else {
 	# Old version of ps
 	$pcmd = join(' ' , @_);
-	open(PS, "ps aulxhwwww $pcmd |");
+	open(PS, "ps aulxhwwww $pcmd 2>/dev/nul |");
 	for($i=0; $line=<PS>; $i++) {
 		chop($line);
 		if ($line =~ /ps aulxhwwww/) { $i--; next; }
@@ -95,7 +106,7 @@ return @plist;
 sub renice_proc
 {
 return undef if (&is_readonly_mode());
-$out = `renice $_[1] -p $_[0] 2>&1`;
+local $out = &backquote_logged("renice $_[1] -p $_[0] 2>&1");
 if ($?) { return $out; }
 return undef;
 }
@@ -105,7 +116,7 @@ return undef;
 sub find_mount_processes
 {
 local($out);
-$out = `fuser -m $_[0]`;
+$out = &backquote_command("fuser -m ".quotemeta($_[0])." 2>/dev/null");
 $out =~ s/[^0-9 ]//g;
 $out =~ s/^\s+//g; $out =~ s/\s+$//g;
 return split(/\s+/, $out);
@@ -116,8 +127,8 @@ return split(/\s+/, $out);
 sub find_file_processes
 {
 local($out, $files);
-$files = join(' ', @_);
-$out = `fuser $files`;
+$files = join(' ', map { quotemeta($_) } map { glob($_) } @_);
+$out = &backquote_command("fuser $files 2>/dev/null");
 $out =~ s/[^0-9 ]//g;
 $out =~ s/^\s+//g; $out =~ s/\s+$//g;
 return split(/\s+/, $out);
@@ -212,9 +223,20 @@ ioctl($ttyfh, 0x540e, 0);
 
 # get_memory_info()
 # Returns a list containing the real mem, free real mem, swap and free swap
+# (In kilobytes).
 sub get_memory_info
 {
 local %m;
+if (open(BEAN, "/proc/user_beancounters")) {
+	# If we are running under Virtuozzo, there may be a limit on memory
+	# use in force that is less than the real system's memory.
+	while(<BEAN>) {
+		if (/^privvmpages\s+(\d+)\s+(\d+)\s+(\d+)/) {
+			return ($3, $3-$1, undef, undef);
+			}
+		}
+	close(BEAN);
+	}
 open(MEMINFO, "/proc/meminfo") || return ();
 while(<MEMINFO>) {
 	if (/^(\S+):\s+(\d+)/) {
@@ -222,18 +244,42 @@ while(<MEMINFO>) {
 		}
 	}
 close(MEMINFO);
-return ( $m{'memtotal'}, $m{'memfree'}+$m{'buffers'}+$m{'cached'},
+return ( $m{'memtotal'}, $m{'cached'} > $m{'memtotal'} ? $m{'memfree'}
+				: $m{'memfree'}+$m{'buffers'}+$m{'cached'},
 	 $m{'swaptotal'}, $m{'swapfree'} );
 }
 
 # os_get_cpu_info()
-# Returns a list containing the 5, 10 and 15 minute load averages
+# Returns a list containing the 5, 10 and 15 minute load averages, and the
+# CPU mhz, model, vendor, cache and count
 sub os_get_cpu_info
 {
 open(LOAD, "/proc/loadavg") || return ();
 local @load = split(/\s+/, <LOAD>);
 close(LOAD);
-return ( $load[0], $load[1], $load[2] );
+local %c;
+open(CPUINFO, "/proc/cpuinfo");
+while(<CPUINFO>) {
+	if (/^(\S[^:]*\S)\s*:\s*(.*)/) {
+		$c{lc($1)} = $2;
+		}
+	}
+close(CPUINFO);
+$c{'model name'} =~ s/\d+\s*mhz//i;
+if ($c{'cache size'} =~ /^(\d+)\s+KB/i) {
+	$c{'cache size'} = $1*1024;
+	}
+elsif ($c{'cache size'} =~ /^(\d+)\s+MB/i) {
+	$c{'cache size'} = $1*1024*1024;
+	}
+if ($c{'cpu mhz'}) {
+	return ( $load[0], $load[1], $load[2],
+		 int($c{'cpu mhz'}), $c{'model name'}, $c{'vendor_id'},
+		 $c{'cache size'}, $c{'processor'}+1 );
+	}
+else {
+	return ( $load[0], $load[1], $load[2] );
+	}
 }
 
 $has_trace_command = &has_command("strace");
@@ -331,5 +377,54 @@ foreach $ia (keys %text) {
 
 $has_fuser_command = 1;
 
+# os_list_scheduling_classes()
+# Returns a list of Linux scheduling classes, if supported. Each element is a
+# 2-element array ref containing a code and description.
+sub os_list_scheduling_classes
+{
+if (&has_command("ionice")) {
+	return ( [ 1, $text{'linux_real'} ],
+		 [ 2, $text{'linux_be'} ],
+		 [ 3, $text{'linux_idle'} ] );
+	}
+return ( );
+}
+
+# os_list_scheduling_priorities()
+# Returns a list of IO priorities, each of which is an array ref containing
+# a number and description
+sub os_list_scheduling_priorities
+{
+return ( [ 0, "0 ($text{'edit_prihigh'})" ],
+	 [ 1 ], [ 2 ], [ 3 ], [ 4 ], [ 5 ], [ 6 ],
+	 [ 7, "7 ($text{'edit_prilow'})" ] );
+}
+
+# os_get_scheduling_class(pid)
+# Returns the IO scheduling class and priority for a running program
+sub os_get_scheduling_class
+{
+local ($pid) = @_;
+local $out = &backquote_command("ionice -p ".quotemeta($pid));
+if ($out =~ /^(realtime|best-effort|idle|none):\s+prio\s+(\d+)/) {
+	return ($1 eq "realtime" ? 1 : $1 eq "best-effort" ? 2 :
+		$1 eq "idle" ? 3 : 0, $2);
+	}
+return ( );
+}
+
+# os_set_scheduling_class(pid, class, priority)
+# Sets the ID scheduling class and priority for some process. Returns an error
+# message on failure, undef on success.
+sub os_set_scheduling_class
+{
+local ($pid, $class, $prio) = @_;
+local $cmd = "ionice -c ".quotemeta($class);
+$cmd .= " -n ".quotemeta($prio) if (defined($prio));
+$cmd .= " -p ".quotemeta($pid);
+local $out = &backquote_logged("$cmd 2>&1 </dev/null");
+return $? ? $out : undef;
+}
+
 1;
 
