@@ -1,15 +1,16 @@
 # proc-lib.pl
 # Functions for managing processes
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
+use POSIX;
+use Config;
+
 &init_config();
-do '../ui-lib.pl';
 if ($module_info{'usermin'} && !$ENV{'FOREIGN_MODULE_NAME'}) {
 	&switch_to_remote_user();
 	}
 do "$config{ps_style}-lib.pl";
-use POSIX;
-use Config;
 if ($module_info{'usermin'}) {
 	%access = ( 'edit' => 1,
 		    'run' => 1,
@@ -60,17 +61,20 @@ return @plist ? %{$plist[0]} : ();
 sub index_links
 {
 local(%linkname, $l);
-print "<b>$text{'index_display'} : </b>&nbsp;&nbsp;\n";
+print "<b>$text{'index_display'} : </b>\n";
+local @links;
 foreach $l ("tree", "user", "size", "cpu", ($has_zone ? ("zone") : ()),
 	    "search", "run") {
 	next if ($l eq "run" && !$access{'run'});
-	if ($l ne $_[0]) { print "<a href=index_$l.cgi>"; }
-	else { print "<b>"; }
-	print $text{"index_$l"};
-	if ($l ne $_[0]) { print "</a>"; }
-	else { print "</b>"; }
-	print "&nbsp;\n";
+	local $link;
+	if ($l ne $_[0]) { $link .= "<a href=index_$l.cgi>"; }
+	else { $link .= "<b>"; }
+	$link .= $text{"index_$l"};
+	if ($l ne $_[0]) { $link .= "</a>"; }
+	else { $link .= "</b>"; }
+	push(@links, $link);
 	}
+print &ui_links_row(\@links);
 print "<p>\n";
 &create_user_config_dirs();
 open(INDEX, ">$index_file");
@@ -96,27 +100,27 @@ return if ($module_info{'usermin'});	# already switched!
 if ($access{'uid'} < 0) {
 	local @u = getpwnam($remote_user);
 	@u || &error("Failed to find user $remote_user");
-	$( = $u[3]; $) = "$u[3] $u[3]";
-	($>, $<) = ($u[2], $u[2]);
+	&switch_to_unix_user(\@u);
 	}
 elsif ($access{'uid'}) {
 	local @u = getpwuid($access{'uid'});
-	$( = $u[3]; $) = "$u[3] $u[3]";
-	($>, $<) = ($u[2], $u[2]);
+	&switch_to_unix_user(\@u);
 	}
 }
 
 # safe_process_exec(command, uid, gid, handle, [input], [fixtags], [bsmode],
-#		    [timeout])
+#		    [timeout], [safe])
 # Executes the given command as the given user/group and writes all output
 # to the given file handle. Finishes when there is no more output or the
 # process stops running. Returns the number of bytes read.
 sub safe_process_exec
 {
-if (&is_readonly_mode()) {
+if (&is_readonly_mode() && !$_[8]) {
 	# Veto command in readonly mode
 	return 0;
 	}
+&webmin_debug_log('CMD', "cmd=$_[0] uid=$_[1] gid=$_[2]")
+	if ($gconfig{'debug_what_cmd'});
 
 if ($gconfig{'os_type'} eq 'windows') {
 	# For Windows, just run the command and read output
@@ -158,15 +162,13 @@ else {
 		if ($_[1]) {
 			if (defined($_[2])) {
 				# switch to given UID and GID
-				$( = $_[2]; $) = "$_[2] $_[2]";
-				($>, $<) = ($_[1], $_[1]);
+				&switch_to_unix_user(
+					[ undef, undef, $_[1], $_[2] ]);
 				}
 			else {
 				# switch to UID and all GIDs
 				local @u = getpwuid($_[1]);
-				$( = $u[3];
-				$) = "$u[3] ".join(" ", $u[3], &other_groups($u[0]));
-				($>, $<) = ($u[2], $u[2]);
+				&switch_to_unix_user(\@u);
 				}
 			}
 
@@ -216,8 +218,8 @@ else {
 				}
 			}
 		elsif ($sel == 0) {
-			# nothing to read. maybe the process is done, and a subprocess
-			# is hanging things up
+			# nothing to read. maybe the process is done, and a
+			# subprocess is hanging things up
 			last if (!kill(0, $pid));
 			}
 		if ($_[7] && time() - $start > $_[7]) {
@@ -259,6 +261,8 @@ if (&is_readonly_mode()) {
 	# When in readonly mode, don't run the command
 	$cmd = "/bin/true";
 	}
+&webmin_debug_log('CMD', "cmd=$cmd uid=$uid gid=$gid")
+	if ($gconfig{'debug_what_cmd'});
 
 eval "use IO::Pty";
 if (!$@) {
@@ -267,27 +271,33 @@ if (!$@) {
 	if (!$ptyfh) {
 		&error("Failed to create new PTY with IO::Pty");
 		}
-	local $ttyfh = $ptyfh->slave();
-	local $tty = $ptyfh->ttyname();
 	local $pid = fork();
 	if (!$pid) {
+		local $ttyfh = $ptyfh->slave();
+		local $tty = $ptyfh->ttyname();
 		if (defined(&close_controlling_pty)) {
 			&close_controlling_pty();
 			}
 		setsid();	# create a new session group
 		$ptyfh->make_slave_controlling_terminal();
 
+		# Turn off echoing, if we can
+		eval "use IO::Stty";
+		if (!$@) {
+			IO::Stty::stty($ttyfh, 'raw', '-echo');
+			}
+
 		close(STDIN); close(STDOUT); close(STDERR);
 		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
 		if ($_[1]) {
-			$( = $u[3]; $) = "$_[2] $_[2]";
-			($>, $<) = ($_[1], $_[1]);
+			&switch_to_unix_user([ undef, undef, $_[1], $_[2] ]);
 			}
 
-		open(STDIN, "<$tty");
-		open(STDOUT, ">$tty");
-		open(STDERR, ">&STDOUT");
-		close($ptyfh);
+		close($ptyfh);		# Used by other side only
+		open(STDIN, "<&".fileno($ttyfh));
+		open(STDOUT, ">&".fileno($ttyfh));
+		open(STDERR, ">&".fileno($ttyfh));
+		close($ttyfh);		# Already dup'd
 		exec($cmd);
 		print "Exec failed : $!\n";
 		exit 1;
@@ -298,6 +308,7 @@ if (!$@) {
 else {
 	# Need to create a PTY using built-in Webmin code
 	local ($ptyfh, $ttyfh, $pty, $tty) = &get_new_pty();
+	$tty || &error("Failed to create new PTY - try installing the IO::Tty Perl module");
 	local $pid = fork();
 	if (!$pid) {
 		if (defined(&close_controlling_pty)) {
@@ -316,6 +327,12 @@ else {
 			open($ttyfh, "+<$tty") || &error("Failed to open $tty : $!");
 			}
 
+		# Turn off echoing, if we can
+		eval "use IO::Stty";
+		if (!$@) {
+			IO::Stty::stty($ttyfh, 'raw', '-echo');
+			}
+
 		if (defined(&open_controlling_pty)) {
 			&open_controlling_pty($ptyfh, $ttyfh, $pty, $tty);
 			}
@@ -324,8 +341,7 @@ else {
 		untie(*STDIN); untie(*STDOUT); untie(*STDERR);
 		#setpgrp(0, $$);
 		if ($_[1]) {
-			$( = $u[3]; $) = "$_[2] $_[2]";
-			($>, $<) = ($_[1], $_[1]);
+			&switch_to_unix_user([ undef, undef, $_[1], $_[2] ]);
 			}
 
 		open(STDIN, "<$tty");
@@ -476,13 +492,16 @@ return &pty_backquote(@_);
 }
 
 # get_cpu_info()
-# Returns a list containing the 5, 10 and 15 minute load averages
+# Returns a list containing the 5, 10 and 15 minute load averages, and possibly
+# the CPU mhz, model, vendor, cache and count
 sub get_cpu_info
 {
 if (defined(&os_get_cpu_info)) {
 	return &os_get_cpu_info();
 	}
-local $out = `uptime 2>&1`;
+&clean_language();
+local $out = &backquote_command("uptime 2>&1");
+&reset_environment();
 return $out =~ /average(s)?:\s+([0-9\.]+),?\s+([0-9\.]+),?\s+([0-9\.]+)/i ?
 		( $2, $3, $4 ) : ( );
 }
@@ -570,5 +589,60 @@ else {
 	}
 }
 
+# nice_selector(name, value)
+# Returns a menu for selecting a nice level
+sub nice_selector
+{
+local ($name, $value) = @_;
+local $l = scalar(@nice_range);
+return &ui_select($name, $value,
+	[ map { [ $_, $_.($_ == $nice_range[0] ? " ($text{'edit_prihigh'})" :
+			  $_ == 0 ? " ($text{'edit_pridef'})" :
+			  $_ == $nice_range[$l-1] ? " ($text{'edit_prilow'})" :
+						    "") ] } @nice_range ]);
+}
+
+# get_kernel_info()
+# Returns the system's kernel version, architecture and OS
+sub get_kernel_info
+{
+if (defined(&os_get_kernel_info)) {
+	return &os_get_kernel_info();
+	}
+else {
+	my $uname = &has_command("uptrack-uname") || &has_command("uname");
+	my $out = &backquote_command("$uname -r 2>/dev/null ; ".
+				     "$uname -m 2>/dev/null ; ".
+				     "$uname -s 2>/dev/null");
+	return split(/\r?\n/, $out);
+	}
+}
+
+# get_system_uptime()
+# Returns uptime in days, minutes and hours
+sub get_system_uptime
+{
+my $out = &backquote_command("LC_ALL='' LANG='' uptime");
+if ($out =~ /up\s+(\d+)\s+(day|days),?\s+(\d+):(\d+)/) {
+	# up 198 days,  2:06
+	return ( $1, $3, $4 );
+	}
+elsif ($out =~ /up\s+(\d+)\s+(day|days),?\s+(\d+)\s+min/) {
+	# up 198 days,  10 mins
+	return ( $1, 0, $3 );
+	}
+elsif ($out =~ /up\s+(\d+):(\d+)/) {
+	# up 3:10
+	return ( 0, $1, $2 );
+	}
+elsif ($out =~ /up\s+(\d+)\s+min/) {
+	# up 45 mins
+	return ( 0, 0, $1 );
+	}
+else {
+	return ( );
+	}
+}
+
 1;
 
