@@ -77,8 +77,8 @@ return split(/\s+/, $out);
 sub find_file_processes
 {
 local($out, $files);
-$files = join(' ', @_);
-$out = `fuser $files 2>/dev/null`;
+$files = join(' ', map { quotemeta($_) } map { glob($_) } @_);
+$out = &backquote_command("fuser $files 2>/dev/null");
 $out =~ s/^\s+//g; $out =~ s/\s+$//g;
 return split(/\s+/, $out);
 }
@@ -87,7 +87,7 @@ return split(/\s+/, $out);
 sub renice_proc
 {
 return undef if (&is_readonly_mode());
-$out = `renice $_[1] -p $_[0] 2>&1`;
+local $out = &backquote_logged("renice $_[1] -p $_[0] 2>&1");
 if ($?) { return $out; }
 return undef;
 }
@@ -197,42 +197,6 @@ while(1) {
 	}
 }
 
-# get_memory_info()
-# Returns a list containing the real mem, free real mem, swap and free swap
-sub get_memory_info
-{
-return ( ) if (!&has_command("top"));
-local @rv;
-open(TOP, "top 2>&1 |");
-while(<TOP>) {
-	if (/Memory:\s+(\S+)\s+real,\s+(\S+)\s+free,\s+(\S+)\s+swap\s+in\s+use,\s+(\S+)\s+swap\s+free/) {
-		local ($real, $free, $swapused, $swapfree) = ($1, $2, $3, $4);
-		@rv = ( &fix_suffix($real),
-			&fix_suffix($free),
-			&fix_suffix($swapused) + &fix_suffix($swapfree),
-			&fix_suffix($swapfree) );
-		}
-	}
-close(TOP);
-return @rv;
-}
-
-sub fix_suffix
-{
-if ($_[0] =~ /^(\d+)K/i) {
-	return $1;
-	}
-elsif ($_[0] =~ /^(\d+)M/i) {
-	return $1*1024;
-	}
-elsif ($_[0] =~ /^(\d+)G/i) {
-	return $1*1024*1024;
-	}
-else {
-	return int($1);
-	}
-}
-
 # os_get_cpu_info()
 # Returns a list containing the 5, 10 and 15 minute load averages
 sub os_get_cpu_info
@@ -246,6 +210,38 @@ else {
 	}
 }
 
+# get_memory_info()
+# Returns a list containing the real mem, free real mem, swap and free swap
+# (In kilobytes).
+sub get_memory_info
+{
+if (!&has_command("kstat")) {
+	return ( );
+	}
+local %stat;
+foreach my $s ("physmem", "freemem") {
+	local $out = &backquote_command("kstat -p -m unix -s $s");
+	if ($out =~ /\s+(\d+)/) {
+		$stat{$s} = $1;
+		}
+	}
+local ($swaptotal, $swapfree);
+&open_execute_command(SWAP, "swap -l", 1);
+while(<SWAP>) {
+	if (/^\S+\s+\d+,\d+\s+\d+\s+(\d+)\s+(\d+)/) {
+		$swaptotal += $1;
+		$swapfree += $2;
+		}
+	}
+close(SWAP);
+local $pagesize = &backquote_command("pagesize 2>/dev/null");
+$pagesize = int($pagesize)/1024;
+$pagesize ||= 8;	# Fallback
+return ($stat{'physmem'}*$pagesize, $stat{'freemem'}*$pagesize,
+	$swaptotal/2, $swapfree/2);
+}
+
+
 foreach $ia (keys %text) {
 	if ($ia =~ /^sysv(_\S+)/) {
 		$info_arg_map{$1} = $text{$ia};
