@@ -13,22 +13,23 @@
 sub mod_core_directives
 {
 local $rv = [
-	[ 'AccessDenyMsg', 0, 3, 'virtual anon global', 1.22 ],
+	[ 'AccessDenyMsg', 0, 3, 'virtual anon global', 1.202 ],
 	[ 'AccessGrantMsg', 0, 3, 'virtual anon global', 0.99 ],
 	[ 'Allow Deny Order', 1, 3, 'limit', 0.99 ],
 	[ 'AllowAll DenyAll', 0, 3, 'directory anon limit ftpaccess', 0.99 ],
 	[ 'AllowFilter', 0, 3, 'virtual anon global', 1.20 ],
 	[ 'AllowForeignAddress', 0, 0, 'virtual anon global', 1.17 ],
-	[ 'AllowGroup', 0, 3, 'limit', 1.11 ],
+	[ 'AllowGroup', 1, 3, 'limit', 1.11 ],
 	[ 'AllowOverwrite', 0, 3, 'virtual anon directory ftpaccess global', 0.99 ],
-	[ 'AllowUser', 0, 3, 'limit', 1.17 ],
+	[ 'AllowUser', 1, 3, 'limit', 1.17 ],
 	[ 'AllowRetrieveRestart', 0, 0, 'virtual anon directory global ftpaccess', 0.99 ],
 	[ 'AllowStoreRestart', 0, 0, 'virtual anon directory global ftpaccess', 0.99 ],
 	[ 'AnonRequirePassword', 0, 6, 'anon', 0.99 ],
 	[ 'AnonymousGroup', 0, 6, 'virtual global', 1.13 ],
 	[ 'AuthAliasOnly', 0, 6, 'virtual anon global', 1.13 ],
 	[ 'AuthUsingAlias', 0, 6, 'anon', 1.20 ],
-	[ 'Bind', 0, 0, 'virtual', 1.16 ],
+	[ 'Bind', 0, 0, 'virtual', '1.16-1.27' ],
+	[ 'DefaultAddress', 0, 0, 'virtual', '1.27' ],
 	[ 'CDPath', 1, 2, 'virtual anon global', 1.20 ],
 	[ 'Class Classes', 1, 3, 'virtual', 1.20 ],
 	[ 'CommandBufferSize', 0, 0, 'virtual global', 1.20 ],
@@ -37,23 +38,25 @@ local $rv = [
 	[ 'DeferWelcome', 0, 0, 'virtual global', 0.99 ],
 	[ 'DeleteAbortedStores', 0, 2, 'virtual directory anon global ftpaccess', 1.20 ],
 	[ 'DenyFilter', 0, 3, 'virtual anon global', 1.20 ],
-	[ 'DenyGroup', 0, 3, 'limit', 1.11 ],
-	[ 'DenyUser', 0, 3, 'limit', 1.17 ],
+	[ 'DenyGroup', 1, 3, 'limit', 1.11 ],
+	[ 'DenyUser', 1, 3, 'limit', 1.17 ],
 	[ 'DisplayConnect', 0, 6, 'virtual global', 1.20 ],
-	[ 'DisplayFirstChdir', 0, 2, 'virtual anon directory global', 0.99 ],
+	[ 'DisplayFirstChdir', 0, 2, 'virtual anon directory global', '0.99-1.31' ],
+	[ 'DisplayChdir', 0, 2, 'virtual anon directory global', 1.31 ],
 	[ 'DisplayGoAway', 0, 6, 'virtual anon global', 1.20 ],
 	[ 'DisplayLogin', 0, 6, 'virtual anon global', 0.99 ],
 	[ 'DisplayQuit', 0, 6, 'virtual anon global', 1.20 ],
 	[ 'Group', 0, 5, 'virtual anon', undef, 0.99, 9 ],
 	[ 'GroupOwner', 0, 5, 'anon directory ftpaccess', 0.99 ],
 	[ 'GroupPassword', 1, 6, 'virtual anon global', 0.99 ],
-	[ 'HiddenStor', 0, 2, 'virtual anon directory global', 1.20 ],
+	[ 'HiddenStor', 0, 2, 'virtual anon directory global', '1.20-1.31' ],
+	[ 'HiddenStores', 0, 2, 'virtual anon directory global', 1.31 ],
 	[ 'HideGroup', 1, 2, 'directory anon', 0.99 ],
 	[ 'HideNoAccess', 0, 2, 'directory anon', 0.99 ],
 	[ 'HideUser', 1, 2, 'directory anon', 0.99 ],
 	[ 'IdentLookups', 0, 0, 'virtual global', 1.15 ],
 	[ 'IgnoreHidden', 0, 2, 'limit', 0.99 ],
-	[ 'MasqueradeAddress', 0, 0, 'virtual', 1.22 ],
+	[ 'MasqueradeAddress', 0, 0, 'virtual', 1.202 ],
 	[ 'MaxClients', 0, 0, 'virtual anon global', 0.99 ],
 	[ 'MaxClientsPerHost', 0, 0, 'virtual anon global', 1.17 ],
 #	[ 'MaxClientsPerUser', 0, 0, 'virtual anon global', 1.20 ],
@@ -66,9 +69,9 @@ local $rv = [
 	[ 'PidFile', 0, 4, 'root', 1.20 ],
 	[ 'Port', 0, 0, 'virtual', 0.99 ],
 	[ 'RequireValidShell', 0, 6, 'virtual anon global', 0.99 ],
-	[ 'RLimitCPU', 0, 4, 'root', 1.22 ],
-	[ 'RLimitMemory', 0, 4, 'root', 1.22 ],
-	[ 'RLimitOpenFiles', 0, 4, 'root', 1.22 ],
+	[ 'RLimitCPU', 0, 4, 'root', 1.202 ],
+	[ 'RLimitMemory', 0, 4, 'root', 1.202 ],
+	[ 'RLimitOpenFiles', 0, 4, 'root', 1.202 ],
 	[ 'ScoreboardPath', 0, 4, 'root', 1.16 ],
 	[ 'ServerAdmin', 0, 4, 'virtual', 0.99 ],
 	[ 'ServerIdent', 0, 0, 'virtual global', 1.20 ],
@@ -87,7 +90,7 @@ local $rv = [
 	[ 'TimeoutLogin', 0, 0, 'root', 0.99 ],
 	[ 'TimeoutNoTransfer', 0, 0, 'root', 0.99 ],
 	[ 'TimeoutStalled', 0, 0, 'root', 1.16 ],
-	[ 'TransferLog', 0, 1, 'virtual anon', undef, 1.14, 10 ],
+	[ 'TransferLog', 0, 1, 'virtual anon global', undef, 1.14, 10 ],
 	[ 'Umask', 0, 2, 'virtual directory ftpaccess', undef, 0.99, 3 ],
 	[ 'UseFtpUsers', 0, 6, 'virtual anon global', 0.99 ],
 	[ 'UseHostsAllowFile', 0, 3, 'virtual anon', 1.20 ],
@@ -171,12 +174,14 @@ for($i=0; defined($type = $in{"allow_type_$i"}); $i++) {
 	if ($mode == 0) { $what = "all"; }
 	elsif ($mode == 1) { $what = "none"; }
 	elsif ($mode == 2) {
-		&check_ipaddress($what) || &error(&text('mod_core_eip', $what));
+		&check_ipaddress($what) || &check_ip6address($what) ||
+			&error(&text('mod_core_eip', $what));
 		}
 	elsif ($mode == 3) {
 		$what =~ /^[0-9\.]+\.$/ ||
-			$what =~ /^([0-9\.]+)\/\d+$/ && &check_ipaddress($1) ||
-				&error(&text('mod_core_enet', $what));
+		    $what =~ /^([0-9\.]+)\/\d+$/ && &check_ipaddress("$1") ||
+		    $what =~ /^([a-f0-9:]+)\/\d+$/ && &check_ip6address("$1") ||
+			&error(&text('mod_core_enet', $what));
 		}
 	elsif ($mode == 4) {
 		$what =~ /^[A-Za-z0-9\.\-]+$/ ||
@@ -230,12 +235,13 @@ return &parse_choice("AllowForeignAddress", "");
 
 sub edit_AllowGroup
 {
+local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
 return (2, $text{'mod_core_agroup'},
-	&opt_input($_[0]->{'value'}, "AllowGroup", $text{'mod_core_all'}, 50));
+	&opt_input($v, "AllowGroup", $text{'mod_core_all'}, 50));
 }
 sub save_AllowGroup
 {
-return &parse_opt("AllowGroup");
+return ( $in{'AllowGroup_def'} ? [ ] : [ split(/\s+/, $in{'AllowGroup'}) ] );
 }
 
 sub edit_AllowOverwrite
@@ -276,12 +282,13 @@ return &parse_choice("AllowStoreRestart", "");
 
 sub edit_AllowUser
 {
+local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
 return (2, $text{'mod_core_auser'},
-	&opt_input($_[0]->{'value'}, "AllowUser", $text{'mod_core_all'}, 50));
+	&opt_input($v, "AllowUser", $text{'mod_core_all'}, 50));
 }
 sub save_AllowUser
 {
-return &parse_opt("AllowUser");
+return ( $in{'AllowUser_def'} ? [ ] : [ split(/\s+/, $in{'AllowUser'}) ] );
 }
 
 sub edit_AnonRequirePassword
@@ -338,7 +345,21 @@ return (1, $text{'mod_core_bind'},
 }
 sub save_Bind
 {
-return &parse_opt("Bind", '^(\d+)\.(\d+)\.(\d+)\.(\d+)$',
+return &parse_opt("Bind", '^(\d+)\.(\d+)\.(\d+)\.(\d+)|([0-9:]+)$',
+		  $text{'mod_core_ebind'});
+}
+
+sub edit_DefaultAddress
+{
+return (1, $text{'mod_core_bind'},
+	&opt_input($_[0]->{'value'}, "DefaultAddress", $text{'mod_core_bind_all'}, 15));
+}
+sub save_DefaultAddress
+{
+$in{'DefaultAddress_def'} || &to_ipaddress($in{'DefaultAddress'}) ||
+	&to_ip6address($in{'DefaultAddress'}) ||
+	&error(text{'mod_core_ebind'});
+return &parse_opt("DefaultAddress", '^\S+$',
 		  $text{'mod_core_ebind'});
 }
 
@@ -469,31 +490,33 @@ return &parse_choice("DeleteAbortedStores", "");
 sub edit_DenyFilter
 {
 return (1, $text{'mod_core_dfilter'},
-	&opt_input($_[0]->{'value'}, "AllowFilter", $text{'default'}, 15));
+	&opt_input($_[0]->{'value'}, "DenyFilter", $text{'default'}, 15));
 }
 sub save_DenyFilter
 {
-return &parse_opt("AllowFilter");
+return &parse_opt("DenyFilter");
 }
 
 sub edit_DenyGroup
 {
+local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
 return (2, $text{'mod_core_dgroup'},
-	&opt_input($_[0]->{'value'}, "DenyGroup", $text{'mod_core_none'}, 50));
+	&opt_input($v, "DenyGroup", $text{'mod_core_none'}, 50));
 }
 sub save_DenyGroup
 {
-return &parse_opt("DenyGroup");
+return ( $in{'DenyGroup_def'} ? [ ] : [ split(/\s+/, $in{'DenyGroup'}) ] );
 }
 
 sub edit_DenyUser
 {
+local $v = @{$_[0]} ? join(" ", (map { $_->{'value'} } @{$_[0]})) : undef;
 return (2, $text{'mod_core_duser'},
-	&opt_input($_[0]->{'value'}, "DenyUser", $text{'mod_core_none'}, 50));
+	&opt_input($v, "DenyUser", $text{'mod_core_none'}, 50));
 }
 sub save_DenyUser
 {
-return &parse_opt("DenyUser");
+return ( $in{'DenyUser_def'} ? [ ] : [ split(/\s+/, $in{'DenyUser'}) ] );
 }
 
 sub edit_DisplayConnect
@@ -518,6 +541,23 @@ sub save_DisplayFirstChdir
 return &parse_opt("DisplayFirstChdir", '^\S+$', $text{'mod_core_efirstcd'});
 }
 
+sub edit_DisplayChdir
+{
+return (1, $text{'mod_core_firstcd'},
+	&opt_input($_[0]->{'words'}->[0], "DisplayChdir",
+		   $text{'mod_core_none'}, 15).
+	&ui_checkbox("DisplayChdir_always", 'true', $text{'mod_core_firstcdt'},
+		     $_[0]->{'words'}->[1] eq 'true'));
+}
+sub save_DisplayChdir
+{
+local @rv = &parse_opt("DisplayChdir", '^\S+$', $text{'mod_core_efirstcd'});
+if ($in{'DisplayChdir_always'}) {
+	$rv[0]->[0] .= ' true';
+	}
+return @rv;
+}
+
 sub edit_DisplayGoAway
 {
 return (2, $text{'mod_core_goaway'},
@@ -589,7 +629,7 @@ sub save_GroupOwner
 {
 if ($in{'GroupOwner_def'}) { return ( [ ] ); }
 else {
-	getgrnam($in{'GroupOwner'}) || &error($text{'mod_core_egowner'});
+	defined(getgrnam($in{'GroupOwner'})) || &error($text{'mod_core_egowner'});
 	return ( [ $in{'GroupOwner'} ] );
 	}
 }
@@ -629,7 +669,7 @@ for($i=0; defined($in{"GroupPassword_n_$i"}); $i++) {
 	else {
 		$salt = substr(time(), 0, 2);
 		push(@rv, $in{"GroupPassword_n_$i"}.' '.
-			  crypt($in{"GroupPassword_p_$i"}, $salt));
+			  &unix_crypt($in{"GroupPassword_p_$i"}, $salt));
 		}
 	}
 return ( \@rv );
@@ -647,6 +687,18 @@ sub save_HiddenStor
 return &parse_choice("HiddenStor", "");
 }
 
+sub edit_HiddenStores
+{
+return (1, $text{'mod_core_hstor'},
+	&choice_input($_[0]->{'value'}, "HiddenStores", "",
+		      "$text{'yes'},on", "$text{'no'},off",
+		      "$text{'default'},"));
+}
+sub save_HiddenStores
+{
+return &parse_choice("HiddenStores", "");
+}
+
 sub edit_HideGroup
 {
 return (2, $text{'mod_core_hgroup'},
@@ -684,9 +736,9 @@ sub save_HideUser
 {
 local @hu = split(/\s+/, $in{'HideUser'});
 foreach $u (@hu) {
-	scalar(getpwnam($u)) || &error($text{'mod_core_ehuser'});
+	defined(getpwnam($u)) || &error($text{'mod_core_ehuser'});
 	}
-return ( \@hg );
+return ( \@hu );
 }
 
 sub edit_IdentLookups
@@ -1109,13 +1161,32 @@ return &parse_select("SyslogLevel", "");
 
 sub edit_TransferLog
 {
-return (2, $text{'mod_core_tlog'},
-	&opt_input($_[0]->{'value'}, "TransferLog", $text{'default'}, 50,
-		   &file_chooser_button("TransferLog")));
+local $mode = $_[0]->{'value'} eq 'NONE' ? 2 :
+	      $_[0]->{'value'} ? 0 : 1;
+local $rv = sprintf "<input type=radio name=TransferLog_def value=1 %s> %s\n",
+		$mode == 1 ? "checked" : "", $text{'default'};
+if ($_[1]->{'version'} >= 1.17) {
+	$rv .= sprintf"<input type=radio name=TransferLog_def value=2 %s> %s\n",
+		$mode == 2 ? "checked" : "", $text{'mod_core_nowhere'};
+	}
+$rv .= sprintf "<input type=radio name=TransferLog_def value=0 %s>\n",
+		$mode == 0 ? "checked" : "";
+$rv .= sprintf "<input name=TransferLog size=50 value='%s'>\n",
+		$mode == 0 ? $_[0]->{'value'} : "";
+return (2, $text{'mod_core_tlog'}, $rv);
 }
 sub save_TransferLog
 {
-return &parse_opt("TransferLog", '^(\/\S+)|NONE$', $text{'mod_core_etlog'});
+if ($in{'TransferLog_def'} == 1) {
+	return ( [ ] );
+	}
+elsif ($in{'TransferLog_def'} == 2) {
+	return ( [ 'NONE' ] );
+	}
+else {
+	$in{'TransferLog'} =~ /^\/\S+$/ || &error($text{'mod_core_etlog'}); 
+	return ( [ $in{'TransferLog'} ] );
+	}
 }
 
 sub edit_tcpBackLog
@@ -1218,12 +1289,39 @@ return &parse_opt("TimeoutStalled", '^\d+$', $text{'mod_core_etstalled'});
 
 sub edit_Umask
 {
-return (1, $text{'mod_core_umask'},
-	&opt_input($_[0]->{'value'}, "Umask", $text{'default'}, 3));
+local $rv;
+$rv .= sprintf "<input type=radio name=Umask_def value=1 %s> %s\n",
+	$_[0]->{'words'}->[0] ? "" : "checked", $text{'default'};
+$rv .= sprintf "<input type=radio name=Umask_def value=0 %s> %s\n",
+	$_[0]->{'words'}->[0] ? "checked" : "", $text{'mod_core_octal'};
+$rv .= sprintf "<input name=Umask size=5 value='%s'>\n",
+	$_[0]->{'words'}->[0];
+
+$rv .= "&nbsp;&nbsp;&nbsp;<b>$text{'mod_core_umask_d'}</b>\n";
+$rv .= sprintf "<input type=radio name=Umask_d_def value=1 %s> %s\n",
+	$_[0]->{'words'}->[1] ? "" : "checked", $text{'default'};
+$rv .= sprintf "<input type=radio name=Umask_d_def value=0 %s> %s\n",
+	$_[0]->{'words'}->[1] ? "checked" : "", $text{'mod_core_octal'};
+$rv .= sprintf "<input name=Umask_d size=5 value='%s'>\n",
+	$_[0]->{'words'}->[1];
+
+return (2, $text{'mod_core_umask'}, $rv);
 }
 sub save_Umask
 {
-return &parse_opt("Umask", '^[0-7]{3}$', $text{'mod_core_eumask'});
+if ($in{'Umask_def'}) {
+	return ( [ ] );
+	}
+else {
+	$in{'Umask'} =~ /^[0-7]{3}$/ || &error($text{'mod_core_eumask'});
+	if ($in{'Umask_d_def'}) {
+		return ( [ $in{'Umask'} ] );
+		}
+	else {
+		$in{'Umask_d'} =~ /^[0-7]{3}$/ || &error($text{'mod_core_eumask'});
+		return ( [ $in{'Umask'}." ".$in{'Umask_d'} ] );
+		}
+	}
 }
 
 sub edit_UseFtpUsers
@@ -1395,7 +1493,7 @@ for($i=0; defined($in{"UserPassword_n_$i"}); $i++) {
 	else {
 		$salt = substr(time(), 0, 2);
 		push(@rv, $in{"UserPassword_n_$i"}.' '.
-			  crypt($in{"UserPassword_p_$i"}, $salt));
+			  &unix_crypt($in{"UserPassword_p_$i"}, $salt));
 		}
 	}
 return ( \@rv );
