@@ -1,7 +1,8 @@
 # proftpd-lib.pl
 # Common functions for the proftpd server config file
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
 
 # Load the site-specific information on the server executable
@@ -24,9 +25,6 @@ if (@get_config_cache) {
 	return \@get_config_cache;
 	}
 @get_config_cache = &get_config_file($config{'proftpd_conf'});
-foreach $inc (&find_directive("Include", \@get_config_cache)) {
-	push(@get_config_cache, &get_config_file($inc));
-	}
 return \@get_config_cache;
 }
 
@@ -34,21 +32,30 @@ return \@get_config_cache;
 sub get_config_file
 {
 local @rv;
-if (opendir(DIR, $_[0])) {
+local $fn = $_[0];
+if ($fn !~ /^\//) {
+	$config{'proftpd_conf'} =~ /^(.*)\//;
+	$fn = "$1/$fn";
+	}
+if (opendir(DIR, $fn)) {
 	# Is a directory .. parse all files!
 	local @files = readdir(DIR);
 	closedir(DIR);
 	foreach $f (@files) {
 		next if ($f =~ /^\./);
-		push(@rv, &get_config_file("$_[0]/$f"));
+		push(@rv, &get_config_file("$fn/$f"));
 		}
 	}
 else {
 	# Just a normal config file
 	local $lnum = 0;
-	open(CONF, $_[0]);
-	@rv = &parse_config_file(CONF, $lnum, $_[0]);
-	close(CONF);
+	if (open(CONF, $fn)) {
+		@rv = &parse_config_file(CONF, $lnum, $fn);
+		close(CONF);
+		foreach $inc (&find_directive("Include", \@rv)) {
+			push(@rv, &get_config_file($inc));
+			}
+		}
 	}
 return @rv;
 }
@@ -240,6 +247,9 @@ sub make_directives
 {
 local @rv;
 local $ver = $_[1];
+if ($ver =~ /^(1)\.(2)(\d+)$/) {
+	$ver = sprintf "%d.%d%2.2d", $1, $2, $3;
+	}
 foreach $d (@{$_[0]}) {
 	local(%dir);
 	$dir{'name'} = $d->[0];
@@ -331,6 +341,7 @@ sub parse_inputs
 # First call editor functions to get new values. Each function returns
 # an array of references to arrays containing the new values for the directive.
 local ($i, @chname, @chval);
+&before_changing();
 foreach $e (@{$_[0]}) {
 	local @dirs = split(/\s+/, $e->{'name'});
 	local $func = "save_".join('_', @dirs);
@@ -346,6 +357,7 @@ for($i=0; $i<@chname; $i++) {
 	&save_directive($chname[$i], $chval[$i], $_[1], $_[2]);
 	}
 &flush_file_lines();
+&after_changing();
 }
 
 # opt_input(value, name, default, size, [units])
@@ -446,28 +458,21 @@ foreach $m (sort { $a cmp $b } (@module_files)) {
 			}
 		}
         }
-print "<table width=100% cellpadding=5>\n";
+local (@titles, @links, @icons);
 for($i=0; $text{"type_$i"}; $i++) {
 	if ($etype{$i}) {
-		if ($c%5 == 0) { print "<tr>\n"; }
-		print "<td valign=top align=center width=20%>\n";
-		&generate_icon("images/type_icon_$i.gif", $text{"type_$i"},
-			       $_[1]."type=$i");
-		print "</td>\n";
-		if ($c%5 == 4) { print "<tr>\n"; }
-		$c++;
+		push(@links, $_[1]."type=$i");
+		push(@titles, $text{"type_$i"});
+		push(@icons, "images/type_icon_$i.gif");
 		}
 	}
 for($i=2; $i<@_; $i++) {
-	if ($c%5 == 0) { print "<tr>\n"; }
-	print "<td valign=top align=center width=20%>\n";
-	&generate_icon($_[$i]->{'icon'}, $_[$i]->{'name'}, $_[$i]->{'link'});
-	print "</td>\n";
-	if ($c%5 == 4) { print "<tr>\n"; }
-	$c++;
+	push(@links, $_[$i]->{'link'});
+	push(@titles, $_[$i]->{'name'});
+	push(@icons, $_[$i]->{'icon'});
 	}
-while($c++ % 5) { print "<td width=20%></td>\n"; }
-print "</table><p>\n";
+&icons_table(\@links, \@titles, \@icons, 5);
+print "<p>\n";
 }
 
 sub lock_proftpd_files
@@ -608,5 +613,267 @@ close(FTPACCESS);
 return \@conf;
 }
 
+# get_or_create_global(&config)
+# Returns an array ref of members of the <Global> section, creating if necessary
+sub get_or_create_global
+{
+local ($conf) = @_;
+local $global = &find_directive_struct("Global", $conf);
+if ($global) {
+	# Already exists .. just return member list
+	return $global->{'members'};
+	}
+else {
+	# Need to add it!
+	local $lref = &read_file_lines($config{'proftpd_conf'});
+	local $olen = @$lref;
+	push(@$lref, "<Global>", "</Global>");
+	&flush_file_lines();
+	$global = { 'name' => 'Global',
+		    'members' => [ { 'line' => $olen,
+				     'eline' => $olen,
+				     'file' => $config{'proftpd_conf'},
+				     'type' => 0,
+				     'name' => 'dummy' } ],
+		    'line' => $olen,
+		    'eline' => $olen+1,
+		    'file' => $config{'proftpd_conf'},
+		    'type' => 1,
+		    'value' => undef,
+		    'words' => [ ] };
+	push(@{$_[0]}, $global);
+	return $global->{'members'};
+	}
+}
+
+# test_config()
+# If possible, test the current configuration and return an error message,
+# or undef.
+sub test_config
+{
+if ($site{'version'} >= 1.2) {
+	# Test the configuration with -t flag
+	local $cmd = "$config{'proftpd_path'} -t -c $config{'proftpd_conf'}";
+	local $out = `$cmd 2>&1 </dev/null`;
+	return $out if ($?);
+	}
+return undef;
+}
+
+# before_changing()
+# If testing all changes, backup the config files so they can be reverted
+# if necessary.
+sub before_changing
+{
+if ($config{'test_always'}) {
+	local $conf = &get_config();
+	local @files = &unique(map { $_->{'file'} } @$conf);
+	local $/ = undef;
+	foreach $f (@files) {
+		if (open(BEFORE, $f)) {
+			$before_changing{$f} = <BEFORE>;
+			close(BEFORE);
+			}
+		}
+	}
+}
+
+# after_changing()
+# If testing all changes, test now and revert the configs and show an error
+# message if a problem was found.
+sub after_changing
+{
+if ($config{'test_always'}) {
+	local $err = &test_config();
+	if ($err) {
+		# Something failed .. revert all files
+		local $f;
+		foreach $f (keys %before_changing) {
+			&open_tempfile(AFTER, ">$f");
+			&print_tempfile(AFTER, $before_changing{$f});
+			&close_tempfile(AFTER);
+			}
+		&error(&text('eafter', "<pre>$err</pre>"));
+		}
+	}
+}
+
+# restart_button()
+# Returns HTML for a link to put in the top-right corner of every page
+sub restart_button
+{
+local $r = &is_proftpd_running();
+return undef if ($r < 0);
+local $args = "redir=".&urlize(&this_url());
+if ($r) {
+	$rv .= "<a href=\"apply.cgi?$args&pid=$1\">$text{'proftpd_apply'}</a><br>\n";
+	$rv .= "<a href=\"stop.cgi?$args&pid=$1\">$text{'proftpd_stop'}</a>\n";
+	}
+else {
+	$rv = "<a href=\"start.cgi?$args\">$text{'proftpd_start'}</a><br>\n";
+	}
+return $rv;
+}
+
+# is_proftpd_running()
+# Returns the PID if ProFTPd is running, 0 if down, -1 if running under inetd
+sub is_proftpd_running
+{
+local $conf = &get_config();
+local $st = &find_directive("ServerType", $conf);
+return -1 if (lc($st) eq "inetd");
+local $pid = &get_proftpd_pid();
+return $pid;
+}
+
+# this_url()
+# Returns the URL in the apache directory of the current script
+sub this_url
+{
+local($url);
+$url = $ENV{'SCRIPT_NAME'};
+if ($ENV{'QUERY_STRING'} ne "") { $url .= "?$ENV{'QUERY_STRING'}"; }
+return $url;
+}
+
+# running_under_inetd()
+# Returns the inetd/xinetd object and program if ProFTPd is running under one
+sub running_under_inetd
+{
+# Never under inetd if not set so in config
+local $conf = &get_config();
+local $st = &find_directive("ServerType", $conf);
+return ( ) if (lc($st) eq "inetd");
+
+local ($inet, $inet_mod);
+if (&foreign_check('inetd')) {
+        # Check if proftpd is in inetd
+        &foreign_require('inetd', 'inetd-lib.pl');
+	local $i;
+        foreach $i (&foreign_call('inetd', 'list_inets')) {
+                if ($i->[1] && $i->[3] eq 'ftp') {
+                        $inet = $i;
+                        last;
+                        }
+                }
+        $inet_mod = 'inetd';
+        }
+elsif (&foreign_check('xinetd')) {
+        # Check if proftpd is in xinetd
+        &foreign_require('xinetd', 'xinetd-lib.pl');
+	local $xi;
+        foreach $xi (&foreign_call("xinetd", "get_xinetd_config")) {
+                if ($xi->{'quick'}->{'disable'}->[0] ne 'yes' &&
+                    $xi->{'value'} eq 'ftp') {
+                        $inet = $xi;
+                        last;
+                        }
+                }
+        $inet_mod = 'xinetd';
+        }
+else {
+        # Not supported on this OS .. assume so
+        $inet = 1;
+	}
+return ($inet, $inet_mod);
+}
+
+# get_proftpd_pid()
+sub get_proftpd_pid
+{
+if ($config{'pid_file'}) {
+	return &check_pid_file($config{'pid_file'});
+	}
+else {
+	local ($pid) = &find_byname("proftpd");
+	return $pid;
+	}
+}
+
+sub get_proftpd_version
+{
+local $out = `$config{'proftpd_path'} -v 2>&1`;
+${$_[0]} = $out if ($_[0]);
+if ($out =~ /ProFTPD\s+Version\s+(\d+)\.([0-9\.]+)/i ||
+    $out =~ /ProFTPD\s+(\d+)\.([0-9\.]+)/i) {
+	local ($v1, $v2) = ($1, $2);
+	$v2 =~ s/\.//g;
+	return "$v1.$v2";
+	}
+return undef;
+}
+
+# apply_configuration()
+# Activate the ProFTPd configuration, either by sending a HUP signal or
+# by stopping and starting
+sub apply_configuration
+{
+# Check if running from inetd
+local $conf = &get_config();
+local $st = &find_directive("ServerType", $conf);
+if ($st eq 'inetd') {
+	return $text{'stop_einetd'};
+	}
+if (&get_proftpd_version() > 1.22) {
+	# Stop and re-start
+	local $err = &stop_proftpd();
+	return $err if ($err);
+	sleep(1);	# Wait for clean shutdown
+	return &start_proftpd();
+	}
+else {
+	# Can just HUP
+	local $pid = &get_proftpd_pid();
+	$pid || return $text{'apply_egone'};
+	&kill_logged('HUP', $pid);
+	return undef;
+	}
+}
+
+# stop_proftpd()
+# Halts the running ProFTPd process, and returns undef on success or any error
+# message on failure.
+sub stop_proftpd
+{
+# Check if running from inetd
+local $conf = &get_config();
+local $st = &find_directive("ServerType", $conf);
+if ($st eq 'inetd') {
+	return $text{'stop_einetd'};
+	}
+if ($config{'stop_cmd'}) {
+	local $out = &backquote_logged("$config{'stop_cmd'} 2>&1 </dev/null");
+	if ($?) {
+		return "<pre>$out</pre>";
+		}
+	}
+else {
+	local $pid = &get_proftpd_pid();
+	$pid && &kill_logged('TERM', $pid) ||
+		return $text{'stop_erun'};
+	}
+return undef;
+}
+
+# start_proftpd()
+# Attempt to start the FTP server, and return undef on success or an error
+# messsage on failure.
+sub start_proftpd
+{
+local $conf = &get_config();
+local $st = &find_directive("ServerType", $conf);
+if ($st eq 'inetd') {
+	return $text{'start_einetd'};
+	}
+local $out;
+if ($config{'start_cmd'}) {
+	$out = &backquote_logged("$config{'start_cmd'} 2>&1 </dev/null");
+	}
+else {
+	$out = &backquote_logged("$config{'proftpd_path'} 2>&1 </dev/null");
+	}
+return $? ? "<pre>$out</pre>" : undef;
+}
+
 1;
 
