@@ -1,8 +1,10 @@
 # qmail-lib.pl
 # Common functions for parsing qmail config files
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
+do 'boxes-lib.pl';
 
 $qmail_alias_dir = "$config{'qmail_dir'}/alias";
 $qmail_bin_dir = "$config{'qmail_dir'}/bin";
@@ -14,6 +16,7 @@ $qmail_mess_dir = "$config{'qmail_dir'}/queue/mess";
 $qmail_info_dir = "$config{'qmail_dir'}/queue/info";
 $qmail_local_dir = "$config{'qmail_dir'}/queue/local";
 $qmail_remote_dir = "$config{'qmail_dir'}/queue/remote";
+$qmail_users_dir = "$config{'qmail_dir'}/users";
 $qmail_assigns_file = "$config{'qmail_dir'}/users/assign";
 
 $config{'perpage'} ||= 20;      # a value of 0 can cause problems
@@ -164,21 +167,20 @@ return wantarray ? @rv : $rv[0];
 sub create_alias
 {
 local $f = "$qmail_alias_dir/.qmail-$_[0]->{'name'}";
-&lock_file($f);
-open(FILE, ">$f");
+&open_lock_tempfile(FILE, ">$f");
 foreach $v (@{$_[0]->{'values'}}) {
-	print FILE $v,"\n";
+	&print_tempfile(FILE, $v,"\n");
 	}
-close(FILE);
-&unlock_file($f);
+&close_tempfile(FILE);
+&set_ownership_permissions(undef, undef, 0644, $f);
 }
 
 # modify_alias(&old, &alias)
 # Modifies an existing qmail alias
 sub modify_alias(&old, &alias)
 {
-&lock_file($_[0]->{'file'});
 if ($_[0]->{'name'} ne $_[1]->{'name'}) {
+	&lock_file($_[0]->{'file'});
 	unlink($_[0]->{'file'});
 	&unlock_file($_[0]->{'file'});
 	}
@@ -211,9 +213,9 @@ sub set_control_file
 {
 &lock_file("$qmail_control_dir/$_[0]");
 if (defined($_[1])) {
-	open(FILE, ">$qmail_control_dir/$_[0]");
-	print FILE $_[1],"\n";
-	close(FILE);
+	&open_tempfile(FILE, ">$qmail_control_dir/$_[0]");
+	&print_tempfile(FILE, $_[1],"\n");
+	&close_tempfile(FILE);
 	}
 else {
 	unlink("$qmail_control_dir/$_[0]");
@@ -243,11 +245,11 @@ sub save_control_file
 {
 &lock_file("$qmail_control_dir/$_[0]");
 if (defined($_[1])) {
-	open(FILE, ">$qmail_control_dir/$_[0]");
+	&open_tempfile(FILE, ">$qmail_control_dir/$_[0]");
 	foreach $l (@{$_[1]}) {
-		print FILE $l,"\n";
+		&print_tempfile(FILE, $l,"\n");
 		}
-	close(FILE);
+	&close_tempfile(FILE);
 	}
 else {
 	unlink("$qmail_control_dir/$_[0]");
@@ -259,40 +261,49 @@ else {
 # Returns a list of all SMTP routes
 sub list_routes
 {
-local $lnum = 0;
-local @rv;
-open(ROUTES, $qmail_routes_file);
-while(<ROUTES>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/^(\S*):(\S*):(\d+)/) {
-		push(@rv, { 'from' => $1,
-			    'to' => $2,
-			    'port' => $3,
-			    'idx' => scalar(@rv),
-			    'line' => $lnum });
+if (!scalar(@list_routes_cache)) {
+	local $lnum = 0;
+	local @rv;
+	open(ROUTES, $qmail_routes_file);
+	while(<ROUTES>) {
+		s/\r|\n//g;
+		s/#.*$//;
+		if (/^(\S*):(\S*):(\d+)/) {
+			push(@rv, { 'from' => $1,
+				    'to' => $2,
+				    'port' => $3,
+				    'idx' => scalar(@rv),
+				    'line' => $lnum });
+			}
+		elsif (/^(\S*):(\S*)/) {
+			push(@rv, { 'from' => $1,
+				    'to' => $2,
+				    'idx' => scalar(@rv),
+				    'line' => $lnum });
+			}
+		$lnum++;
 		}
-	elsif (/^(\S*):(\S*)/) {
-		push(@rv, { 'from' => $1,
-			    'to' => $2,
-			    'idx' => scalar(@rv),
-			    'line' => $lnum });
-		}
-	$lnum++;
+	close(ROUTES);
+	@list_routes_cache = @rv;
 	}
-close(ROUTES);
-return @rv;
+return @list_routes_cache;
 }
 
 # create_route(&route)
 sub create_route
 {
+&list_routes();	# force cache init
 &lock_file($qmail_routes_file);
 local $lref = &read_file_lines($qmail_routes_file);
 push(@$lref, $_[0]->{'from'}.':'.$_[0]->{'to'}.
 	     ($_[0]->{'port'} ? ':'.$_[0]->{'port'} : ''));
 &flush_file_lines();
 &unlock_file($qmail_routes_file);
+
+# Update in memory cache
+$_[0]->{'line'} = @$lref-1;
+$_[0]->{'idx'} = scalar(@list_routes_cache);
+push(@list_routes_cache, $_[0]);
 }
 
 # modify_route(&old, &route)
@@ -305,6 +316,13 @@ splice(@$lref, $_[0]->{'line'}, 1,
        ($_[1]->{'port'} ? ':'.$_[1]->{'port'} : ''));
 &flush_file_lines();
 &unlock_file($qmail_routes_file);
+
+# Update in memory cache
+if ($_[0] ne $_[1]) {
+	$_[1]->{'line'} = $_[0]->{'line'};
+	$_[1]->{'idx'} = $_[0]->{'idx'};
+	$list_routes_cache[$_[1]->{'idx'}] = $_[1];
+	}
 }
 
 # delete_route(&route)
@@ -315,6 +333,13 @@ local $lref = &read_file_lines($qmail_routes_file);
 splice(@$lref, $_[0]->{'line'}, 1);
 &flush_file_lines();
 &unlock_file($qmail_routes_file);
+
+# delete from cache
+splice(@list_routes_cache, $_[0]->{'idx'}, 1);
+foreach my $v (@list_routes_cache) {
+	$v->{'line'}-- if ($v->{'line'} > $_[0]->{'line'});
+	$v->{'idx'}-- if ($v->{'idx'} > $_[0]->{'idx'});
+	}
 }
 
 # route_form([&route])
@@ -362,36 +387,40 @@ print "</table></td></tr></table></form>\n";
 # Returns a list of all virtualdomains file entries
 sub list_virts
 {
-local $lnum = 0;
-local @rv;
-open(VIRTS, $qmail_virts_file);
-while(<VIRTS>) {
-	s/\r|\n//g;
-	s/#.*$//;
-	if (/^(\S+)\@(\S+):(\S*)/) {
-		push(@rv, { 'user' => $1,
-			    'domain' => $2,
-			    'from' => "$1\@$2",
-			    'prepend' => $3,
-			    'line' => $lnum,
-			    'idx' => scalar(@rv) } );
+if (!scalar(@list_virts_cache)) {
+	local $lnum = 0;
+	local @rv;
+	open(VIRTS, $qmail_virts_file);
+	while(<VIRTS>) {
+		s/\r|\n//g;
+		s/#.*$//;
+		if (/^(\S+)\@(\S+):(\S*)/) {
+			push(@rv, { 'user' => $1,
+				    'domain' => $2,
+				    'from' => "$1\@$2",
+				    'prepend' => $3,
+				    'line' => $lnum,
+				    'idx' => scalar(@rv) } );
+			}
+		elsif (/^(\S*):(\S*)/) {
+			push(@rv, { 'domain' => $1,
+				    'from' => $1,
+				    'prepend' => $2,
+				    'line' => $lnum,
+				    'idx' => scalar(@rv) } );
+			}
+		$lnum++;
 		}
-	elsif (/^(\S*):(\S*)/) {
-		push(@rv, { 'domain' => $1,
-			    'from' => $1,
-			    'prepend' => $2,
-			    'line' => $lnum,
-			    'idx' => scalar(@rv) } );
-		}
-	$lnum++;
+	close(VIRTS);
+	@list_virts_cache = @rv;
 	}
-close(VIRTS);
-return @rv;
+return @list_virts_cache;
 }
 
 # create_virt(&virt)
 sub create_virt
 {
+&list_virts();	# force cache init
 &lock_file($qmail_virts_file);
 local $lref = &read_file_lines($qmail_virts_file);
 push(@$lref, join(":", $_[0]->{'user'} ? "$_[0]->{'user'}\@$_[0]->{'domain'}"
@@ -399,6 +428,11 @@ push(@$lref, join(":", $_[0]->{'user'} ? "$_[0]->{'user'}\@$_[0]->{'domain'}"
 		       $_[0]->{'prepend'}));
 &flush_file_lines();
 &unlock_file($qmail_virts_file);
+
+# Update in memory cache
+$_[0]->{'line'} = @$lref-1;
+$_[0]->{'idx'} = scalar(@list_virts_cache);
+push(@list_virts_cache, $_[0]);
 }
 
 # delete_virt(&virt)
@@ -409,6 +443,13 @@ local $lref = &read_file_lines($qmail_virts_file);
 splice(@$lref, $_[0]->{'line'}, 1);
 &flush_file_lines();
 &unlock_file($qmail_virts_file);
+
+# delete from cache
+splice(@list_virts_cache, $_[0]->{'idx'}, 1);
+foreach my $v (@list_virts_cache) {
+	$v->{'line'}-- if ($v->{'line'} > $_[0]->{'line'});
+	$v->{'idx'}-- if ($v->{'idx'} > $_[0]->{'idx'});
+	}
 }
 
 # modify_virt(&old, &virt)
@@ -422,6 +463,13 @@ splice(@$lref, $_[0]->{'line'}, 1,
 		       $_[1]->{'prepend'}));
 &flush_file_lines();
 &unlock_file($qmail_virts_file);
+
+# Update in memory cache
+if ($_[0] ne $_[1]) {
+	$_[1]->{'line'} = $_[0]->{'line'};
+	$_[1]->{'idx'} = $_[0]->{'idx'};
+	$list_virts_cache[$_[1]->{'idx'}] = $_[1];
+	}
 }
 
 # virt_form(&virt)
@@ -544,35 +592,39 @@ return $r;
 # Returns a list of qmail user assignments
 sub list_assigns
 {
-local @rv;
-local $lnum = 0;
-open(ASSIGNS, $qmail_assigns_file);
-while(<ASSIGNS>) {
-	s/\r|\n//g;
-	last if ($_ eq '.');
-	local @line = split(/:/, $_, 8);
-	if ($line[0] =~ /^([\+=])(\S*)/) {
-		local $asn = { 'address' => $2,
-			       'mode' => $1,
-			       'user' => $line[1],
-			       'uid' => $line[2],
-			       'gid' => $line[3],
-			       'home' => $line[4],
-			       'dash' => $line[5],
-			       'preext' => $line[6],
-			       'idx' => scalar(@rv),
-			       'line' => $lnum };
-		push(@rv, $asn);
+if (!scalar(@list_assigns_cache)) {
+	local @rv;
+	local $lnum = 0;
+	open(ASSIGNS, $qmail_assigns_file);
+	while(<ASSIGNS>) {
+		s/\r|\n//g;
+		last if ($_ eq '.');
+		local @line = split(/:/, $_, 8);
+		if ($line[0] =~ /^([\+=])(\S*)/) {
+			local $asn = { 'address' => $2,
+				       'mode' => $1,
+				       'user' => $line[1],
+				       'uid' => $line[2],
+				       'gid' => $line[3],
+				       'home' => $line[4],
+				       'dash' => $line[5],
+				       'preext' => $line[6],
+				       'idx' => scalar(@rv),
+				       'line' => $lnum };
+			push(@rv, $asn);
+			}
+		$lnum++;
 		}
-	$lnum++;
+	close(ASSIGNS);
+	@list_assigns_cache = @rv;
 	}
-close(ASSIGNS);
-return @rv;
+return @list_assigns_cache;
 }
 
 # create_assign(&assign)
 sub create_assign
 {
+&list_assigns();	# force cache init
 &lock_file($qmail_assigns_file);
 local $lref = &read_file_lines($qmail_assigns_file);
 local $dot;
@@ -589,6 +641,11 @@ splice(@$lref, $i, 0, join(":", "$_[0]->{'mode'}$_[0]->{'address'}",
 push(@$lref, ".") if (!$dot);
 &flush_file_lines();
 &unlock_file($qmail_assigns_file);
+
+# Update in memory cache
+$_[0]->{'line'} = @$lref-1;
+$_[0]->{'idx'} = scalar(@list_assigns_cache);
+push(@list_assigns_cache, $_[0]);
 }
 
 # modify_assign(&old, &assign)
@@ -602,6 +659,13 @@ $lref->[$_[0]->{'line'}] = join(":", "$_[1]->{'mode'}$_[1]->{'address'}",
 				$_[1]->{'preext'}, '');
 &flush_file_lines();
 &unlock_file($qmail_assigns_file);
+
+# Update in memory cache
+if ($_[0] ne $_[1]) {
+	$_[1]->{'line'} = $_[0]->{'line'};
+	$_[1]->{'idx'} = $_[0]->{'idx'};
+	$list_assigns_cache[$_[1]->{'idx'}] = $_[1];
+	}
 }
 
 # delete_assign(&assign)
@@ -612,6 +676,13 @@ local $lref = &read_file_lines($qmail_assigns_file);
 splice(@$lref, $_[0]->{'line'}, 1);
 &flush_file_lines();
 &unlock_file($qmail_assigns_file);
+
+# delete from cache
+splice(@list_assigns_cache, $_[0]->{'idx'}, 1);
+foreach my $v (@list_assigns_cache) {
+	$v->{'line'}-- if ($v->{'line'} > $_[0]->{'line'});
+	$v->{'idx'}-- if ($v->{'idx'} > $_[0]->{'idx'});
+	}
 }
 
 # assign_form([&assign])
@@ -683,7 +754,50 @@ else {
 # Tells qmail to reload its configuration files by sending a HUP signal
 sub restart_qmail
 {
-&kill_byname_logged("qmail-send", HUP);
+if ($config{'apply_cmd'}) {
+	&system_logged("($config{'apply_cmd'}) >/dev/null 2>&1 </dev/null");
+	}
+else {
+	&kill_byname_logged("qmail-send", HUP);
+	}
+}
+
+# stop_qmail()
+# Attempts to stop qmail, and returns an error message if it fails
+sub stop_qmail
+{
+if ($config{'stop_cmd'}) {
+	&system_logged("( $config{'stop_cmd'} ) >/dev/null 2>&1 </dev/null &");
+	return undef;
+	}
+else {
+	if (&kill_byname_logged("qmail-send", TERM)) {
+		return undef;
+		}
+	else {
+		return $text{'stop_err'};
+		}
+	}
+}
+
+# start_qmail()
+# Attempts to start qmail, and returns an error message if it fails
+sub start_qmail
+{
+if ($config{'start_cmd'}) {
+	&system_logged("( $config{'start_cmd'} ) >/dev/null 2>&1 </dev/null &");
+	}
+else {
+	&system_logged("$qmail_start_cmd >/dev/null 2>&1 </dev/null &");
+	}
+return undef;
+}
+
+# Returns the PID of qmail-send if running
+sub is_qmail_running
+{
+local ($pid) = &find_byname("^\\S*qmail-send");
+return kill(0, $pid) ? $pid : undef;
 }
 
 1;
