@@ -52,13 +52,15 @@ return $_[0]->[3] =~ /quota/ ? 3 : 0;
 # be 1 for user only, 2 for group only or 3 for user and group
 sub quotaon
 {
+return if (&is_readonly_mode());
 local($out, $qf, @qfile, $flags);
 if ($_[1]%2 == 1) {
 	# turn on user quotas
 	$qf = "$_[0]/quota.user";
 	if (!(-r $qf)) {
-		open(QUOTAFILE, "> $qf"); close(QUOTAFILE);
-		chmod(0600, $qf);
+		&open_tempfile(QUOTAFILE, ">$qf", 0, 1);
+		&close_tempfile(QUOTAFILE);
+		&set_ownership_permissions(undef, undef, 0600, $qf);
 		&system_logged("$config{'quotacheck_command'} $_[0]");
 		}
 	$out = &backquote_logged("$config{'user_quotaon_command'} $_[0] 2>&1");
@@ -68,8 +70,9 @@ if ($_[1] > 1) {
 	# turn on group quotas
 	$qf = "$_[0]/quota.group";
 	if (!(-r $qf)) {
-		open(QUOTAFILE, "> $qf"); close(QUOTAFILE);
-		chmod(0600, $qf);
+		&open_tempfile(QUOTAFILE, ">$qf", 0, 1);
+		&close_tempfile(QUOTAFILE);
+		&set_ownership_permissions(undef, undef, 0600, $qf);
 		&system_logged("$config{'quotacheck_command'} $_[0]");
 		}
 	$out = &backquote_logged("$config{'group_quotaon_command'} $_[0] 2>&1");
@@ -82,6 +85,7 @@ return undef;
 # Turn off quotas for some filesystem
 sub quotaoff
 {
+return if (&is_readonly_mode());
 local($out);
 if ($_[1]%2 == 1) {
 	$out = &backquote_logged("$config{'user_quotaoff_command'} $_[0] 2>&1");
@@ -100,14 +104,16 @@ return undef;
 sub user_filesystems
 {
 local($n, $_, %mtab);
-open(QUOTA, "$config{'user_quota_command'} $_[0] |");
+open(QUOTA, "$config{'user_quota_command'} ".quotemeta($_[0])." |");
 $n=0; while(<QUOTA>) {
 	chop;
 	if (/^(Disk|\s+Filesystem)/) { next; }
 	if (/^(\S+)$/) {
 		# Bogus wrapped line
 		$filesys{$n,'filesys'} = $1;
-		<QUOTA>=~/^.{15}(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/;
+		local $nl = <QUOTA>;
+		$nl =~ /^\s+(\S+)\s+(\S+)\s+(\S+)(.{8}\s+)(\S+)\s+(\S+)\s+(\S+)(.*)/ ||
+		  $nl =~ /^.{15}(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/;
 		$filesys{$n,'ublocks'} = int($1);
 		$filesys{$n,'sblocks'} = int($2);
 		$filesys{$n,'hblocks'} = int($3);
@@ -116,7 +122,8 @@ $n=0; while(<QUOTA>) {
 		$filesys{$n,'hfiles'} = int($6);
 		$n++;
 		}
-	elsif (/^(.{15})(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/) {
+	elsif (/^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(.{8}\s+)(\S+)\s+(\S+)\s+(\S+)(.*)/ ||
+	       /^(.{15})(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/) {
 		$filesys{$n,'ublocks'} = int($2);
 		$filesys{$n,'sblocks'} = int($3);
 		$filesys{$n,'hblocks'} = int($4);
@@ -138,14 +145,16 @@ return $n;
 sub group_filesystems
 {
 local($n, $_, %mtab);
-open(QUOTA, "$config{'group_quota_command'} $_[0] |");
+open(QUOTA, "$config{'group_quota_command'} ".quotemeta($_[0])." |");
 $n=0; while(<QUOTA>) {
 	chop;
 	if (/^(Disk|\s+Filesystem)/) { next; }
 	if (/^(\S+)$/) {
 		# Bogus wrapped line
 		$filesys{$n,'filesys'} = $1;
-		<QUOTA>=~/^.{15}(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/;
+		local $nl = <QUOTA>;
+		$nl =~ /^\s+(\S+)\s+(\S+)\s+(\S+)(.{8}\s+)(\S+)\s+(\S+)\s+(\S+)(.*)/ ||
+		  $nl =~ /^.{15}(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/;
 		$filesys{$n,'ublocks'} = int($1);
 		$filesys{$n,'sblocks'} = int($2);
 		$filesys{$n,'hblocks'} = int($3);
@@ -154,7 +163,8 @@ $n=0; while(<QUOTA>) {
 		$filesys{$n,'hfiles'} = int($6);
 		$n++;
 		}
-	elsif (/^(.{15})(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/) {
+	elsif (/^\s*(\S+)\s+(\S+)\s+(\S+)\s+(\S+)(.{8}\s+)(\S+)\s+(\S+)\s+(\S+)(.*)/ ||
+	       /^(.{15})(.{8}).(.{7})(.{8}).{8}(.{8}).(.{7})(.{8})/) {
 		$filesys{$n,'ublocks'} = int($2);
 		$filesys{$n,'sblocks'} = int($3);
 		$filesys{$n,'hblocks'} = int($4);
@@ -181,14 +191,18 @@ if ($?) { return -1; }
 @rep = split(/\n/, $rep);
 @rep = grep { !/^root\s/ } @rep[3..$#rep];
 for($n=0; $n<@rep; $n++) {
-	if ($rep[$n] =~ /(\S+)\s*[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s.{0,15}\s(\d+)\s+(\d+)\s+(\d+)/ || $rep[$n] =~ /(\S+)\s+..(.{8})(.{8})(.{8}).{7}(.{8})(.{8})(.{8})/) {
+	if ($rep[$n] =~ /(\S+)\s*[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s(.{0,15})\s(\d+)\s+(\d+)\s+(\d+)(.*)/ || $rep[$n] =~ /(\S+)\s+..(.{8})(.{8})(.{8})(.{7})(.{8})(.{8})(.{8})(.*)/) {
 		$user{$n,'user'} = $1;
 		$user{$n,'ublocks'} = int($2);
 		$user{$n,'sblocks'} = int($3);
 		$user{$n,'hblocks'} = int($4);
-		$user{$n,'ufiles'} = int($5);
-		$user{$n,'sfiles'} = int($6);
-		$user{$n,'hfiles'} = int($7);
+		$user{$n,'gblocks'} = $5;
+		$user{$n,'ufiles'} = int($6);
+		$user{$n,'sfiles'} = int($7);
+		$user{$n,'hfiles'} = int($8);
+		$user{$n,'gfiles'} = $9;
+		$user{$n,'gblocks'} = &trunc_space($user{$n,'gblocks'});
+		$user{$n,'gfiles'} = &trunc_space($user{$n,'gfiles'});
 		}
 	}
 return $n;
@@ -205,14 +219,18 @@ if ($?) { return -1; }
 @rep = split(/\n/, $rep);
 @rep = @rep[3..$#rep];
 for($n=0; $n<@rep; $n++) {
-	if ($rep[$n] =~ /(\S+)\s*[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s.{0,15}\s(\d+)\s+(\d+)\s+(\d+)/ || $rep[$n] =~ /(\S+)\s+..(.{8})(.{8})(.{8}).{7}(.{8})(.{8})(.{8})/) {
+	if ($rep[$n] =~ /(\S+)\s*[\-\+]{2}\s+(\d+)\s+(\d+)\s+(\d+)\s(.{0,15})\s(\d+)\s+(\d+)\s+(\d+)(.*)/ || $rep[$n] =~ /(\S+)\s+..(.{8})(.{8})(.{8})(.{7})(.{8})(.{8})(.{8})/) {
 		$group{$n,'group'} = $1;
 		$group{$n,'ublocks'} = int($2);
 		$group{$n,'sblocks'} = int($3);
 		$group{$n,'hblocks'} = int($4);
-		$group{$n,'ufiles'} = int($5);
-		$group{$n,'sfiles'} = int($6);
-		$group{$n,'hfiles'} = int($7);
+		$group{$n,'gblocks'} = $5;
+		$group{$n,'ufiles'} = int($6);
+		$group{$n,'sfiles'} = int($7);
+		$group{$n,'hfiles'} = int($8);
+		$group{$n,'gfiles'} = $9;
+		$group{$n,'gblocks'} = &trunc_space($group{$n,'gblocks'});
+		$group{$n,'gfiles'} = &trunc_space($group{$n,'gfiles'});
 		}
 	}
 return $n;
@@ -224,9 +242,15 @@ sub edit_quota_file
 local($rv, $line, %mtab, @m);
 @line = split(/\n/, $_[0]);
 for($i=0; $i<@line; $i++) {
-	if ($line[$i] =~ /^(\S+): blocks in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/ && $1 eq $_[1]) {
-		# found lines to change
-		$rv .= "$1: blocks in use: $2, limits (soft = $_[2], hard = $_[3])\n";
+	if ($line[$i] =~ /^(\S+): (blocks|kbytes) in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/ && $1 eq $_[1]) {
+		# found lines to change, old style
+		$rv .= "$1: $2 in use: $3, limits (soft = $_[2], hard = $_[3])\n";
+		$line[++$i] =~ /^\s*inodes in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/;
+		$rv .= "\tinodes in use: $1, limits (soft = $_[4], hard = $_[5])\n";
+		}
+	elsif ($line[$i] =~ /^(\S+): in use: (\d+)k, limits \(soft = (\d+)k, hard = (\d+)k\)$/ && $1 eq $_[1]) {
+		# found lines to change, new style
+		$rv .= "$1: in use: ${2}k, limits (soft = $_[2]k, hard = $_[3]k)\n";
 		$line[++$i] =~ /^\s*inodes in use: (\d+), limits \(soft = (\d+), hard = (\d+)\)$/;
 		$rv .= "\tinodes in use: $1, limits (soft = $_[4], hard = $_[5])\n";
 		}
@@ -249,7 +273,8 @@ return undef;
 sub copy_user_quota
 {
 for($i=1; $i<@_; $i++) {
-	$out = &backquote_logged("$config{'user_copy_command'} $_[0] $_[$i] 2>&1");
+	$out = &backquote_logged("$config{'user_copy_command'} ".
+				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
 	if ($?) { return $out; }
 	}
 return undef;
@@ -260,7 +285,8 @@ return undef;
 sub copy_group_quota
 {
 for($i=1; $i<@_; $i++) {
-	$out = &backquote_logged("$config{'group_copy_command'} $_[0] $_[$i] 2>&1");
+	$out = &backquote_logged("$config{'group_copy_command'} ".
+				quotemeta($_[0])." ".quotemeta($_[$i])." 2>&1");
 	if ($?) { return $out; }
 	}
 return undef;
@@ -329,6 +355,14 @@ return ($text{'grace_seconds'}, $text{'grace_minutes'}, $text{'grace_hours'},
 	$text{'grace_days'});
 }
 
+# fs_block_size(dir, device, filesystem)
+# Returns the size of blocks on some filesystem, or undef if unknown.
+# Always 1024, because the ENV setting above forces this.
+sub fs_block_size
+{
+return 1024;
+}
+
 %name_to_unit = ( "second", 0, "seconds", 0,
 		  "minute", 1, "minutes", 1,
 		  "hour", 2, "hours", 2,
