@@ -24,18 +24,28 @@ foreach $z (sort { $a->{'value'} cmp $b->{'value'} } @servers) {
 print "</select></td></tr>\n";
 
 print "<tr> <td><b>$text{'acl_edit'}</b></td> <td>\n";
-printf "<input type=radio name=edit value=1 %s> $text{'yes'}\n",
-        $_[0]->{'edit'} ? "checked" : "";
-printf "<input type=radio name=edit value=0 %s> $text{'no'}</td>\n",
-        $_[0]->{'edit'} ? "" : "checked";
+print &ui_yesno_radio("edit", $_[0]->{'edit'}),"</td>\n";
 
 print "<td><b>$text{'acl_find'}</b></td> <td>\n";
-printf "<input type=radio name=find value=1 %s> $text{'yes'}\n",
-        $_[0]->{'find'} ? "checked" : "";
-printf "<input type=radio name=find value=0 %s> $text{'no'}</td> </tr>\n",
-        $_[0]->{'find'} ? "" : "checked";
+print &ui_yesno_radio("find", $_[0]->{'find'}),"</td> </tr>\n";
 
+print "<tr> <td><b>$text{'acl_auto'}</b></td> <td>\n";
+print &ui_yesno_radio("auto", $_[0]->{'auto'}),"</td>\n";
 
+print "<td><b>$text{'acl_add'}</b></td> <td>\n";
+print &ui_yesno_radio("add", $_[0]->{'add'}),"</td> </tr>\n";
+
+print "<tr> <td><b>$text{'acl_forcefast'}</b></td> <td>\n";
+print &ui_yesno_radio("forcefast", $_[0]->{'forcefast'}),"</td>\n";
+
+print "<td><b>$text{'acl_forcetype'}</b></td> <td>\n";
+print &ui_yesno_radio("forcetype", $_[0]->{'forcetype'}),"</td> </tr>\n";
+
+print "<tr> <td><b>$text{'acl_forcelink'}</b></td> <td>\n";
+print &ui_yesno_radio("forcelink", $_[0]->{'forcelink'}),"</td>\n";
+
+print "<td><b>$text{'acl_links'}</b></td> <td>\n";
+print &ui_yesno_radio("links", $_[0]->{'links'}),"</td> </tr>\n";
 }
 
 # acl_security_save(&options)
@@ -50,5 +60,11 @@ else {
         }
 $_[0]->{'edit'} = $in{'edit'};
 $_[0]->{'find'} = $in{'find'};
+$_[0]->{'auto'} = $in{'auto'};
+$_[0]->{'add'} = $in{'add'};
+$_[0]->{'forcefast'} = $in{'forcefast'};
+$_[0]->{'forcetype'} = $in{'forcetype'};
+$_[0]->{'forcelink'} = $in{'forcelink'};
+$_[0]->{'links'} = $in{'links'};
 }
 
