@@ -173,11 +173,12 @@ if [ "$upgrading" = 1 ]; then
 	$perl "$wadir/newmods.pl" $config_dir $allmods
 
 	# Update miniserv.conf with new root directory and mime types file
-	grep -v "^root=" $config_dir/miniserv.conf | grep -v "^mimetypes=" >$tempdir/$$.miniserv.conf
+	grep -v "^root=" $config_dir/miniserv.conf | grep -v "^mimetypes=" | grep -v "^server=" >$tempdir/$$.miniserv.conf
 	if [ $? != "0" ]; then exit 1; fi
 	mv $tempdir/$$.miniserv.conf $config_dir/miniserv.conf
 	echo "root=$wadir" >> $config_dir/miniserv.conf
 	echo "mimetypes=$wadir/mime.types" >> $config_dir/miniserv.conf
+	echo "server=MiniServ/$ver" >> $config_dir/miniserv.conf
 	grep logout= $config_dir/miniserv.conf >/dev/null
 	if [ $? != "0" ]; then
 		echo "logout=$config_dir/logout-flag" >> $config_dir/miniserv.conf
@@ -294,6 +295,9 @@ else
 		exit 8
 	fi
 	$perl -e '$c = crypt("xx", "yy"); exit($c ? 0 : 1)'
+	if [ $? != "0" ]; then
+		$perl -e 'use Crypt::UnixCrypt' >/dev/null 2>&1
+	fi
 	if [ $? != "0" ]; then
 		echo "ERROR: Perl crypt function does not work. Maybe Perl has"
 		echo "not been properly installed on your system"
@@ -444,12 +448,12 @@ else
 			read atbootyn
 			if [ "$atbootyn" = "y" -o "$atbootyn" = "Y" ]; then
 				atboot=1
+				makeboot=1
 			fi
 		else
 			echo "Webmin does not support being started at boot time on your system."
 		fi
 	fi
-	makeboot=$atboot
 
 	# Copy files to target directory
 	echo "***********************************************************************"
@@ -480,7 +484,11 @@ else
 	echo "env_WEBMIN_VAR=$var_dir" >> $cfile
 	echo "atboot=$atboot" >> $cfile
 	echo "logout=$config_dir/logout-flag" >> $cfile
-	echo "listen=10000" >> $cfile
+	if [ "$listen" != "" ]; then
+		echo "listen=$listen" >> $cfile
+	else
+		echo "listen=10000" >> $cfile
+	fi
 	echo "denyfile=\\.pl\$" >> $cfile
 	echo "log=1" >> $cfile
 	echo "blockhost_failures=5" >> $cfile
@@ -497,6 +505,13 @@ else
 	if [ "$pam" != "" ]; then
 		echo "pam=$pam" >> $cfile
 	fi
+	echo premodules=WebminCore >> $cfile
+	echo "server=MiniServ/$ver" >> $cfile
+
+	# Append package-specific info to config file
+	if [ -r "$wadir/miniserv-conf" ]; then
+		cat "$wadir/miniserv-conf" >>$cfile
+	fi
 
 	md5pass=`$perl -e 'print crypt("test", "\\$1\\$A9wB3O18\\$zaZgqrEmb9VNltWTL454R/") eq "\\$1\\$A9wB3O18\\$zaZgqrEmb9VNltWTL454R/" ? "1\n" : "0\n"'`
 
@@ -518,7 +533,7 @@ else
 	if [ "$?" = "0" ]; then
 		# We can generate a new SSL key for this host
 		host=`hostname`
-		openssl req -newkey rsa:512 -x509 -nodes -out $tempdir/cert -keyout $tempdir/key -days 1825 >/dev/null 2>&1 <<EOF
+		openssl req -newkey rsa:2048 -x509 -nodes -out $tempdir/cert -keyout $tempdir/key -days 1825 >/dev/null 2>&1 <<EOF
 .
 .
 .
@@ -569,7 +584,7 @@ if [ "$noperlpath" = "" ]; then
 fi
 
 echo "Creating start and stop scripts.."
-rm -f $config_dir/stop $config_dir/start $config_dir/restart
+rm -f $config_dir/stop $config_dir/start $config_dir/restart $config_dir/reload
 echo "#!/bin/sh" >>$config_dir/start
 echo "echo Starting Webmin server in $wadir" >>$config_dir/start
 echo "trap '' 1" >>$config_dir/start
@@ -595,7 +610,12 @@ echo "kill \`cat \$pidfile\`" >>$config_dir/stop
 echo "#!/bin/sh" >>$config_dir/restart
 echo "$config_dir/stop && $config_dir/start" >>$config_dir/restart
 
-chmod 755 $config_dir/start $config_dir/stop $config_dir/restart
+echo "#!/bin/sh" >>$config_dir/reload
+echo "echo Reloading Webmin server in $wadir" >>$config_dir/reload
+echo "pidfile=\`grep \"^pidfile=\" $config_dir/miniserv.conf | sed -e 's/pidfile=//g'\`" >>$config_dir/reload
+echo "kill -USR1 \`cat \$pidfile\`" >>$config_dir/reload
+
+chmod 755 $config_dir/start $config_dir/stop $config_dir/restart $config_dir/reload
 echo "..done"
 echo ""
 
@@ -604,7 +624,7 @@ if [ "$upgrading" = 1 ]; then
 else
 	echo "Copying config files.."
 fi
-$perl "$wadir/copyconfig.pl" "$os_type" "$os_version" "$wadir" $config_dir "" $allmods
+newmods=`$perl "$wadir/copyconfig.pl" "$os_type/$real_os_type" "$os_version/$real_os_version" "$wadir" $config_dir "" $allmods`
 if [ "$upgrading" != 1 ]; then
 	# Store the OS and version
 	echo "os_type=$os_type" >> $config_dir/config
@@ -627,6 +647,14 @@ if [ "$upgrading" != 1 ]; then
 
 	# Turn on logging by default
 	echo "log=1" >> $config_dir/config
+
+	# Use licence module specified by environment variable
+	if [ "$licence_module" != "" ]; then
+		echo licence_module=$licence_module >>$config_dir/config
+	fi
+
+	# Disallow unknown referers by default
+	echo "referers_none=1" >>$config_dir/config
 else
 	# one-off hack to set log variable in config from miniserv.conf
 	grep log= $config_dir/config >/dev/null
@@ -635,6 +663,12 @@ else
 		grep logtime= $config_dir/miniserv.conf >> $config_dir/config
 		grep logclear= $config_dir/miniserv.conf >> $config_dir/config
 	fi
+
+	# Disallow unknown referers if not set
+	grep referers_none= $config_dir/config >/dev/null
+	if [ "$?" != "0" ]; then
+		echo "referers_none=1" >>$config_dir/config
+	fi
 fi
 echo $ver > $config_dir/version
 echo "..done"
@@ -664,8 +698,21 @@ fi
 oldthemeline=`grep "^theme=" $config_dir/config`
 oldtheme=`echo $oldthemeline | sed -e 's/theme=//g'`
 if [ "$theme" != "" ] && [ "$oldthemeline" = "" ] && [ -d "$wadir/$theme" ]; then
-	echo "theme=$theme" >> $config_dir/config
-	echo "preroot=$theme" >> $config_dir/miniserv.conf
+	themelist=$theme
+fi
+
+# Set a special overlay if none was set before
+if [ "$overlay" = "" ]; then
+	overlay=`cat "$wadir/defaultoverlay" 2>/dev/null`
+fi
+if [ "$overlay" != "" ] && [ "$theme" != "" ] && [ -d "$wadir/$overlay" ]; then
+	themelist="$themelist $overlay"
+fi
+
+# Apply the theme and maybe overlay
+if [ "$themelist" != "" ]; then
+	echo "theme=$themelist" >> $config_dir/config
+	echo "preroot=$themelist" >> $config_dir/miniserv.conf
 fi
 
 # Set the product field in the global config
@@ -711,27 +758,41 @@ EOF
 fi
 
 echo "Changing ownership and permissions .."
-chown -R root $config_dir
-chgrp -R bin $config_dir
-chmod -R og-rw $config_dir
-chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/config >/dev/null 2>&1
-chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/autoreply.pl >/dev/null 2>&1
-chmod 755 $config_dir/{sendmail,qmailadmin,postfix}*/filter.pl >/dev/null 2>&1
-chmod 755 $config_dir/squid*/squid-auth.pl >/dev/null 2>&1
-chmod 755 $config_dir/squid*/users >/dev/null 2>&1
-chmod 755 $config_dir/cron*/range.pl >/dev/null 2>&1
+# Make all config dirs non-world-readable
+for m in $newmods; do
+	chown -R root $config_dir/$m
+	chgrp -R bin $config_dir/$m
+	chmod -R og-rw $config_dir/$m
+done
+# Make miniserv config files non-world-readable
+for f in miniserv.conf miniserv.pem miniserv.users; do
+	chown -R root $config_dir/$f
+	chgrp -R bin $config_dir/$f
+	chmod -R og-rw $config_dir/$f
+done
 chmod +r $config_dir/version
 if [ "$nochown" = "" ]; then
+	# Make program directory non-world-writable, but executable
 	chown -R root "$wadir"
 	chgrp -R bin "$wadir"
 	chmod -R og-w "$wadir"
 	chmod -R a+rx "$wadir"
 fi
 if [ $var_dir != "/var" ]; then
+	# Make log directory non-world-readable or writable
 	chown -R root $var_dir
 	chgrp -R bin $var_dir
 	chmod -R og-rwx $var_dir
 fi
+# Fix up bad permissions from some older installs
+for m in ldap-client ldap-server ldap-useradmin mailboxes mysql postgresql servers virtual-server; do
+	if [ -d "$config_dir/$m" ]; then
+		chown root $config_dir/$m
+		chgrp bin $config_dir/$m
+		chmod og-rw $config_dir/$m
+		chmod og-rw $config_dir/$m/config 2>/dev/null
+	fi
+done
 echo "..done"
 echo ""
 
@@ -749,6 +810,14 @@ if [ "$nopostinstall" = "" ]; then
 	echo ""
 fi
 
+# Enable background collection
+if [ "$upgrading" != 1 -a -r $config_dir/system-status/enable-collection.pl ]; then
+	echo "Enabling background status collection .."
+	$config_dir/system-status/enable-collection.pl 5
+	echo "..done"
+	echo ""
+fi
+
 # Run package-defined post-install script
 if [ -r "$srcdir/setup-post.sh" ]; then
 	. "$srcdir/setup-post.sh"
