@@ -1,12 +1,15 @@
 # squid-lib.pl
 # Functions for configuring squid.conf
 
-do '../web-lib.pl';
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
 do 'parser-lib.pl';
 %access = &get_module_acl();
 $auth_program = "$module_config_directory/squid-auth.pl";
 $auth_database = "$module_config_directory/users";
+@caseless_acl_types = ( "url_regex", "urlpath_regex", "proxy_auth_regex",
+			"srcdom_regex", "dstdom_regex", "ident_regex" );
 
 # Get the squid version
 if (open(VERSION, "$module_config_directory/version")) {
@@ -14,23 +17,6 @@ if (open(VERSION, "$module_config_directory/version")) {
 	close(VERSION);
 	}
 
-# icons_table(&links, &titles, &icons)
-sub icons_table
-{
-local($i);
-print "<table width=100% cellpadding=5> <tr>\n";
-for($i=0; $i<@{$_[0]}; $i++) {
-	if ($i%4 == 0) { print "<tr>\n"; }
-	print "<td width=25% align=center valign=top>\n";
-	&generate_icon($_[2]->[$i], $_[1]->[$i], $_[0]->[$i]);
-	print "</td>\n";
-        if ($i%4 == 3) { print "</tr>\n"; }
-        }
-while($i++%4) { print "<td width=25%></td>\n"; }
-print "</table><p>\n";
-}
-
-
 # choice_input(text, name, &config, default, [display, option]+)
 # Display a number of radio buttons for selecting some option
 sub choice_input
@@ -254,9 +240,9 @@ else {
 sub opt_bytes_input
 {
 local($v, $rv, $u, %ss);
-%ss = (	"KB"=>	$text{'lib_kb'},
-	"MB"=>	$text{'lib_mb'},
-	"GB"=>	$text{'lib_gb'} );
+@ss = (	[ "KB", $text{'lib_kb'} ],
+	[ "MB", $text{'lib_mb'} ],
+	[ "GB", $text{'lib_gb'} ] );
 $v = &find_config($_[1], $_[2]);
 $rv = "<td valign=top><b>$_[0]</b></td> <td valign=top nowrap>\n";
 $rv .= sprintf "<input type=radio name=$_[1]_def value=1 %s> $_[3]\n",
@@ -266,10 +252,12 @@ $rv .= sprintf "<input type=radio name=$_[1]_def value=0 %s> ",
 $rv .= sprintf "<input name=$_[1] size=$_[4] value=\"%s\">\n",
 	$v ? $v->{'values'}->[0] : "";
 $rv .= "<select name=$_[1]_u>\n";
-foreach $u (keys %ss) {
-	$rv .= sprintf "<option value=$u %s>$ss{$u}\n",
-		$v && $v->{'values'}->[1] eq $u ? "selected" : "";
+foreach $u (@ss) {
+	$rv .= sprintf "<option value=$u->[0] %s>$u->[1]\n",
+		$v && $v->{'values'}->[1] eq $u->[0] ? "selected" : "";
 	}
+$rv .= sprintf "<option value='' %s>bytes\n",
+	$v && $v->{'values'}->[1] eq "" ? "selected" : "";
 $rv .= "</select></td>\n";
 return $rv;
 }
@@ -337,30 +325,46 @@ if ($squid_version >= 2.5) {
 # Returns HTML for a link to put in the top-right corner of every page
 sub restart_button
 {
-local($conf, $pidstruct, $pidfile, $pid, $args);
-$conf = &get_config();
-if ($pidstruct = &find_config("pid_filename", $conf)) {
-	$pidfile = $pidstruct->{'values'}->[0];
-	}
-else { $pidfile = $config{'pid_file'}; }
-if ($pidfile eq "none") {
-	($pid) = &find_byname("squid");
-	}
-else {
-	if (open(PID, $pidfile)) {
-		<PID> =~ /(\d+)/; $pid = $1;
-		close(PID);
-		}
-	}
-$args = "redir=".&urlize(&this_url())."&pid=$pid";
-if ($pid && kill(0, $pid)) {
-	return "<a href=\"restart.cgi?$args\">$text{'lib_buttac'}</a><br>\n".
+return undef if ($config{'restart_pos'} == 2);
+local $pid = &is_squid_running();
+local $args = "redir=".&urlize(&this_url())."&pid=$pid";
+if ($pid) {
+	return ($access{'restart'} ? "<a href=\"restart.cgi?$args\">$text{'lib_buttac'}</a><br>\n" : "").
 	       ($access{'start'} ? "<a href=\"stop.cgi?$args\">$text{'lib_buttss'}</a>\n" : "");
 	}
 else {
-	return $access{'start'} ? "<a href=\"start.cgi?$args\">$text{'lib_buttss1'}</a><br>\n" : "";
+	return $access{'start'} ? "<a href=\"start.cgi?$args\">$text{'lib_buttss1'}</a>\n" : "";
 	}
-close(PID);
+}
+
+# is_squid_running()
+# Returns the process ID if squid is running
+sub is_squid_running
+{
+local $conf = &get_config();
+
+# Find all possible PID files
+local @pidfiles;
+local $pidstruct = &find_config("pid_filename", $conf);
+push(@pidfiles, $pidstruct->{'values'}->[0]) if ($pidstruct);
+local $def_pidstruct = &find_config("pid_filename", $conf);
+push(@pidfiles, $def_pidstruct->{'values'}->[0]) if ($def_pidstruct);
+push(@pidfiles, $config{'pid_file'}) if ($config{'pid_file'});
+@pidfiles = grep { $_ ne "none" } @pidfiles;
+
+# Try check one
+foreach my $pidfile (@pidfiles) {
+	my $pid = &check_pid_file($pidfile);
+	return $pid if ($pid);
+	}
+
+if (!@pidfiles) {
+	# Fall back to checking for Squid process
+	local ($pid) = &find_byname("squid");
+	return $pid;
+	}
+
+return 0;
 }
 
 # this_url()
@@ -426,7 +430,8 @@ else { $pidfile = $config{'pid_file'}; }
 push(@list, $pidfile);
 
 # add other log directories
-foreach $d ("cache_access_log","cache_log","cache_store_log","cache_swap_log") {
+foreach $d ("cache_access_log", "access_log", "cache_log",
+	    "cache_store_log", "cache_swap_log") {
 	if (($str = &find_config($d, $_[2])) &&
 	    $str->{'values'}->[0] =~ /^(\S+)\/[^\/]+$/) {
 		push(@list, $1);
@@ -496,7 +501,7 @@ return $rv;
 # caches list
 sub check_cache
 {
-local (@cachestruct, @caches, $c);
+local (@cachestruct, @caches, $c, $coss);
 if (@cachestruct = &find_config("cache_dir", $_[0])) {
 	if ($squid_version >= 2.3) {
 		@caches = map { $_->{'values'}->[1] } @cachestruct;
@@ -504,16 +509,74 @@ if (@cachestruct = &find_config("cache_dir", $_[0])) {
 	else {
 		@caches = map { $_->{'values'}->[0] } @cachestruct;
 		}
+	($coss) = grep { $_->{'values'}->[0] eq "coss" } @cachestruct;
 	}
 else {
 	@caches = ( $config{'cache_dir'} );
 	}
 @{$_[1]} = @caches;
-foreach $c (@caches) {
-	return 0 if (!-d $c || !-d "$c/00");
+if ($coss) {
+	# Allow COSS files too
+	foreach $c (@caches) {
+		return 0 if (!-f $c && (!-d $c || !-d "$c/00"));
+		}
+	}
+else {
+	# Check for dirs only
+	foreach $c (@caches) {
+		return 0 if (!-d $c || !-d "$c/00");
+		}
 	}
 return 1;
 }
 
+# get_squid_port()
+# Returns the port Squid is listening on
+sub get_squid_port
+{
+local $conf = &get_config();
+local $port;
+if ($squid_version >= 2.3) {
+	local ($p, $v);
+	LOOP: foreach $p (&find_config("http_port", $conf)) {
+		foreach $v (@{$p->{'values'}}) {
+			if ($v =~ /^(\d+)$/) {
+				$port = $1;
+				}
+			elsif ($v =~ /^(\S+):(\d+)$/) {
+				$port = $2;
+				}
+			last LOOP if ($port);
+			}
+		}
+	}
+else {
+	$port = &find_value("http_port", $conf);
+	}
+return defined($port) ? $port : 3128;
+}
+
+# apply_configuration()
+# Activate the current Squid configuration
+sub apply_configuration
+{
+if ($config{'squid_restart'}) {
+	local $out = &backquote_logged("$config{'squid_restart'} 2>&1");
+	return "<pre>$out</pre>" if ($?);
+	}
+else {
+	$out = &backquote_logged("$config{'squid_path'} -f $config{'squid_conf'} -k reconfigure 2>&1");
+	return "<pre>$out</pre>" if ($? && $out !~ /warning/i);
+	}
+return undef;
+}
+
+# list_cachemgr_actions()
+# Returns a list of actions for use in the cachemgr_passwd directive
+sub list_cachemgr_actions
+{
+return ("5min" ,"60min" ,"asndb" ,"authenticator" ,"cbdata" ,"client_list" ,"comm_incoming" ,"config" ,"counters" ,"delay" ,"digest_stats" ,"dns" ,"events" ,"filedescriptors" ,"fqdncache" ,"histograms" ,"http_headers" ,"info" ,"io" ,"ipcache" ,"mem" ,"menu" ,"netdb" ,"non_peers" ,"objects" ,"offline_toggle" ,"pconn" ,"peer_select" ,"redirector" ,"refresh" ,"server_list" ,"shutdown" ,"store_digest" ,"storedir" ,"utilization" ,"via_headers" ,"vm_objects");
+}
+
 1;
 
