@@ -15,10 +15,8 @@ local $user = $_[0]->{'user'};
 local $pass = $_[0]->{'plainpass'};
 
 local $conf = &get_config();
-local $authprog = &find_value("authenticate_program", $conf);
-unless ( $authprog ) { return; }
-$authprog =~ /(\S+)\s+(\/\S+)$/ || return;
-$file = $2;
+local $file = &get_auth_file($conf);
+return if (!$file);
 
 &lock_file($file);
 local @users = &list_auth_users($file);
@@ -38,15 +36,13 @@ close(FILE);
 # Delete this Squid user if in sync
 sub useradmin_delete_user
 {
-local $conf = &get_config();
 return if (!$config{'sync_delete'});
 
 local $name = $_[0]->{'user'};
 
-local $authprog = &find_value("authenticate_program", $conf);
-unless ( $authprog ) { return; }
-$authprog =~ /(\S+)\s+(\/\S+)$/ || return;
-$file = $2;
+local $conf = &get_config();
+local $file = &get_auth_file($conf);
+return if (!$file);
 
 &lock_file($file);
 local @users = &list_auth_users($file);
@@ -76,17 +72,14 @@ if ($config{'crypt_conf'} == 1) {
 	return if ($@);
         }
 local $conf = &get_config();
+local $file = &get_auth_file($conf);
+return if (!$file);
 
 local $name = $_[0]->{'user'};
 local $oldname = $_[1]->{'user'};
 local $pass = $_[0]->{'plainpass'};
 return if ($name eq $oldname && $_[0]->{'passmode'} == 4);
 
-local $authprog = &find_value("authenticate_program", $conf);
-unless ( $authprog ) { return; }
-$authprog =~ /(\S+)\s+(\/\S+)$/ || return;
-$file = $2;
-
 &lock_file($file);
 local @users = &list_auth_users($file);
 local $user;
@@ -121,7 +114,7 @@ sub encryptpwd {
     }
     else {
       $salt = substr(time(), -2);
-      my $pwd = crypt($_[0], $_[1]);
+      my $pwd = &unix_crypt($_[0], $_[1]);
       return $pwd;
       }
   }
