@@ -10,18 +10,23 @@ $sysclock_file = "/etc/sysconfig/clock";
 sub list_timezones
 {
 local @rv;
+local %done;
 &open_readfile(ZONE, $timezones_file) || return ( );
 while(<ZONE>) {
 	s/\r|\n//g;
 	s/^\s*#.*$//;
 	if (/^(\S+)\s+(\S+)\s+(\S+)\s+(\S.*)/) {
 		push(@rv, [ $3, $4 ]);
+		$done{$3}++;
 		}
 	elsif (/^(\S+)\s+(\S+)\s+(\S+)/) {
 		push(@rv, [ $3, undef ]);
+		$done{$3}++;
 		}
 	}
 close(ZONE);
+push(@rv, [ "GMT", "GMT" ]) if (!$done{'GMT'});
+push(@rv, [ "UTC", "UTC" ]) if (!$done{'UTC'});
 return sort { $a->[0] cmp $b->[0] } @rv;
 }
 
