@@ -1,72 +1,14 @@
 #!/usr/local/bin/perl
 
-do "../web-lib.pl";
+BEGIN { push(@INC, ".."); };
+use WebminCore;
 &init_config();
-require '../ui-lib.pl';
 %access = &get_module_acl();
 $cron_cmd = "$module_config_directory/sync.pl";
 if ($config{'zone_style'}) {
 	do "$config{'zone_style'}-lib.pl";
 	}
-
-sub p_link
-{
-    my ( $dest, $text ) = @_;
-    return "<a href=\"". $dest. "\">". $text. "</a>";
-}
-
-sub p_radio
-{
-    my ( $name, $checked, @list ) = @_;
-    local ($out, $size, $i);
-    $size = @list; $i = 0;
-
-    do
-    {
-	$out .= " <input type=radio name=".$name." value=".$list[$i];
-	$out .= " checked" if( $checked eq $list[$i++] );
-	$out .="> ".$list[$i++];
-    } while( $i < $size );
-
-    return $out;
-}
-
-sub p_entry
-{
-    my ( $name, $value, $size ) = @_;
-
-    $size ? return "<input name=". $name. " size=". $size." value=\"". $value."\">" : return "<input name=". $name. " value=\"". $value."\">";
-}
-
-sub p_select_wdl
-{
-  my ( $name, $selected, @list ) = @_;
-  local $out = "<select name=$name>";
-  local $i = 0;
-  local $size = @list;
-
-  do
-  {
-	$out .= "<option value=$list[$i++]";
-	$out .= " selected" if( $selected eq $list[$i] );
-	$out .= ">$list[$i++]";
-  } while( $i < $size );
-  $out .= "</select>";
-
-}
-
-sub p_select
-{
-  my ( $name, $selected, @list ) = @_;
-  local (@newlist, $item);
-
-  foreach $item ( @list )
-  {
-	push( @newlist, $item, $item );
-  }
-
-  p_select_wdl( $name, $selected, @newlist );
-}
+&foreign_require("webmincron");
 
 sub find_cron_job
 {
@@ -77,6 +19,11 @@ local ($job) = grep { $_->{'command'} eq $cron_cmd &&
 return $job;
 }
 
+sub find_webmin_cron_job
+{
+return &webmincron::find_webmin_cron($module_name, 'sync_time_cron');
+}
+
 # sync_time(server, hardware-too)
 # Syncs the system and maybe hardware time with some server. Returns undef
 # on success, or an error message on failure.
@@ -94,7 +41,7 @@ elsif ($?) {
 	local ($err, $serv);
 	foreach $serv (@servs) {
 		$err = undef;
-		&open_socket( $_[0], 37, SOCK, \$err);
+		&open_socket($serv, 37, SOCK, \$err);
 		read(SOCK, $rawtime, 4);
 		close(SOCK);
 		last if (!$err && $rawtime);
@@ -117,14 +64,23 @@ else {
 	}
 
 if ($_[1]) {
-	# Set hardware clock time
-	local @tm = localtime($rawtime);
-	return &set_hardware_time(@tm);
+	# Set hardware clock time to match system time (which is now correct)
+	local $flags = &get_hwclock_flags();
+	local $out = &backquote_logged("hwclock $flags --systohc");
+	return $? ? $out : undef;
 	}
 
 return undef;
 }
 
+# sync_time_cron()
+# Called from webmin cron to sync from the configured server
+sub sync_time_cron
+{
+my $err = &sync_time($config{'timeserver'}, $config{'timeserver_hardware'});
+print STDERR $err;
+}
+
 sub has_timezone
 {
 return 0 if (!defined(&list_timezones));
@@ -171,10 +127,12 @@ else {
 }
 
 # get_hardware_time()
-# Returns the current hardware time, in localtime format
+# Returns the current hardware time, in localtime format. On failure returns
+# an empty array, and sets the global $get_hardware_time_error
 sub get_hardware_time
 {
 local $flags = &get_hwclock_flags();
+$get_hardware_time_error = undef;
 local $out = `hwclock $flags`;
 if ($out =~ /^(\S+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\d+)\s+/) {
 	return ($6, $5, $4, $3, &month_to_number($2), $7-1900, &weekday_to_number($1));
@@ -183,6 +141,9 @@ elsif ($out =~ /^(\S+)\s+(\d+)\s+(\S+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(am|pm)\s+/
 	return ($7, $6, $5+($8 eq 'pm' ? 12 : 0), $2, &month_to_number($3), $4-1900, &weekday_to_number($1));
 	}
 else {
+	$get_hardware_time_error = &text('index_ehwclock',
+			"<tt>".&html_escape("hwclock $flags")."</tt>",
+			"<pre>".&html_escape($out)."</pre>");
 	return ( );
 	}
 }
@@ -200,9 +161,10 @@ sub set_hardware_time
 local ($second, $minute, $hour, $date, $month, $year) = @_;
 $month++;
 $year += 1900;
-local $format = "--set --date=\"$month/$date/$year $hour:$minute:$second\"";
+local $format = "--set --date=".
+		quotemeta("$month/$date/$year $hour:$minute:$second");
 local $flags = &get_hwclock_flags();
-local $out = &backquote_logged("hwclock $flags $format");
+local $out = &backquote_logged("hwclock $flags $format 2>&1");
 return $? ? $out : undef;
 }
 
@@ -226,7 +188,7 @@ elsif ($config{'seconds'} == 1) {
 else {
 	$format = $month.$date.$hour.$minute.substr($year, -2);
 	}
-local $out = &backquote_logged("echo yes | date $format");
+local $out = &backquote_logged("echo yes | date ".quotemeta($format)." 2>&1");
 if ($gconfig{'os_type'} eq 'freebsd' || $gconfig{'os_type'} eq 'netbsd') {
 	return int($?/256) == 1 ? $out : undef;
 	}
@@ -261,5 +223,21 @@ sub number_to_weekday
 return ucfirst($weekday_names[$_[0]]);
 }
 
+# Returns 1 if this system supports setting the hardware clock.
+sub support_hwtime
+{
+if ($config{'hwtime'} == 1) {
+	return 1;
+	}
+elsif ($config{'hwtime'} == 0) {
+	return 0;
+	}
+else {
+	return &has_command("hwclock") &&
+	       !&running_in_xen() && !&running_in_vserver() &&
+	       !&running_in_openvz() && !&running_in_zone();
+	}
+}
+
 1;
 
