@@ -7,7 +7,7 @@ do 'webmin-lib.pl';
 # Converts logged information from this module into human-readable form
 sub parse_webmin_log
 {
-local ($user, $script, $action, $type, $object, $p) = @_;
+my ($user, $script, $action, $type, $object, $p) = @_;
 if ($action eq 'install') {
 	return &text('log_install', "<i>$p->{'desc'}</i>");
 	}
@@ -28,6 +28,9 @@ elsif ($action eq 'theme') {
 	return $p->{'theme'} ? &text('log_theme', "<tt>$p->{'theme'}</tt>")
 			     : $text{'log_theme_def'};
 	}
+elsif ($action eq 'deletecache') {
+	return &text('log_deletecache', $object);
+	}
 elsif ($text{"log_$action"}) {
 	return $text{"log_$action"};
 	}
